/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.modfact.corba.helper;

import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.TypeCodePackage.Bounds;
import org.omg.mof.Model.Classifier;
import org.omg.mof.Model.ModelElement;
import org.omg.mof.Model.MofAttribute;
import org.omg.mof.Model.Operation;
import org.omg.mof.Model.Reference;
import org.omg.mof.Reflective.MofError;
import org.omg.mof.Reflective.NotSet;

/**
 * @author Xavier Blanc
 */
public interface JavaCommon extends IDLCommon {

	/**
 	 * Return the string prefixed by a '_' if a Java conflict exists 
 	 */
	public String testJavaConflict(String _input);
		
	/**
	 * Return the format1 of string but preserving the naming conflicts
	 */
	public String format1Idl2JavaConflict(String _input);
	
	/**
	 * Return the format2 of string but preserving the naming conflicts
	 */
	public String format2Idl2JavaConflict(String _input);

	/**
	 * Return the java qualified name of a model element
	 */
	public String javaQualifiedName(ModelElement element)
		throws MofError, NotSet;

	/**
	 * Return the java class name of a class
	 */
	public String javaClassName(org.omg.mof.Model.Class clazz)
		throws NotSet, MofError;

	/**
	 * Return the java classifier name of a classifier
	 */
	public String javaClassifierName(org.omg.mof.Model.Classifier clazz)
		throws NotSet, MofError;

	/**
	 * Return the java type corresponding to the classifier
	 */
	public String javaType(Classifier classifier)
		throws MofError, BadKind, Bounds, NotSet;

	/**
	 * Return the name of the package
	 */
	public String javaPackage(org.omg.mof.Model.Package _package)
		throws MofError;

	/**
	 * Return the string for an attribute
	 */
	public String attributeTemplate(MofAttribute _attribute)
		throws MofError, BadKind, Bounds, NotSet;

	/**
	 * Return the Java string for a reference
	 */
	public String referenceTemplate(Reference _reference)
		throws MofError, NotSet;

	/**
	 * Return the Java string for an operation
	 */
	public String operationTemplate(Operation _operation)
		throws MofError, BadKind, Bounds, NotSet;

	/**
	 * Return the Java string for BaseObject
	 */
	public String refBaseObjectTemplate();

	/**
	 * Return the Java string for Object
	 */
	public String refObjectTemplate();

	/**
	 * Return the Java string for Association
	 */
	public String refAssociationTemplate();

	/**
	 * Return the Java string for package
	 */
	public String refPackageTemplate();

	/**
	 * Invers prefix for Java Impl
	 */
	public String idlPrefix2Java(String prefix);

}
