/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.helper;

/**
 * 
 */
public interface XMIImportCommon {

	/**
	 * Pass from a TypeCode to the corresponding Java type code
	 * @param _type The CORBA Type Code.
	 * @return A String representing the corresponding Java type.
	 */
	public String typeCode2Java(org.omg.CORBA.TypeCode _type);

	/**
	 * Tests if a MOF Object is contained in a vector.
	 * @param _vector The vector to parse.
	 * @param _object The MOF Object.
	 * @return TRUE if the vector contains the MOF Object.
	 */
	public boolean vectorContains(java.util.Vector _vector, org.omg.mof.Reflective.RefBaseObject _object);

	/**
	 * Tests if a MOF Object is contained in a vector.
	 * @param _array The array to parse.
	 * @param _object The MOF Object.
	 * @return TRUE if the vector contains the MOF Object.
	 */
	public boolean arrayContains(org.omg.CORBA.Object[] _array, org.omg.mof.Reflective.RefBaseObject _object);

	/**
		 * Gets the default value of an Attribute.
		 * @param _type The CORBA Type Code of the attribute.
		 * @return The default value of the attribute.
		 */
	public String defaultAttributeValue(org.omg.CORBA.TypeCode _type) throws org.omg.mof.Reflective.MofError;

	/**
	 * Gets the default value of an Attribute.
	 * @param _type The CORBA Type Code of the attribute.
	 * @param _classifier The Object type of the attribute.
	 * @return The default value of the attribute.
	 */
	public String defaultAttributeValue(org.omg.CORBA.TypeCode _type, org.omg.mof.Model.Classifier _classifier)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Gets the default value of an Attribute.
	 * @param _classifier The Object type of the attribute.
	 * @return The default value of the attribute.
	 */
	public String defaultAttributeValue(org.omg.mof.Model.Classifier _classifier) throws org.omg.mof.Reflective.MofError;
	

	/**
	 * Gets the references of an association.
	 * @param _association The association.
	 * @return The references.
	 */
	public org.omg.mof.Model.Reference[] getReferences(org.omg.mof.Model.Association _association)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet;
		
		
	/**
	 * Tests if the DataType Name represents a PrimitiveType.
	 * @param datatype_name The name of the datatype.
	 */
	public boolean isPrimitiveType(String datatype_name);
	
	
	/** 
	 * Add a value to a hashtable composed by an object as key (a model) and a vector as value (models keys).
	 * @param hashtable The hashtable.
	 * @param key The key in the hastable.
	 * @param object The referenced object to add to the vector corresponding to the key of the hashtable.
	 */
	public void addHashtableValue(java.util.Hashtable hashtable, Object key, Object object);
	

}
