/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.idl2java;


import org.omg.mof.Model.PackageHelper;
import org.objectweb.modfact.corba.generator.BracketGenerator;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.logging.ModFactLogger;

public class AssociationImplementationGenerator extends BracketGenerator {

	ModFactLogger logger = null;

	org.omg.mof.Model.Association[] input;

	/**
	 * Set Input
	 */
	public void setInput(org.omg.mof.Model.ModelElement[] elt) {
		input = new org.omg.mof.Model.Association[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (org.omg.mof.Model.Association) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	//Helper
	JavaCommon javaHelper;
	MOFCommon mofHelper;

	public  void generate()
		throws org.omg.mof.Reflective.NotSet, org.omg.mof.Reflective.MofError {

		//logger.log(Level.FINE, "Association Implementation startGeneration");
		outputln("// le Code gnr de l'association: " + input[0].name());
		associationImpl(input[0]);
		flushFile();
	}

	public  void associationImpl(
		org.omg.mof.Model.Association _association)
		throws org.omg.mof.Reflective.NotSet, org.omg.mof.Reflective.MofError {

		//les formats du nom de l'association
		String _nformat1 = javaHelper.format1Idl2JavaConflict(_association.name());
		String _aformat1 = javaHelper.javaQualifiedName(_association);
		String _aformat2 = javaHelper.format2(_association.name());

		//Dcalration du Package
		output("package ");

		// Affichage du nom du package
		outputln(javaHelper.javaPackage(PackageHelper.narrow(_association.container())) + " ;");

		outputln(
			"public class "
				+ _nformat1
				+ "Impl extends "
				+ _aformat1
				+ "POA {");
		outputln(
				javaHelper.format1Idl2JavaConflict(_association.container().name()+"Package")
				+ " _model_package;");

		//Affichage de RefBaseObject et RefAssociation
		output(javaHelper.refBaseObjectTemplate());
		outputln();
		output(javaHelper.refAssociationTemplate());

		//Code propre  l'association
		org.omg.mof.Model.AssociationEnd[] _ends =
			mofHelper.associationEndsOfAssociation(_association);
		org.omg.mof.Model.AssociationEnd _source = _ends[0];
		org.omg.mof.Model.AssociationEnd _dest = _ends[1];

		//Associationd_end1
		String _sourceformat1 = javaHelper.javaQualifiedName(_source.type());
		String _sourceformat2 = javaHelper.format2(_source.name());
		String _sourceformat2JavaConflict = javaHelper.format2Idl2JavaConflict(_source.name());
		boolean _source_navig = _source.is_navigable();
		boolean _source_chang = _source.is_changeable();
		org.omg.mof.Model.MultiplicityType _msource = _source.multiplicity();
		boolean _source_ordered = _msource.isOrdered;
		boolean _source_unique = _msource.isUnique;
		int _source_upper = _msource.upper;
		//int _source_lower = _msource.lower ;

		//Association_end2
		String _destformat1 = javaHelper.javaQualifiedName(_dest.type());
		String _destformat2 = javaHelper.format2(_dest.name());
		String _destformat2JavaConflict = javaHelper.format2Idl2JavaConflict(_dest.name());
		boolean _dest_navig = _dest.is_navigable();
		boolean _dest_chang = _dest.is_changeable();
		org.omg.mof.Model.MultiplicityType _mdest = _dest.multiplicity();
		boolean _dest_ordered = _mdest.isOrdered;
		boolean _dest_unique = _mdest.isUnique;
		int _dest_upper = _mdest.upper;
		//int _dest_lower = _mdest.lower ;

		outputln(
			"private java.util.Vector _all_"
				+ _aformat2
				+ " = new java.util.Vector();");

		outputln("// Operation all_is_of_type_links");
		outputln(
			"public "
				+ _aformat1
				+ "Link[] all_"
				+ _aformat2
				+ "_Links() throws org.omg.mof.Reflective.MofError {");
		outputln(
			_aformat1
				+ "Link[] temp1 = new "
				+ _aformat1
				+ "Link [_all_"
				+ _aformat2
				+ ".size()];");
		outputln(
			"for (int i=0; i< _all_"
				+ _aformat2
				+ ".size();i++) temp1[i]= ("
				+ _aformat1
				+ "Link)_all_"
				+ _aformat2
				+ ".elementAt(i);");
		outputln("return temp1;");
		outputln("}");

		outputln("//Operation exists");
		outputln(
			"public boolean exists("
				+ _sourceformat1
				+ " "
				+ _sourceformat2JavaConflict
				+ ", "
				+ _destformat1
				+ " "
				+ _destformat2JavaConflict
				+ ") throws org.omg.mof.Reflective.MofError {");
		outputln("boolean exist = false;");
		outputln("int i= 0;");
		outputln("while((i< _all_" + _aformat2 + ".size())&& !exist){");
		outputln(
			_aformat1
				+ "Link temp1 = ("
				+ _aformat1
				+ "Link)_all_"
				+ _aformat2
				+ ".elementAt(i);");
		outputln(
			"if (temp1."
				+ _sourceformat2JavaConflict
				+ "._is_equivalent("
				+ _sourceformat2JavaConflict
				+ ") && temp1."
				+ _destformat2JavaConflict
				+ "._is_equivalent("
				+ _destformat2JavaConflict
				+ ")) exist = true;");
		outputln("i++;");
		outputln("}");
		outputln("return exist ;");
		outputln("} ");

		if (_source_navig) {
			outputln("// Operation " + _sourceformat2);
			output("public " + _sourceformat1);
			if ((_source_upper > 1) || (_source_upper == -1)) {
				outputln(
					"[] "
						+ _sourceformat2JavaConflict
						+ " ("
						+ _destformat1
						+ " "
						+ _destformat2JavaConflict
						+ ") throws org.omg.mof.Reflective.MofError {");
				outputln(
					"java.util.Vector temp1 = new java.util.Vector();");
				outputln(
					"for (int i=0; i< _all_"
						+ _aformat2
						+ ".size(); i++){");
				outputln(
					_aformat1
						+ "Link temp2 = ("
						+ _aformat1
						+ "Link)_all_"
						+ _aformat2
						+ ".elementAt(i);");
				outputln(
					"if (temp2."
						+ _destformat2JavaConflict
						+ "._is_equivalent("
						+ _destformat2JavaConflict
						+ ")) temp1.addElement(temp2."
						+ _sourceformat2JavaConflict
						+ ");");
				outputln("}");
				outputln(
					_sourceformat1
						+ "[] temp3 = new "
						+ _sourceformat1
						+ "[temp1.size()];");
				outputln(
					"for (int i=0; i< temp1.size();i++) temp3[i]= ("
						+ _sourceformat1
						+ ")temp1.elementAt(i);");
				outputln("return temp3;");
				outputln("}");
			} else {
				outputln(
					" "
						+ _sourceformat2JavaConflict
						+ "("
						+ _destformat1
						+ " "
						+ _destformat2JavaConflict
						+ ") throws org.omg.mof.Reflective.MofError {");
				outputln("boolean exist = false;");
				outputln("int i= 0;");
				outputln(_aformat1 + "Link temp1 = null;");
				outputln(
					"while((i< _all_" + _aformat2 + ".size())&& !exist){");
				outputln(
					"temp1= ("
						+ _aformat1
						+ "Link)_all_"
						+ _aformat2
						+ ".elementAt(i);");
				outputln(
					"if (temp1."
						+ _destformat2JavaConflict
						+ "._is_equivalent("
						+ _destformat2JavaConflict
						+ ")) exist = true;");
				outputln("i++;");
				outputln("}");
				outputln("return temp1." + _sourceformat2JavaConflict + ";");
				outputln("}");
			}
		}

		if (_dest_navig) {
			outputln("// Operation " + _destformat2);
			output("public " + _destformat1);
			if ((_dest_upper > 1) || (_dest_upper == -1)) {
				outputln(
					"[] "
						+ _destformat2JavaConflict
						+ "("
						+ _sourceformat1
						+ " "
						+ _sourceformat2JavaConflict
						+ ") throws org.omg.mof.Reflective.MofError {");
				outputln(
					"java.util.Vector temp1 = new java.util.Vector();");
				outputln(
					"for (int i=0; i< _all_"
						+ _aformat2
						+ ".size(); i++){");
				outputln(
					_aformat1
						+ "Link temp2= ("
						+ _aformat1
						+ "Link)_all_"
						+ _aformat2
						+ ".elementAt(i);");
				outputln(
					"if (temp2."
						+ _sourceformat2JavaConflict
						+ "._is_equivalent("
						+ _sourceformat2JavaConflict
						+ ")) temp1.addElement(temp2."
						+ _sourceformat2JavaConflict
						+ ");");
				outputln("}");
				outputln(
					_destformat1
						+ "[] temp3 = new "
						+ _destformat1
						+ "[temp1.size()];");
				outputln(
					"for (int i=0; i< temp1.size();i++) temp3[i]= ("
						+ _destformat1
						+ ")temp1.elementAt(i);");
				outputln("return temp3;");
				outputln("}");
			} else {
				outputln(
					" "
						+ _destformat2JavaConflict
						+ "("
						+ _sourceformat1
						+ " "
						+ _sourceformat2JavaConflict
						+ ") throws org.omg.mof.Reflective.MofError {");
				outputln("boolean exist = false;");
				outputln("int i= 0;");
				outputln(_aformat1 + "Link temp1 = null;");
				outputln(
					"while((i< _all_" + _aformat2 + ".size())&& !exist){");
				outputln(
					"temp1= ("
						+ _aformat1
						+ "Link)_all_"
						+ _aformat2
						+ ".elementAt(i);");
				outputln(
					"if (temp1."
						+ _sourceformat2JavaConflict
						+ "._is_equivalent("
						+ _sourceformat2JavaConflict
						+ ")) exist = true;");
				outputln("i++;");
				outputln("}");
				outputln("return temp1." + _destformat2JavaConflict + ";");
				outputln("}");
			}

		}

		if (_source_chang && _dest_chang) {
			outputln("// Operation add");
			outputln(
				"public void add("
					+ _sourceformat1
					+ " "
					+ _sourceformat2JavaConflict
					+ ", "
					+ _destformat1
					+ " "
					+ _destformat2JavaConflict
					+ ") throws org.omg.mof.Reflective.MofError {");
			outputln(
				_aformat1
					+ "Link temp = new "
					+ _aformat1
					+ "Link("
					+ _sourceformat2JavaConflict
					+ ", "
					+ _destformat2JavaConflict
					+ ");");
			outputln("_all_" + _aformat2 + ".addElement(temp);");
			outputln("}");
		}

		if (_source_chang
			&& _source_navig
			&& _dest_chang
			&& ((_source_upper > 1) || (_source_upper == -1))
			&& _source_ordered) {
			outputln("// Operation add_before" + _sourceformat2);
			outputln(
				"public void add_before_"
					+ _sourceformat2
					+ "("
					+ _sourceformat1
					+ "  "
					+ _sourceformat2JavaConflict
					+ ",  "
					+ _destformat1
					+ "  "
					+ _destformat2JavaConflict
					+ ", "
					+ _sourceformat1
					+ " before) throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{ ");
			outputln("boolean exist = false;");
			outputln("int i= 0;");
			outputln(
				"while((i< _all_" + _aformat2 + ".size())&& !exist){");
			outputln(
				_aformat1
					+ "Link temp1= ("
					+ _aformat1
					+ "Link)_all_"
					+ _aformat2
					+ ".elementAt(i);");
			outputln(
				"if (temp1."
					+ _sourceformat2JavaConflict
					+ "._is_equivalent(before)) exist = true;");
			outputln("i++;");
			outputln("}");
			outputln("if(exist) {");
			outputln(
				_aformat1
					+ "Link temp = new "
					+ _aformat1
					+ "Link("
					+ _sourceformat2JavaConflict
					+ ", "
					+ _destformat2JavaConflict
					+ ");");
			outputln(
				"_all_" + _aformat2 + ".insertElementAt(temp, i-1);");
			outputln("}");
			outputln(
				"else throw new org.omg.mof.Reflective.NotFound() ;");
			outputln("}");
		}

		if (_dest_chang
			&& _dest_navig
			&& _source_chang
			&& ((_dest_upper > 1) || (_dest_upper == -1))
			&& _dest_ordered) {
			outputln("// Operation add_before" + _destformat2);
			outputln(
				"public void add_before_"
					+ _destformat2
					+ "("
					+ _sourceformat1
					+ "  "
					+ _sourceformat2JavaConflict
					+ ",  "
					+ _destformat1
					+ "  "
					+ _destformat2JavaConflict
					+ ", "
					+ _destformat1
					+ " before) throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{ ");
			outputln("boolean exist = false;");
			outputln("int i= 0;");
			outputln(
				"while((i< _all_" + _aformat2 + ".size())&& !exist){");
			outputln(
				_aformat1
					+ "Link temp1= ("
					+ _aformat1
					+ "Link)_all_"
					+ _aformat2
					+ ".elementAt(i);");
			outputln(
				"if (temp1."
					+ _destformat2
					+ "._is_equivalent(before)) exist = true;");
			outputln("i++;");
			outputln("}");
			outputln("if(exist) {");
			outputln(
				_aformat1
					+ "Link temp = new "
					+ _aformat1
					+ "Link("
					+ _sourceformat2JavaConflict
					+ ", "
					+ _destformat2JavaConflict
					+ ");");
			outputln(
				"_all_" + _aformat2 + ".insertElementAt(temp, i-1);");
			outputln("}");
			outputln(
				"else throw new org.omg.mof.Reflective.NotFound() ;");
			outputln("}");
		}

		if (_source_navig && _source_chang) {
			outputln("// Operation modify" + _sourceformat2);
			outputln(
				"public void modify_"
					+ _sourceformat2
					+ "("
					+ _sourceformat1
					+ "  "
					+ _sourceformat2JavaConflict
					+ ", "
					+ _destformat1
					+ "  "
					+ _destformat2JavaConflict
					+ ", "
					+ _sourceformat1
					+ " new_"
					+ _sourceformat2
					+ ") throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{ ");
			outputln("boolean exist = false;");
			outputln("int i= 0;");
			outputln(
				"while((i< _all_" + _aformat2 + ".size())&& !exist){");
			outputln(
				_aformat1
					+ "Link temp1= ("
					+ _aformat1
					+ "Link)_all_"
					+ _aformat2
					+ ".elementAt(i);");
			outputln(
				"if (temp1."
					+ _sourceformat2JavaConflict
					+ "._is_equivalent("
					+ _sourceformat2JavaConflict
					+ ") && temp1."
					+ _destformat2JavaConflict
					+ "._is_equivalent("
					+ _destformat2JavaConflict
					+ ")) exist = true;");
			outputln("i++;");
			outputln("}");
			outputln("if(exist) {");
			outputln(
				_aformat1
					+ "Link temp = new "
					+ _aformat1
					+ "Link(new_"
					+ _sourceformat2
					+ ", "
					+ _destformat2JavaConflict
					+ ");");
			outputln("_all_" + _aformat2 + ".setElementAt(temp, i-1);");
			outputln("}");
			outputln(
				"else throw new org.omg.mof.Reflective.NotFound() ;");
			outputln("}");
		}

		if (_dest_navig && _dest_chang) {
			outputln("// Operation modify" + _sourceformat2);
			outputln(
				"public void modify_"
					+ _destformat2
					+ " ("
					+ _sourceformat1
					+ "  "
					+ _sourceformat2JavaConflict
					+ ", "
					+ _destformat1
					+ "  "
					+ _destformat2JavaConflict
					+ ", "
					+ _destformat1
					+ " new_"
					+ _destformat2
					+ ") throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{ ");
			outputln("boolean exist = false;");
			outputln("int i= 0;");
			outputln(
				"while((i< _all_" + _aformat2 + ".size())&& !exist){");
			outputln(
				_aformat1
					+ "Link temp1= ("
					+ _aformat1
					+ "Link)_all_"
					+ _aformat2
					+ ".elementAt(i);");
			outputln(
				"if (temp1."
					+ _sourceformat2JavaConflict
					+ "._is_equivalent("
					+ _sourceformat2JavaConflict
					+ ") && temp1."
					+ _destformat2JavaConflict
					+ "._is_equivalent("
					+ _destformat2JavaConflict
					+ ")) exist = true;");
			outputln("i++;");
			outputln("}");
			outputln("if(exist) {");
			outputln(
				_aformat1
					+ "Link temp = new "
					+ _aformat1
					+ "Link("
					+ _sourceformat2JavaConflict
					+ ", new_"
					+ _destformat2
					+ ");");
			outputln("_all_" + _aformat2 + ".setElementAt(temp, i-1);");
			outputln("}");
			outputln(
				"else throw new org.omg.mof.Reflective.NotFound() ;");
			outputln("}");
		}

		if (_source_chang && _dest_chang) {
			outputln(
				"public void remove ("
					+ _sourceformat1
					+ "  "
					+ _sourceformat2JavaConflict
					+ ", "
					+ _destformat1
					+ "  "
					+ _destformat2JavaConflict
					+ ") throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{ ");
			outputln("boolean exist = false;");
			outputln("int i= 0;");
			outputln(
				"while((i< _all_" + _aformat2 + ".size())&& !exist){");
			outputln(
				_aformat1
					+ "Link temp1= ("
					+ _aformat1
					+ "Link)_all_"
					+ _aformat2
					+ ".elementAt(i);");
			outputln(
				"if (temp1."
					+ _sourceformat2JavaConflict
					+ "._is_equivalent("
					+ _sourceformat2JavaConflict
					+ ") && temp1."
					+ _destformat2JavaConflict
					+ "._is_equivalent("
					+ _destformat2JavaConflict
					+ ")) exist = true;");
			outputln("i++;");
			outputln("}");
			outputln("if(exist) {");
			outputln("_all_" + _aformat2 + ".removeElementAt(i-1);");
			outputln("}");
			outputln(
				"else throw new org.omg.mof.Reflective.NotFound() ;");
			outputln("}");
		}
		outputln("}");
	}
	/**
	 * Returns the javaHelper.
	 * @return JavaCommon
	 */
	public JavaCommon getJavaHelper() {
		return javaHelper;
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Sets the javaHelper.
	 * @param javaHelper The javaHelper to set
	 */
	public void setJavaHelper(JavaCommon javaHelper) {
		this.javaHelper = javaHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

}