/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.idl2java;

import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.TypeCodePackage.Bounds;
import org.omg.mof.Model.PackageHelper;
import org.omg.mof.Reflective.NotSet;

import org.objectweb.modfact.corba.generator.BracketGenerator;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.logging.ModFactLogger;

public class ClassImplementationGenerator extends BracketGenerator {

	ModFactLogger logger = null;

	org.omg.mof.Model.Class[] input;

	/**
	 * Set Input
	 */
	public void setInput(org.omg.mof.Model.ModelElement[] elt){
		input = new org.omg.mof.Model.Class[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (org.omg.mof.Model.Class) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	//Helper
	JavaCommon javaHelper;
	MOFCommon mofHelper;

	public void generate()
		throws
			org.omg.mof.Reflective.NotSet,
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds {
		outputln("// le Code gnr de la Class: " + input[0].name());
		classImpl(input[0]);
		flushFile();
	}

	public void constructeurClass(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet , BadKind , Bounds {

		String _nformat1 = javaHelper.format1Idl2JavaConflict(javaHelper.javaClassName(_class));
		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));

		if (_class
			.visibility()
			.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
			output("public " + _nformat1 + "Impl(");

			// trouver toutes les attributs classifier_level de cette classe
			org.omg.mof.Model.MofAttribute[] attributesND =
				mofHelper.allAttributesOfClass(
					_class,
					org.omg.mof.Model.ScopeKind.instance_level,
					false);

			for (int i = 0; i < attributesND.length; i++) {
				String _type_format1 = javaHelper.javaType(attributesND[i].type());
				String _aformat2 = javaHelper.format2(attributesND[i].name());
				if ((attributesND[i].multiplicity().upper > 1)
					|| (attributesND[i].multiplicity().upper == -1))
					output(_type_format1 + "[]  " + _aformat2 + "s");
				else
					output(_type_format1 + " " + _aformat2);
				output(", ");
			}
			outputln(_cformat1 + "Class _proxy){");
			for (int i = 0; i < attributesND.length; i++) {
				String _aformat2 = javaHelper.format2(attributesND[i].name());
				int _lower = attributesND[i].multiplicity().lower;
				int _upper = attributesND[i].multiplicity().upper;
				if ((_upper > 1) || (_upper == -1)) {
					output(
						"for(int i=0;i<" + _aformat2 + "s.length; i++) ");
					output(
						" _"
							+ _aformat2
							+ "s.addElement("
							+ _aformat2
							+ "s[i]);");
				} else {
					outputln("_" + _aformat2 + " = " + _aformat2 + " ;");
					if ((_lower == 0) && (_upper == 1))
						outputln("_exist_" + _aformat2 + " = true;");
				}
			}
			outputln("_class_proxy = _proxy ;");
			outputln("  }");
			outputln("}");

		}
	}

	public void allOfType(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError , NotSet{
		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));
		outputln(
			"public " + _cformat1 + "[] all_of_type_" + _cformat2 + "() {");
		outputln("return _class_proxy.all_of_type_" + _cformat2 + "();");
		outputln("}");
	}

	public void allOfClasses(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError , NotSet{
		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));
		outputln(
			"public " + _cformat1 + "[] all_of_class_" + _cformat2 + "() {");
		outputln(
			"return _class_proxy.all_of_class_" + _cformat2 + "();");
		outputln("}");
	}

	public void createClass(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError , BadKind , Bounds , NotSet{
		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));
		if (_class
			.visibility()
			.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
			output("public " + _cformat1 + " create_" + _cformat2 + "(");
			org.omg.mof.Model.MofAttribute[] attributesND =
				mofHelper.allAttributesOfClass(
					_class,
					org.omg.mof.Model.ScopeKind.instance_level,
					false);
			for (int i = 0; i < attributesND.length; i++) {
				String _type_format1 = javaHelper.javaType(attributesND[i].type());
				String _aformat2 = javaHelper.format2(attributesND[i].name());
				if ((attributesND[i].multiplicity().upper > 1)
					|| (attributesND[i].multiplicity().upper == -1))
					output(_type_format1 + "[]  " + _aformat2);
				else
					output(_type_format1 + " " + _aformat2);
				if (i != (attributesND.length - 1))
					output(", ");
			}
			outputln(") throws org.omg.mof.Reflective.MofError {");
			output("return _class_proxy.create_" + _cformat2 + "(");
			for (int i = 0; i < attributesND.length; i++) {
				String _aformat2 = javaHelper.format2(attributesND[i].name());
				output(_aformat2);
				if (i != (attributesND.length - 1))
					output(", ");

			}
			outputln(");");
			outputln("}");
		}
	}

	public void classImpl(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.NotSet, org.omg.mof.Reflective.MofError , BadKind , Bounds{

		//les formats du nom de la classe
		String _nformat1 = javaHelper.format1Idl2JavaConflict(javaHelper.javaClassName(_class));
		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));

		//Dcalration du Package
		output("package ");

		// Affichage du nom  du package
		outputln(javaHelper.javaPackage(PackageHelper.narrow(_class.container())) + " ;");

		outputln(
			"public class "
				+ _nformat1
				+ "Impl extends "
				+ _cformat1
				+ "POA {");
		outputln();

		//genrer du code pour RefBaseObject et RefObject
		output(javaHelper.refBaseObjectTemplate());
		outputln();
		output(javaHelper.refObjectTemplate());

		//Dclaration de classe proxy
		outputln("private " + _cformat1 + "Class _class_proxy;");

		// Tous les attributs classifier_level
		org.omg.mof.Model.MofAttribute[] attributesC =
			mofHelper.allAttributesOfClass(
				_class,
				org.omg.mof.Model.ScopeKind.classifier_level,
				true);
		for (int i = 0; i < attributesC.length; i++) {
			if (attributesC[i]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _code_attribute =
					javaHelper.attributeTemplate(attributesC[i]);
				outputln(_code_attribute);
			}
		}

		// Toutes les oprations classifier_level
		org.omg.mof.Model.Operation[] operationsC =
			mofHelper.allOperationsOfClass(
				_class,
				org.omg.mof.Model.ScopeKind.classifier_level);
		for (int i = 0; i < operationsC.length; i++) {
			if (operationsC[i]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _code_operation =
					javaHelper.operationTemplate(operationsC[i]);
				outputln(_code_operation);
			}
		}

		// Traitement des CreateClass, all_of_type et all_of_classes de la classe et de ses superclasses
		java.util.Vector _sup_classes = mofHelper.superClassesOrdered(_class);

		//Gnrer le code de la mthode Create et de la mthode all_of_classes
		for (int i = 0; i < _sup_classes.size(); i++) {
			org.omg.mof.Model.Class _current =
				(org.omg.mof.Model.Class) _sup_classes.elementAt(i);
			if (!_current.is_abstract()) {
				createClass(_current);
				allOfClasses(_current);
			}
			allOfType(_current);
		}

		// Tous les attributs Instance_level
		org.omg.mof.Model.MofAttribute[] attributes =
			mofHelper.allAttributesOfClass(
				_class,
				org.omg.mof.Model.ScopeKind.instance_level,
				true);
		for (int i = 0; i < attributes.length; i++) {
			if (attributes[i]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _code_attribute =
					javaHelper.attributeTemplate(attributes[i]);
				outputln(_code_attribute);
			}
		}

		// Toutes les oprations Instance_level
		org.omg.mof.Model.Operation[] operations =
			mofHelper.allOperationsOfClass(
				_class,
				org.omg.mof.Model.ScopeKind.instance_level);
		for (int i = 0; i < operations.length; i++) {
			if (operations[i]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _code_operation =
					javaHelper.operationTemplate(operations[i]);
				outputln(_code_operation);
			}
		}

		// Toutes les rfrences Instance et Classifier level
		org.omg.mof.Model.Reference[] references =
			mofHelper.allReferencesOfClass(_class);
		for (int i = 0; i < references.length; i++) {
			if (references[i]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _code_reference =
					javaHelper.referenceTemplate(references[i]);
				outputln(_code_reference);
			}
		}
		constructeurClass(_class);
	}

	/**
	 * Returns the javaHelper.
	 * @return JavaCommon
	 */
	public JavaCommon getJavaHelper() {
		return javaHelper;
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Sets the javaHelper.
	 * @param javaHelper The javaHelper to set
	 */
	public void setJavaHelper(JavaCommon javaHelper) {
		this.javaHelper = javaHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

}