/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.idl2java;

import org.objectweb.modfact.corba.generator.BracketGenerator;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.logging.ModFactLogger;

import java.util.Vector;

import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.TypeCodePackage.Bounds;
import org.omg.mof.Model.Import;
import org.omg.mof.Model.Namespace;
import org.omg.mof.Model.PackageHelper;
import org.omg.mof.Model.ScopeKind;
import org.omg.mof.Model.VisibilityKind;

public class PackageImplementationGenerator extends BracketGenerator {

	ModFactLogger logger = null;

	org.omg.mof.Model.Package[] input;

	/**
	 * Set Input
	 */
	public void setInput(org.omg.mof.Model.ModelElement[] elt) {
		input = new org.omg.mof.Model.Package[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (org.omg.mof.Model.Package) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	//Helper
	JavaCommon javaHelper;
	MOFCommon mofHelper;

	//Log facility
	// Logger logger = Logger.getLogger("fr.lip6.src.mdafactory.generation.idl2javaImpl.PackageImplementation");	

	public void generate()
		throws
			org.omg.mof.Reflective.NotSet,
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds {
		outputln(
			"// le Code gnr du Package: " + input[0].name() + "Package");
		packageImpl(input[0]);
		flushFile();
	}

	public void constructeurPackage(org.omg.mof.Model.Package _package)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.mof.Reflective.NotSet,
			BadKind,
			Bounds {
		String className =
			javaHelper.format1Idl2JavaConflict(_package.name() + "Package")
				+ "Impl";
		String _nformat1 = javaHelper.format1Idl2JavaConflict(_package.name());
		String _pformat1 = javaHelper.javaQualifiedName(_package);
		String _pformat2 = javaHelper.format2(_package.name());

		Vector _attributes = new Vector();
		org.omg.mof.Model.Class[] classes =
			mofHelper.classesOfPackage(_package);
		for (int i = 0; i < classes.length; i++) {
			org.omg.mof.Model.MofAttribute[] attributes =
				mofHelper.attributesOfClass(
					classes[i],
					ScopeKind.classifier_level,
					true);
			for (int k = 0; k < attributes.length; k++)
				_attributes.addElement(attributes[k]);

		}

		java.util.Vector _attributes_name =
			new java.util.Vector(_attributes.size());
		for (int i = 0; i < _attributes.size(); i++) {
			org.omg.mof.Model.MofAttribute _current =
				(org.omg.mof.Model.MofAttribute) _attributes.elementAt(i);
			String _attributeqname = " ";
			for (int j = 0; j < _current.qualified_name().length; j++) {
				if (j > 0)
					_attributeqname =
						javaHelper.format2(_current.qualified_name()[j])
							+ "_"
							+ _attributeqname;
				else
					_attributeqname =
						javaHelper.format2(_current.qualified_name()[j]);
			}
			_attributes_name.addElement(_attributeqname);
		}

		output("public " + className + "(");
		for (int i = 0; i < _attributes.size(); i++) {
			org.omg.mof.Model.MofAttribute _current =
				(org.omg.mof.Model.MofAttribute) _attributes.elementAt(i);
			String _current_type = javaHelper.javaType(_current.type());
			String _attributeqname = (String) _attributes_name.elementAt(i);

			if (!((i != 0) || (i != (_attributes.size() - 1))))
				output(", ");
			if ((_current.multiplicity().upper > 1)
				|| (_current.multiplicity().upper == -1))
				output(_current_type + "[] " + _attributeqname);
			else
				output(_current_type + " " + _attributeqname);
		}
		outputln(") {");
		for (int i = 0; i < _attributes.size(); i++) {
			org.omg.mof.Model.MofAttribute _current =
				(org.omg.mof.Model.MofAttribute) _attributes.elementAt(i);
			String _aformat2 = javaHelper.format2(_current.name());
			String _attributeqname = (String) _attributes_name.elementAt(i);
			output("_" + _aformat2 + "_val = " + _attributeqname);
			output("; ");

		}
		outputln("} ");
	}

	public void packageImpl(org.omg.mof.Model.Package _package)
		throws
			org.omg.mof.Reflective.NotSet,
			org.omg.mof.Reflective.MofError,
			BadKind,
			Bounds {

		//Dclaration du Package
		output("package ");

		// les deux formats du nom du package
		String className =
			javaHelper.format1Idl2JavaConflict(_package.name() + "Package")
				+ "Impl";
		String superClass =
			javaHelper.format1Idl2JavaConflict(_package.name() + "Package")
				+ "POA";
		String _pformat1 = javaHelper.javaQualifiedName(_package);
		String _pformat2 = javaHelper.format2(_package.name());

		// le nom du package
		outputln(javaHelper.javaPackage(_package) + " ;");

		outputln(
			"public class " + className + " extends " + superClass + " {");

		// le code du RefBaseObject et du RefPackageObject
		output(javaHelper.refBaseObjectTemplate());
		outputln();
		output(javaHelper.refPackageTemplate());

		// Cration de classe
		org.omg.mof.Model.Class classes[] =
			mofHelper.classesOfPackage(_package);

		for (int k = 0; k < classes.length; k++) {
			if (classes[k]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _cformat1 = javaHelper.javaQualifiedName(classes[k]);
				String _cformat2 = javaHelper.format2(javaHelper.javaClassName(classes[k]));

				//Dclaration des attributs statiques
				org.omg.mof.Model.MofAttribute[] attributes =
					mofHelper.allAttributesOfClass(
						classes[k],
						org.omg.mof.Model.ScopeKind.classifier_level,
						true);
				for (int i = 0; i < attributes.length; i++) {
					String _type_format1 =
						javaHelper.javaType(attributes[i].type());
					String _aformat2 = javaHelper.format2(attributes[i].name());
					if ((attributes[i].multiplicity().upper > 1)
						|| (attributes[i].multiplicity().upper == -1))
						output(
							_type_format1
								+ "[]  _"
								+ _aformat2
								+ "_val ;");
					else
						output(
							_type_format1 + " _" + _aformat2 + "_val ;");
				}

				outputln(
					"private "
						+ _cformat1
						+ "Class  _"
						+ _cformat2
						+ "_ref = null ;");
				outputln(
					"public " + _cformat1 + "Class " + _cformat2 + "_ref(){");
				outputln(" if( _" + _cformat2 + "_ref == null ) {");
				output(
					_cformat1
						+ "ClassImpl  _"
						+ _cformat2
						+ "_temp = new "
						+ _cformat1
						+ "ClassImpl(");
				for (int i = 0; i < attributes.length; i++) {
					String _aformat2 = javaHelper.format2(attributes[i].name());
					output("_" + _aformat2 + "_val");
					output(", ");
				}
				outputln("_this());");
				outputln("try {");
				outputln(
					"_poa().activate_object(_" + _cformat2 + "_temp);");
				outputln(
					"org.omg.CORBA.Object obj = _poa().servant_to_reference(_"
						+ _cformat2
						+ "_temp);");
				outputln(
					"_"
						+ _cformat2
						+ "_ref = "
						+ _cformat1
						+ "ClassHelper.narrow(obj);");
				outputln("}");
				outputln("catch (java.lang.Exception ex) {");
				outputln("ex.printStackTrace();");
				outputln("System.exit(0);");
				outputln("}");
				outputln("}");
				outputln("return _" + _cformat2 + "_ref;");
				outputln("}");
			}

		}

		//Association
		org.omg.mof.Model.Association[] associations =
			mofHelper.associationsOfPackage(_package);

		for (int k = 0; k < associations.length; k++) {
			if (associations[k]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _aformat1 =
					javaHelper.javaQualifiedName(associations[k]);
				String _aformat2 = javaHelper.format2(associations[k].name());
				outputln(
					"private "
						+ _aformat1
						+ "  _"
						+ _aformat2
						+ "_ref = null ;");
				outputln(
					"public " + _aformat1 + " " + _aformat2 + "_ref(){");
				outputln(" if( _" + _aformat2 + "_ref == null ) {");
				outputln(
					_aformat1
						+ "Impl _"
						+ _aformat2
						+ "_temp = new "
						+ _aformat1
						+ "Impl();");
				outputln("try {");
				outputln(
					"_poa().activate_object(_" + _aformat2 + "_temp);");
				outputln(
					"org.omg.CORBA.Object obj = _poa().servant_to_reference(_"
						+ _aformat2
						+ "_temp);");
				outputln(
					"_"
						+ _aformat2
						+ "_ref = "
						+ _aformat1
						+ "Helper.narrow(obj);");
				outputln("}");
				outputln("catch (java.lang.Exception ex){");
				outputln("ex.printStackTrace();");
				outputln("System.exit(0);");
				outputln("}");
				outputln("}");
				outputln("return _" + _aformat2 + "_ref ;");
				outputln("}");
			}
		}

		//nested packages

		org.omg.mof.Model.Import[] imports =
			mofHelper.importsOfPackage(_package);
		for (int i = 0; i < imports.length; i++) {
			Import _import = imports[i];
			Namespace imported = _import.imported_namespace();
			if (imported._is_a(PackageHelper.id())) {
				org.omg.mof.Model.Package cluteredPackage =
					PackageHelper.narrow(imported);
				if ((_import.visibility().equals(VisibilityKind.public_vis))
					&& (_import.is_clustered())
					&& (cluteredPackage
						.visibility()
						.equals(VisibilityKind.public_vis))) {

					Namespace importedNamespace =
						imports[i].imported_namespace();
					if (importedNamespace._is_a(PackageHelper.id())) {
						org.omg.mof.Model.Package importedPackage =
							PackageHelper.narrow(importedNamespace);
						String _aformat1 = 
							javaHelper.javaPackage(importedPackage)+"."+javaHelper.format1Idl2JavaConflict(importedPackage.name() + "Package");
						String _aformat2 = javaHelper.format2(importedPackage.name());
						outputln(
							"private "
								+ _aformat1
								+ "  _"
								+ _aformat2
								+ "_ref = null ;");
						outputln(
							"public "
								+ _aformat1
								+ " "
								+ _aformat2
								+ "_ref(){");
						outputln(
							" if( _" + _aformat2 + "_ref == null ) {");
						outputln(
							_aformat1
								+ "Impl _"
								+ _aformat2
								+ "_temp = new "
								+ _aformat1
								+ "Impl();");
						outputln("try {");
						outputln(
							"_poa().activate_object(_"
								+ _aformat2
								+ "_temp);");
						outputln(
							"org.omg.CORBA.Object obj = _poa().servant_to_reference(_"
								+ _aformat2
								+ "_temp);");
						outputln(
							"_"
								+ _aformat2
								+ "_ref = "
								+ _aformat1
								+ "Helper.narrow(obj);");
						outputln("}");
						outputln("catch (java.lang.Exception ex){");
						outputln("ex.printStackTrace();");
						outputln("System.exit(0);");
						outputln("}");
						outputln("}");
						outputln("return _" + _aformat2 + "_ref ;");
						outputln("}");
					}
				}
			}
		}

		org.omg.mof.Model.Package[] packages =
			mofHelper.packagesOfPackage(_package);
		for (int i = 0; i < packages.length; i++) {
			org.omg.mof.Model.Package nestedPackage = packages[i];
			if (nestedPackage.visibility().equals(VisibilityKind.public_vis)) {
				String _aformat1 = javaHelper.javaQualifiedName(packages[i])+"."+javaHelper.format1Idl2JavaConflict(packages[i].name() + "Package");
				String _aformat2 = javaHelper.format2(packages[i].name());
				outputln(
					"private "
						+ _aformat1
						+ "  _"
						+ _aformat2
						+ "_ref = null ;");
				outputln(
					"public " + _aformat1 + " " + _aformat2 + "_ref(){");
				outputln(" if( _" + _aformat2 + "_ref == null ) {");
				outputln(
					_aformat1
						+ "Impl _"
						+ _aformat2
						+ "_temp = new "
						+ _aformat1
						+ "Impl();");
				outputln("try {");
				outputln(
					"_poa().activate_object(_" + _aformat2 + "_temp);");
				outputln(
					"org.omg.CORBA.Object obj = _poa().servant_to_reference(_"
						+ _aformat2
						+ "_temp);");
				outputln(
					"_"
						+ _aformat2
						+ "_ref = "
						+ _aformat1
						+ "Helper.narrow(obj);");
				outputln("}");
				outputln("catch (java.lang.Exception ex){");
				outputln("ex.printStackTrace();");
				outputln("System.exit(0);");
				outputln("}");
				outputln("}");
				outputln("return _" + _aformat2 + "_ref ;");
				outputln("}");

			}
		}

		constructeurPackage(_package);
		outputln("}");
	}
	/**
	 * Returns the javaHelper.
	 * @return JavaCommon
	 */
	public JavaCommon getJavaHelper() {
		return javaHelper;
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Sets the javaHelper.
	 * @param javaHelper The javaHelper to set
	 */
	public void setJavaHelper(JavaCommon javaHelper) {
		this.javaHelper = javaHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

}
