/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.provider;

import java.util.*;

import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.mof.Reflective.MofError;
import org.omg.mof.Reflective.NotSet;

import org.objectweb.modfact.corba.helper.IDLCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;

/**
 * @author Xavier Blanc
 *
 */
public class IDLProvider implements IDLCommon {

	MOFCommon mofHelper;

	public IDLProvider(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

	public IDLProvider() {
	}

	/**
	 * Format 1
	 */
	public String format1(String input) {
		Vector s1 = splitAndDelete(input);
		String buffer = " ";
		for (int i = 0; i < s1.size(); i++) {
			char[] s2 = s1.elementAt(i).toString().toCharArray();
			s2[0] = Character.toUpperCase(s1.elementAt(i).toString().charAt(0));
			String s3 = String.valueOf(s2);
			buffer = buffer.concat(s3);
		}
		return buffer.trim();
	}
	
	/**
	 * Format 1 (with the first letter in lower case)
	 */
    public String format1FirstMin(String _input) {
	    String format1FirstMin = format1(_input);
	    if (format1FirstMin.length() > 1) {
	        return format1FirstMin.substring(0, 1).toLowerCase() + format1FirstMin.substring(1);
	    } else {
		    return format1FirstMin.toLowerCase();
	    }
    }

	/**
	 * Format 2
	 */
	public String format2(String input) {
		Vector s1 = splitAndDelete(input);
		String buffer = " ";
		for (int i = 0; i < s1.size(); i++) {
			String s2 = s1.elementAt(i).toString().toLowerCase();
			buffer = buffer.concat(s2);
			if (i != (s1.size() - 1))
				buffer = buffer.concat("_");
		}

		return buffer.trim();
	}

	/**
	 * Format 3
	 */
	public String format3(String input) {
		Vector s1 = splitAndDelete(input);
		String buffer = " ";
		int i;
		for (i = 0; i < s1.size(); i++) {
			String s2 = s1.elementAt(i).toString().toUpperCase();
			buffer = buffer.concat(s2);
			if (i != (s1.size() - 1))
				buffer = buffer.concat("_");
		}
		return buffer.trim();
	}


	/**
	 * QualfiedName
	 */
	public String qualifiedName(org.omg.mof.Model.ModelElement modelElement)
		throws org.omg.mof.Reflective.MofError , NotSet{
		boolean bool = false;
		String elementName = "";

		if (modelElement._is_a(org.omg.mof.Model.ClassHelper.id())) {
			elementName =
				className(org.omg.mof.Model.ClassHelper.narrow(modelElement));
		} else {
			elementName = modelElement.name();
			if (elementName.startsWith("*"))
				elementName = elementName.substring(1);
			// Use the format1 method to format the elementName
			elementName = format1(elementName);
		}
		
		String _namespacestring = elementName;

		org.omg.mof.Model.ModelElement current = modelElement;
		while (!bool) {
			try {
				current = current.container();
				_namespacestring =
					format1(current.name()) + "::" + _namespacestring;
			} catch (org.omg.mof.Reflective.NotSet ex) {
				bool = true;
			}

		}
		return _namespacestring;
	}

	public String className(org.omg.mof.Model.Class clazz)
		throws NotSet, MofError {
		String idlSubstitute = mofHelper.idlSubstituteIdentifierForClass(clazz);
		if (idlSubstitute.compareTo("") == 0)
			return format1(clazz.name());
		else return idlSubstitute;
	}

	public boolean isBaseType(org.omg.CORBA.TypeCode _typecode)
		throws BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		//logger.log(Level.FINE, "IDL typeCode2IDL");
		int val = _typecode.kind().value();
		if (val == TCKind._tk_alias
			|| val == TCKind._tk_struct
			|| val == TCKind._tk_enum
			|| val == TCKind._tk_sequence)
			return false;
		else
			return true;
	}

	////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////
	// ADDED METHOD
	////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////

	Vector splitAndDelete(String _input) {
		Vector parts = new Vector();
		String delim = "-_    .";
		StringTokenizer _sstring = new StringTokenizer(_input, delim);
		while (_sstring.hasMoreTokens()) {
			String part = _sstring.nextToken();
			int i = 0;
			int j = 0;
			boolean isPart = false;
			boolean beginPart = true;
			while (i < part.length()) {
				if (beginPart) {
					if ((Character.isDigit(part.charAt(i)))
						|| (Character.isUpperCase(part.charAt(i))))
						i++;
					else
						beginPart = false;
				} else {
					if ((Character.isDigit(part.charAt(i)))
						|| (Character.isLowerCase(part.charAt(i))))
						i++;
					else
						isPart = true;
				}
				if (isPart) {
					parts.addElement(part.substring(j, i));
					j = i;
					isPart = false;
					beginPart = true;
				}
				if (i == j)
					i++; //cas de caractre non spcif dans le filtre
			}
			if (i != j)
				parts.addElement(part.substring(j, i));
		}
		return parts;
	}
	
	/**
	 * Tests if the _input is a IDL keyword
	 * @param _input The word to test.
	 * @return The correct word to prevent IDL conflicts.
	 */
	public String testIdlConflict(String input) {
		String keywordsIDL2 = "abstract|any|attribute|boolean|case|char|const|context|custom|default|double|exception|enum|factory|false|fixed|float|in|include|inout|interface|local|long|module|native|object|octet|oneway|out|private|public|raises|readonly|sequence|short|string|struct|supports|switch|true|truncatable|typedef|unsigned|union|valuebase|valuetype|void|wchar|wstring";
		String keywordsIDL3 = "component|consumes|emits|eventtype|finder|getraises|home|import|multiple|primarykey|provides|publishes|setraises|typeid|typeprefix|uses";
		if (input.toLowerCase().matches(keywordsIDL2 + "|" + keywordsIDL3))
			return "_" + input;
		else
			return input;
	}
	
	/**
	 * String standardization : MOF Format 2 to prevent IDL conflicts <br>
	 * Helper.format2("AbcDefGhi")= "abc_def_ghi"
	 */
	public String format2IdlConflict(String input) {
		return testIdlConflict(format2(input).trim());
	}
	
}
