/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.provider;

import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.TypeCodePackage.Bounds;
import org.omg.mof.Model.ClassHelper;
import org.omg.mof.Model.Classifier;
import org.omg.mof.Model.DataType;
import org.omg.mof.Model.DataTypeHelper;
import org.omg.mof.Model.ModelElement;
import org.omg.mof.Model.Namespace;
import org.omg.mof.Model.PackageHelper;
import org.omg.mof.Reflective.MofError;
import org.omg.mof.Reflective.NotSet;

import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;

/**
 * @author Xavier Blanc
 */
public class JavaProvider extends IDLProvider implements JavaCommon {

	public JavaProvider(MOFCommon helper) {
		super(helper);
		mofHelper = helper;
	}

	MOFCommon mofHelper;
	
	public String testJavaConflict(String _input) {
		if (_input
			.matches("((.+(Package|Helper|Holder|Operations|POA|POATie)\\z)|(abstract|default|if|private|throw|boolean|do|implements|protected|throws|break|double|import|public|transient|byte|else|instanceof|return|try|case|extends|int|short|void||catch|final|interface|static|volatilechar|finally|long|super|while|class|float|native|switch|const|for|new|synchronized|continue|goto|package|this|true|false|null|clone|equals|finalize|getClass|hashCode|notify|notifyAll|toString|wait))"))
			return "_" + _input;
		else return _input;
	}

	/**
	 * @see fr.lip6.modfact.helper.JavaCommon#format1Idl2JavaConflict(String)
	 */
	public String format1Idl2JavaConflict(String _input) {
		Vector s1 = splitAndDelete(_input);
		String buffer = " ";
		for (int i = 0; i < s1.size(); i++) {
			char[] s2 = s1.elementAt(i).toString().toCharArray();
			s2[0] = Character.toUpperCase(s1.elementAt(i).toString().charAt(0));
			String s3 = String.valueOf(s2);
			buffer = buffer.concat(s3);
		}
		return testJavaConflict(buffer.trim());
	}
	
    /**
	 * @see fr.lip6.modfact.helper.JavaCommon#format2Idl2JavaConflict(String)
	 */
    public String format2Idl2JavaConflict(String _input) {
    	Vector s1 = splitAndDelete(_input);
		String buffer = " ";
		for (int i = 0; i < s1.size(); i++) {
			String s2 = s1.elementAt(i).toString().toLowerCase();
			buffer = buffer.concat(s2);
			if (i != (s1.size() - 1))
				buffer = buffer.concat("_");
		}
		return testJavaConflict(buffer.trim());
    }

	private Hashtable javaQualifiedNames = new Hashtable();
	/**
	 * @see fr.lip6.modfact.helper.JavaCommon#javaQualifiedName(ModelElement)
	 */
	public String javaQualifiedName(ModelElement element) throws MofError , NotSet {
		String qualifiedName;
		// voir dans la table si c'est dja calcul
		String _key = element.ref_mof_id();
		if (javaQualifiedNames.containsKey(_key))
			qualifiedName = (String) javaQualifiedNames.get(_key);
		else {
			if (element._is_a(ClassHelper.id())) qualifiedName = javaClassName(ClassHelper.narrow(element));
			else qualifiedName = format1(element.name());
			try {
				boolean outermost = false;
				while (!outermost) {
					Namespace container = element.container();
					qualifiedName = format1(container.name()) + "." + qualifiedName;
					//If container is idl_tagged
					if (container._is_a(PackageHelper.id())) {
						String prefix =
							mofHelper.idlPrefixForPackage(
								PackageHelper.narrow(container));
						if (prefix.compareTo("") != 0)
							qualifiedName =
								idlPrefix2Java(prefix) + "." + qualifiedName;
					}
					if (mofHelper.isOutermostPackage(container))
						outermost = true;
					else
						element = container;
				}
			} catch (NotSet ex) {
			}
		}
		return qualifiedName;

	}

	public String javaClassName(org.omg.mof.Model.Class clazz)
		throws NotSet, MofError {
		String idlSubstitute = mofHelper.idlSubstituteIdentifierForClass(clazz);
		String n = clazz.name();
		if (idlSubstitute.compareTo("") == 0)
			return format1(clazz.name());
		else
			return idlSubstitute;
	}
	
	/**
	 * Return the java classifier name of a classifier
	 */
	public String javaClassifierName(org.omg.mof.Model.Classifier clazz)
		throws NotSet, MofError {
		String idlSubstitute = mofHelper.idlSubstituteIdentifierForClass(clazz);
		String n = clazz.name();
		if (idlSubstitute.compareTo("") == 0)
			return format1(clazz.name());
		else
			return idlSubstitute;	
	}

	//partie qui traite les TypesJava
	public String idl2Java(String _type) {
		if (_type.equalsIgnoreCase("boolean"))
			return "boolean";
		else if (_type.equalsIgnoreCase("octet"))
			return "byte";
		else if (_type.equalsIgnoreCase("char"))
			return "char";
		else if (_type.equalsIgnoreCase("wchar"))
			return "char";
		else if (_type.equalsIgnoreCase("short"))
			return "Short";
		else if (_type.equalsIgnoreCase("unsigned short"))
			return "Short";
		else if (_type.equalsIgnoreCase("long"))
			return "int";
		else if (_type.equalsIgnoreCase("unsigned long"))
			return "int";
		else if (_type.equalsIgnoreCase("long long"))
			return "long";
		else if (_type.equalsIgnoreCase("unsigned long long"))
			return "long";
		else if (_type.equalsIgnoreCase("double"))
			return "double";
		else if (_type.equalsIgnoreCase("float"))
			return "float";
		else if (_type.equalsIgnoreCase("string"))
			return "String";
		else if (_type.equalsIgnoreCase("wstring"))
			return "String";
		else if (_type.equalsIgnoreCase("CORBA::TypeCode"))
			return "org.omg.CORBA.TypeCode";
		else if (_type.equalsIgnoreCase("any"))
			return "org.omg.CORBA.Any";
		else if (_type.equalsIgnoreCase("Object"))
			return "Object";
		// je pourrais dclacher une exception  revoir la igne ci-dessous
		else
			return _type;
	}

	//	partie qui traite les TypesJava
	public String typeCode2Java(TypeCode type) throws BadKind {
		switch (type.kind().value()) {
			case TCKind._tk_boolean :
				return "boolean";
			case TCKind._tk_octet :
				return "byte";
			case TCKind._tk_char :
				return "char";
			case TCKind._tk_wchar :
				return "char";
			case TCKind._tk_short :
				return "Short";
			case TCKind._tk_ushort :
				return "Short";
			case TCKind._tk_long :
				return "int";
			case TCKind._tk_ulong :
				return "int";
			case TCKind._tk_longlong :
				return "long";
			case TCKind._tk_ulonglong :
				return "long";
			case TCKind._tk_double :
				return "double";
			case TCKind._tk_float :
				return "float";
			case TCKind._tk_string :
				return "String";
			case TCKind._tk_wstring :
				return "String";
			case TCKind._tk_TypeCode :
				return "org.omg.CORBA.TypeCode";
			case TCKind._tk_any :
				return "org.omg.CORBA.Any";
			case TCKind._tk_objref :
				return "org.omg.CORBA.Object";
			default :
				return "String"; //should be something else
		}
	}

	/**
	 * @see fr.lip6.modfact.helper.JavaCommon#javaType(Classifier)
	 */
	public String javaType(Classifier classifier)
		throws MofError, BadKind, Bounds , NotSet{

		String resu = ""; // = javaQualifiedName(classifier);
		if (classifier._is_a(DataTypeHelper.id())) {
			//Base Type
			DataType dt = DataTypeHelper.narrow(classifier);
			TypeCode tc = dt.type_code();
			if (tc.kind().value() == org.omg.CORBA.TCKind._tk_alias) {
				try {
					resu = typeCode2Java(tc.content_type());
				} catch (org.omg.CORBA.TypeCodePackage.BadKind badKind) {
					resu = typeCode2Java(tc);
				}
			} else {
				if (isBaseType(tc)) {
						resu = typeCode2Java(tc);
				} else {
					try {
						Namespace container = classifier.container();
						if (container._is_a(ClassHelper.id())) { //What Else ???
							resu =
								javaQualifiedName(container)
									+ "ClassPackage."
									+ format1(dt.name());
						} else
							resu = javaQualifiedName(classifier);
					} catch (NotSet ex) {
						resu = format1(dt.name());
					}
				}
			}
		} else {
			resu = javaQualifiedName(classifier);
		}
		return resu;
	}

	/**
	 * @see fr.lip6.modfact.helper.JavaCommon#javaPackage(Namespace)
	 */
	private Hashtable packagesNameTable = new Hashtable();
	public String javaPackage(org.omg.mof.Model.Package namespace)
		throws MofError {

		// si c'est dja calculer
		String _key = namespace.ref_mof_id();
		String packageName = null;
		if (packagesNameTable.containsKey(_key))
			packageName = (String) packagesNameTable.get(_key);
		else {
			//calculer le prefix
			String prefix = mofHelper.idlPrefixForPackage(namespace);
			String[] qualifiedName = namespace.qualified_name();
			packageName = format1Idl2JavaConflict(qualifiedName[0]);
			for (int i = 1; i < qualifiedName.length; i++)
				packageName = format1(qualifiedName[i]) + "." + packageName;
			if (prefix != "")
				packageName = prefix + "." + packageName;
		}
		return packageName;
	}

	//Attribute Template
	//une table de hashage pour minimiser le retratement des attributs
	private java.util.Hashtable _attributs = new java.util.Hashtable();

	public String attributeTemplate(org.omg.mof.Model.MofAttribute _attribute)
		throws org.omg.mof.Reflective.MofError, BadKind, Bounds , NotSet {

		//voir si c'est dja clacul
		String _key = _attribute.ref_mof_id();
		if (_attributs.containsKey(_key)) {
			String _temp = (String) _attributs.get(_key);
			return _temp;
		} else {
			//dcalration de la chaine qui contiendra la code
			String _code_attribute = "//" + _attribute.annotation() + "\n";

			String _type_format1 = javaType(_attribute.type());
			String _aformat2 = format2(_attribute.name());
			org.omg.mof.Model.MultiplicityType _multiplicity =
				_attribute.multiplicity();
			int _upper = _multiplicity.upper;
			int _lower = _multiplicity.lower;
			boolean _is_ordered = _multiplicity.isOrdered;
			boolean _is_unique = _multiplicity.isUnique;
			boolean _is_changeable = _attribute.is_changeable();
			boolean _is_derived = _attribute.is_derived();

			//Dclaration de l'attribut if non derived
			if (!_is_derived) {
			if ((_upper > 1) || (_upper == -1))
				_code_attribute =
					_code_attribute.concat(
						"private java.util.Vector  _"
							+ _aformat2
							+ "s = new java.util.Vector();\n");
			else
				_code_attribute =
					_code_attribute.concat(
						"private "
							+ _type_format1
							+ "  _"
							+ _aformat2
							+ " ;\n");
			if ((_lower == 0) && (_upper == 1))
				_code_attribute =
					_code_attribute.concat(
						"private boolean _exist_"
							+ _aformat2
							+ " = false;\n");
			}

			//Dclaration des mthodes qui convient
			if ((_lower == 0) && (_upper == 1)) {
				if (_is_derived) {
					_code_attribute =
						_code_attribute.concat(
							"public "
								+ _type_format1
								+ " "
								+ _aformat2
								+ "() throws org.omg.mof.Reflective.NotSet, org.omg.mof.Reflective.MofError { \n");
					_code_attribute =
						_code_attribute.concat(
							"//inserer le code contenu\n");
					_code_attribute = _code_attribute.concat("}\n");
				} else {
					_code_attribute =
						_code_attribute.concat(
							"public "
								+ _type_format1
								+ " "
								+ _aformat2
								+ "() throws org.omg.mof.Reflective.NotSet, org.omg.mof.Reflective.MofError { \n");
					_code_attribute =
						_code_attribute.concat(
							"if (_exist_"
								+ _aformat2
								+ ") return _"
								+ _aformat2
								+ ";\n");
					_code_attribute =
						_code_attribute.concat(
							"else throw new org.omg.mof.Reflective.NotSet();\n");
					_code_attribute = _code_attribute.concat("}\n");
				}
			}

			if ((_lower == 1) && (_upper == 1)) {
				if (_is_derived) {
					_code_attribute =
						_code_attribute.concat(
							"public "
								+ _type_format1
								+ " "
								+ _aformat2
								+ "() throws org.omg.mof.Reflective.MofError {\n ");
					_code_attribute =
						_code_attribute.concat(
							"//inserer le code contenu\n");
					_code_attribute = _code_attribute.concat("}\n");
				} else {
					_code_attribute =
						_code_attribute.concat(
							"public "
								+ _type_format1
								+ " "
								+ _aformat2
								+ "() throws org.omg.mof.Reflective.MofError {\n ");
					_code_attribute =
						_code_attribute.concat(
							"return _" + _aformat2 + ";\n");
					_code_attribute = _code_attribute.concat("}\n");
				}
			}

			if ((_upper > 1) || (_upper == -1)) {
				if (_is_derived) {
					_code_attribute =
						_code_attribute.concat(
							"\tpublic "
								+ _type_format1
								+ "[]  "
								+ _aformat2
								+ "() throws org.omg.mof.Reflective.MofError { \n");
					_code_attribute =
						_code_attribute.concat(
							"//inserer le code contenu\n");
					_code_attribute = _code_attribute.concat("}\n");
				} else {
					_code_attribute =
						_code_attribute.concat(
							"public "
								+ _type_format1
								+ "[]  "
								+ _aformat2
								+ "() throws org.omg.mof.Reflective.MofError { \n");
					_code_attribute =
						_code_attribute.concat(
							_type_format1
								+ "[] temp1 = new "
								+ _type_format1
								+ "[_"
								+ _aformat2
								+ "s.size()];\n");
					_code_attribute =
						_code_attribute.concat(
							"for(int i=0; i< _"
								+ _aformat2
								+ "s.size(); i++) temp1[i] = ("
								+ _type_format1
								+ ")_"
								+ _aformat2
								+ "s.elementAt(i);\n");
					_code_attribute =
						_code_attribute.concat("return temp1;\n");
					_code_attribute = _code_attribute.concat("}\n");
				}
			}

			if ((_upper == 1) && (_is_changeable)) {
				_code_attribute =
					_code_attribute.concat(
						"public void set_"
							+ _aformat2
							+ "("
							+ _type_format1
							+ " new_value) throws org.omg.mof.Reflective.MofError { \n");
				if (!_is_derived) {
					_code_attribute =
						_code_attribute.concat(
							"_" + _aformat2 + " = new_value ;\n");
					if (_lower == 0) {
						_code_attribute =
							_code_attribute.concat(
								"_exist_" + _aformat2 + " = true;\n");
					}
				} else _code_attribute = _code_attribute.concat(
					"throw new org.omg.mof.Reflective.MofError();\n");
					_code_attribute = _code_attribute.concat("}\n");
			}

			if (((_upper > 1) || (_upper == -1)) && (_is_changeable)) {
				_code_attribute =
					_code_attribute.concat(
						"public void set_"
							+ _aformat2
							+ "("
							+ _type_format1
							+ "[] new_value) throws org.omg.mof.Reflective.MofError { \n");
							if (!_is_derived) {
				_code_attribute =
					_code_attribute.concat(
						"_" + _aformat2 + "s.removeAllElements();\n");
				_code_attribute =
					_code_attribute.concat(
						"for(int i= 0; i< new_value.length ; i++) _"
							+ _aformat2
							+ "s.addElement(new_value[i]);\n");}
							else _code_attribute = _code_attribute.concat(
						"throw new org.omg.mof.Reflective.MofError();\n");
				_code_attribute = _code_attribute.concat("}\n");
			}

			if ((_lower == 0) && (_upper == 1) && (_is_changeable)) {
				_code_attribute =
					_code_attribute.concat(
						"public void unset_"
							+ _aformat2
							+ "() throws org.omg.mof.Reflective.MofError{ \n");
							if (!_is_derived) {
				_code_attribute =
					_code_attribute.concat(
						"_exist_" + _aformat2 + " = false;\n");}
						else _code_attribute =_code_attribute.concat(
						"throw new org.omg.mof.Reflective.MofError();\n");
				_code_attribute = _code_attribute.concat("}\n");
			}

			if ((_upper > 1) || (_upper == -1) && (_is_changeable)) {
				_code_attribute =
					_code_attribute.concat(
						"public void add_"
							+ _aformat2
							+ " ("
							+ _type_format1
							+ " new_element) throws org.omg.mof.Reflective.MofError{ \n");
							if (!_is_derived) {
				_code_attribute =
					_code_attribute.concat(
						"_" + _aformat2 + "s.addElement(new_element);\n");}
						else _code_attribute = _code_attribute.concat(
						"throw new org.omg.mof.Reflective.MofError();\n");
				_code_attribute = _code_attribute.concat("}\n");
			}

			if ((_upper > 1)
				|| (_upper == -1)
				&& (_is_changeable)
				&& (_is_ordered)) {
				_code_attribute =
					_code_attribute.concat(
						"public void add_"
							+ _aformat2
							+ "_before ("
							+ _type_format1
							+ " new_element, "
							+ _type_format1
							+ " before_element) throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{ \n");
							if (!_is_derived) {
				_code_attribute =
					_code_attribute.concat(
						"if(_"
							+ _aformat2
							+ "s.contains(before_element)) {\n");
				_code_attribute =
					_code_attribute.concat(
						"int i = _"
							+ _aformat2
							+ "s.indexOf(before_element);\n");
				_code_attribute =
					_code_attribute.concat(
						"_"
							+ _aformat2
							+ "s.insertElementAt(new_element, i);\n");
				_code_attribute = _code_attribute.concat("}\n");
				_code_attribute =
					_code_attribute.concat(
						"else throw new org.omg.mof.Reflective.NotFound() ;\n");}
						else _code_attribute =_code_attribute.concat(
						"throw new org.omg.mof.Reflective.MofError();\n");
				_code_attribute = _code_attribute.concat("}\n");
			}

			if ((_upper > 1)
				|| (_upper == -1)
				&& (_is_changeable)
				&& (_is_ordered)
				&& (!_is_unique)) {
				_code_attribute =
					_code_attribute.concat(
						"public void add_"
							+ _aformat2
							+ "_at ("
							+ _type_format1
							+ " new_element, int position) throws org.omg.mof.Reflective.BadPosition, org.omg.mof.Reflective.MofError{ \n");
							if (!_is_derived) {
				_code_attribute =
					_code_attribute.concat(
						"if(_"
							+ _aformat2
							+ "s.size()> position) _"
							+ _aformat2
							+ "s.insertElementAt(new_element, position);\n");
				_code_attribute =
					_code_attribute.concat(
						"else throw new org.omg.mof.Reflective.BadPosition() ;\n");}
						else _code_attribute = _code_attribute.concat(
						"throw new org.omg.mof.Reflective.MofError();\n");
				_code_attribute = _code_attribute.concat("}\n");
			}

			if ((_upper > 1) || (_upper == -1) && (_is_changeable)) {
				_code_attribute =
					_code_attribute.concat(
						"public void modify_"
							+ _aformat2
							+ " ("
							+ _type_format1
							+ " old_element,"
							+ _type_format1
							+ " new_element) throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{ \n");
							if (!_is_derived) {
				_code_attribute =
					_code_attribute.concat(
						"if( _"
							+ _aformat2
							+ "s.contains(old_element)) {\n");
				_code_attribute =
					_code_attribute.concat(
						"int i =_"
							+ _aformat2
							+ "s.indexOf(old_element);\n");
				_code_attribute =
					_code_attribute.concat(
						"_"
							+ _aformat2
							+ "s.setElementAt(new_element, i);\n");
				_code_attribute = _code_attribute.concat("}\n");
				_code_attribute =
					_code_attribute.concat(
						"else throw new org.omg.mof.Reflective.NotFound();\n");}
						else _code_attribute = _code_attribute.concat(
						"throw new org.omg.mof.Reflective.MofError();\n");
				_code_attribute = _code_attribute.concat("}\n");
			}

			if ((_upper > 1)
				|| (_upper == -1)
				&& (_is_changeable)
				&& (_is_ordered)
				&& (!_is_unique)) {
				_code_attribute =
					_code_attribute.concat(
						"public void modify_"
							+ _aformat2
							+ "_at ("
							+ _type_format1
							+ " new_element, int position) throws org.omg.mof.Reflective.BadPosition, org.omg.mof.Reflective.MofError{\n");
							if (!_is_derived) {
				_code_attribute =
					_code_attribute.concat(
						"if(_"
							+ _aformat2
							+ "s.size()> position) _"
							+ _aformat2
							+ "s.setElementAt(new_element, position);\n");
				_code_attribute =
					_code_attribute.concat(
						"else throw new org.omg.mof.Reflective.BadPosition() ;\n");}
						else _code_attribute = _code_attribute.concat(
						"throw new org.omg.mof.Reflective.MofError();\n");
				_code_attribute = _code_attribute.concat("}\n");
			}

			if ((_upper > 1)
				|| (_upper == -1)
				&& (_upper != _lower)
				&& (_is_changeable)) {
				_code_attribute =
					_code_attribute.concat(
						"public void remove_"
							+ _aformat2
							+ " ("
							+ _type_format1
							+ " old_element ) throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError { \n");
							if (!_is_derived) {
				_code_attribute =
					_code_attribute.concat(
						"if(_"
							+ _aformat2
							+ "s.contains(old_element)) _"
							+ _aformat2
							+ "s.removeElement(old_element);\n");
				_code_attribute =
					_code_attribute.concat(
						"else throw new org.omg.mof.Reflective.NotFound() ;\n");}
						else _code_attribute = _code_attribute.concat(
						"throw new org.omg.mof.Reflective.MofError();\n");
						
				_code_attribute = _code_attribute.concat("}\n");
			}

			if ((_upper > 1)
				|| (_upper == -1)
				&& (_upper != _lower)
				&& (_is_changeable)
				&& (_is_ordered)
				&& (!_is_unique)) {
				_code_attribute =
					_code_attribute.concat(
						"public void remove_"
							+ _aformat2
							+ "_at (int position) throws org.omg.mof.Reflective.BadPosition, org.omg.mof.Reflective.MofError { \n");
							if (!_is_derived) {
				_code_attribute =
					_code_attribute.concat(
						"if(_"
							+ _aformat2
							+ "s.size()> position) _"
							+ _aformat2
							+ "s.remove( position);\n");
				_code_attribute =
					_code_attribute.concat(
						"else throw new org.omg.mof.Reflective.BadPosition() ;\n");}
						else _code_attribute = _code_attribute.concat(
						"throw new org.omg.mof.Reflective.MofError();\n");
				_code_attribute = _code_attribute.concat("}\n");
			}
			_attributs.put(_key, _code_attribute);
			return _code_attribute;
		}
	}

	//Reference Template
	//une table de hashage pour minimiser le retratement des attributs
	private java.util.Hashtable _references = new java.util.Hashtable();

	public String referenceTemplate(org.omg.mof.Model.Reference _reference)
		throws org.omg.mof.Reflective.MofError , NotSet {

		//voir si c'est dja clacul
		String _key = _reference.ref_mof_id();
		if (_references.containsKey(_key)) {
			String _temp = (String) _references.get(_key);
			return _temp;
		} else {
			//dcalration de la chaine qui contiendra la code
			String _code_reference = "//" + _reference.annotation() + "\n";

			String _type_format1 = javaQualifiedName(_reference.type());
			String _aformat2 = format2(_reference.name());
			String _aformat2Idl2JavaConflict = format2Idl2JavaConflict(_reference.name());
			org.omg.mof.Model.MultiplicityType _multiplicity =
				_reference.multiplicity();
			int _upper = _multiplicity.upper;
			int _lower = _multiplicity.lower;
			boolean _is_ordered = _multiplicity.isOrdered;
			boolean _is_unique = _multiplicity.isUnique;
			boolean _is_changeable = _reference.is_changeable();

			// Dclaration des mthode de la rfrence
			if ((_upper > 1) || (_upper == -1))
				_code_reference =
					_code_reference.concat(
						"private java.util.Vector  _"
							+ _aformat2
							+ "s = new java.util.Vector();\n");
			else
				_code_reference =
					_code_reference.concat(
						"private "
							+ _type_format1
							+ "  _"
							+ _aformat2
							+ " = null;\n");

			//Dclaration des mthodes qui convient
			if ((_lower == 0) && (_upper == 1)) {
				_code_reference =
					_code_reference.concat(
						"public "
							+ _type_format1
							+ " "
							+ _aformat2Idl2JavaConflict
							+ "() throws org.omg.mof.Reflective.NotSet, org.omg.mof.Reflective.MofError { \n");
				_code_reference =
					_code_reference.concat(
						"if (_"
							+ _aformat2
							+ "!= null) return _"
							+ _aformat2
							+ ";\n");
				_code_reference =
					_code_reference.concat(
						"else throw new org.omg.mof.Reflective.NotSet();\n");
				_code_reference = _code_reference.concat("}\n");
			}

			if ((_lower == 1) && (_upper == 1)) {
				_code_reference =
					_code_reference.concat(
						"public "
							+ _type_format1
							+ " "
							+ _aformat2Idl2JavaConflict
							+ "() throws org.omg.mof.Reflective.MofError { \n");
				_code_reference =
					_code_reference.concat("return _" + _aformat2 + ";\n");
				_code_reference = _code_reference.concat("}\n");
			}

			if ((_upper > 1) || (_upper == -1)) {
				_code_reference =
					_code_reference.concat(
						"public "
							+ _type_format1
							+ "[]  "
							+ _aformat2Idl2JavaConflict
							+ "() throws org.omg.mof.Reflective.MofError { \n");
				_code_reference =
					_code_reference.concat(
						""
							+ _type_format1
							+ "[] temp1 = new "
							+ _type_format1
							+ "[_"
							+ _aformat2
							+ "s.size()];\n");
				_code_reference =
					_code_reference.concat(
						"for(int i=0; i< _"
							+ _aformat2
							+ "s.size(); i++) temp1[i] = ("
							+ _type_format1
							+ ")_"
							+ _aformat2
							+ "s.elementAt(i);\n");
				_code_reference = _code_reference.concat("return temp1;\n");
				_code_reference = _code_reference.concat("}\n");
			}

			if ((_upper == 1) && (_is_changeable)) {
				_code_reference =
					_code_reference.concat(
						"public void set_"
							+ _aformat2
							+ "("
							+ _type_format1
							+ " new_value) throws org.omg.mof.Reflective.MofError { \n");
				_code_reference =
					_code_reference.concat(
						"_" + _aformat2 + " = new_value ;\n");
				_code_reference = _code_reference.concat("}\n");
			}

			if (((_upper > 1) || (_upper == -1)) && (_is_changeable)) {
				_code_reference =
					_code_reference.concat(
						"public void set_"
							+ _aformat2
							+ "("
							+ _type_format1
							+ "[] new_value) throws org.omg.mof.Reflective.MofError { \n");
				_code_reference =
					_code_reference.concat(
						"_" + _aformat2 + "s.removeAllElements();\n");
				_code_reference =
					_code_reference.concat(
						"for(int i= 0; i< new_value.length ; i++) _"
							+ _aformat2
							+ "s.addElement(new_value[i]);\n");
				_code_reference = _code_reference.concat("}\n");
			}

			if ((_lower == 0) && (_upper == 1) && (_is_changeable)) {
				_code_reference =
					_code_reference.concat(
						"public void unset_"
							+ _aformat2
							+ "() throws org.omg.mof.Reflective.MofError{ \n");
				_code_reference =
					_code_reference.concat("_" + _aformat2 + " = null;\n");
				_code_reference = _code_reference.concat("}\n");
			}

			if ((_upper > 1) || (_upper == -1) && (_is_changeable)) {
				_code_reference =
					_code_reference.concat(
						"public void add_"
							+ _aformat2
							+ " ("
							+ _type_format1
							+ " new_element) throws org.omg.mof.Reflective.MofError{ \n");
				_code_reference =
					_code_reference.concat(
						"_" + _aformat2 + "s.addElement(new_element);\n");
				_code_reference = _code_reference.concat("}\n");
			}

			if ((_upper > 1)
				|| (_upper == -1)
				&& (_is_changeable)
				&& (_is_ordered)) {
				_code_reference =
					_code_reference.concat(
						"public void add_"
							+ _aformat2
							+ "_before ("
							+ _type_format1
							+ " new_element, "
							+ _type_format1
							+ " before_element) throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{ \n");
				_code_reference =
					_code_reference.concat(
						"if(_"
							+ _aformat2
							+ "s.contains(before_element)) {\n");
				_code_reference =
					_code_reference.concat(
						"int i = _"
							+ _aformat2
							+ "s.indexOf(before_element);\n");
				_code_reference =
					_code_reference.concat(
						"_"
							+ _aformat2
							+ "s.insertElementAt(new_element, i);\n");
				_code_reference = _code_reference.concat("}\n");
				_code_reference =
					_code_reference.concat(
						"else throw new org.omg.mof.Reflective.NotFound() ;\n");
				_code_reference = _code_reference.concat("}\n");
			}

			if ((_upper > 1) || (_upper == -1) && (_is_changeable)) {
				_code_reference =
					_code_reference.concat(
						"public void modify_"
							+ _aformat2
							+ " ("
							+ _type_format1
							+ " old_element,"
							+ _type_format1
							+ " new_element) throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{ \n");
				_code_reference =
					_code_reference.concat(
						"if( _"
							+ _aformat2
							+ "s.contains(old_element)) {\n");
				_code_reference =
					_code_reference.concat(
						"int i =_"
							+ _aformat2
							+ "s.indexOf(old_element);\n");
				_code_reference =
					_code_reference.concat(
						"_"
							+ _aformat2
							+ "s.setElementAt(new_element, i);\n");
				_code_reference = _code_reference.concat("}\n");
				_code_reference =
					_code_reference.concat(
						"else throw new org.omg.mof.Reflective.NotFound();\n");
				_code_reference = _code_reference.concat("}\n");
			}

			if ((_upper > 1)
				|| (_upper == -1)
				&& (_upper != _lower)
				&& (_is_changeable)) {
				_code_reference =
					_code_reference.concat(
						"public void remove_"
							+ _aformat2
							+ " ("
							+ _type_format1
							+ " old_element ) throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError { \n");
				_code_reference =
					_code_reference.concat(
						"if(_"
							+ _aformat2
							+ "s.contains(old_element)) _"
							+ _aformat2
							+ "s.removeElement(old_element);\n");
				_code_reference =
					_code_reference.concat(
						"else throw new org.omg.mof.Reflective.NotFound() ;\n");
				_code_reference = _code_reference.concat("}\n");
			}
			_references.put(_key, _code_reference);
			return _code_reference;
		}
	}

	//Attribute Template
	//une table de hashage pour minimiser le retratement des attributs
	private java.util.Hashtable _operations = new java.util.Hashtable();

	public String operationTemplate(org.omg.mof.Model.Operation _operation)
		throws org.omg.mof.Reflective.MofError, BadKind, Bounds , NotSet{

		//voir si c'est dja clacul
		String _key = _operation.ref_mof_id();
		if (_operations.containsKey(_key)) {
			String _temp = (String) _operations.get(_key);
			return _temp;
		} else {

			String _code_operation = "//" + _operation.annotation() + "\n";
			//String _oformat2 = format2(_operation.name());
			String _oformat2 = format2Idl2JavaConflict(_operation.name());

			org.omg.mof.Model.ModelElement[] _contents = _operation.contents();

			boolean bool = false;
			int j = 0;
			while (!bool && (j < _contents.length)) {
				org.omg.mof.Model.Parameter _parameter =
					org.omg.mof.Model.ParameterHelper.narrow(_contents[j]);
				if ((_parameter.direction())
					.equals(org.omg.mof.Model.DirectionKind.return_dir)) {
					bool = true;
					if ((_parameter.multiplicity().upper > 1)
						|| (_parameter.multiplicity().upper == -1))
						_code_operation =
							_code_operation.concat(
								"public "
									+ javaType(_parameter.type())
									+ "[] "
									+ _oformat2
									+ " (");
					else
						_code_operation =
							_code_operation.concat(
								"public "
									+ javaType(_parameter.type())
									+ " "
									+ _oformat2
									+ "(");
					j = j + 1;
				}
			}
			if (!bool)
				_code_operation = _code_operation.concat("public void "+_oformat2 +"(");
			for (int i = 0; i < _contents.length; i++) {
				org.omg.mof.Model.Parameter _parameter =
					org.omg.mof.Model.ParameterHelper.narrow(_contents[i]);
				String _type_format1 = javaType(_parameter.type());
				String _pformat2 = format2(_parameter.name());
				if (!(_parameter
					.direction()
					.equals(org.omg.mof.Model.DirectionKind.return_dir))) {
					if ((_parameter.multiplicity().upper > 1)
						|| (_parameter.multiplicity().upper == -1))
						_code_operation =
							_code_operation.concat(
								_type_format1 + "[] " + _pformat2 + " ");
					else
						_code_operation =
							_code_operation.concat(
								_type_format1 + " " + _oformat2 + " ");
					if (i != (_contents.length - 1))
						_code_operation = _code_operation.concat(", ");
				}
			}
			_code_operation = _code_operation.concat(") throws ");
			org.omg.mof.Model.MofException[] _exceptions =
				_operation.exceptions();
			for (int i = 0; i < _exceptions.length; i++) {
				org.omg.mof.Model.MofException _exception = _exceptions[i];
				String _eformat1 = format1Idl2JavaConflict(_exception.name());
				_code_operation = _code_operation.concat(_eformat1);
				_code_operation = _code_operation.concat(", ");
			}
			_code_operation =
				_code_operation.concat("org.omg.mof.Reflective.MofError {\n");
			_code_operation =
				_code_operation.concat("//inserer le code contenu\n");
			_code_operation =
				_code_operation.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\" method not implemented \",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_operation = _code_operation.concat("}\n");
			_references.put(_key, _code_operation);
			return _code_operation;
		}
	}

	//Ajout de la partie RefBaseObject
	private String _code_ref_base_object = null;
	public String refBaseObjectTemplate() {

		// voir si c'ets dja Calcul
		if (_code_ref_base_object != null)
			return _code_ref_base_object;

		else {
			String _code_temp = "";
			_code_temp =
				_code_temp.concat("//Ajout de la partie RefBaseObject \n");
			_code_temp = _code_temp.concat("// Operation ref_mof_id\n");
			_code_temp =
				_code_temp.concat(
					"public java.lang.String ref_mof_id(){\n");
			_code_temp =
				_code_temp.concat(
					"int id = _orb().object_to_string(_this()).hashCode();\n");
			_code_temp =
				_code_temp.concat("return String.valueOf(id);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_meta_object\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective.RefObject ref_meta_object(){\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_meta_object method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp = _code_temp.concat("// Operation ref_itself\n");
			_code_temp =
				_code_temp.concat(
					"public boolean ref_itself(org.omg.mof.Reflective.RefBaseObject other_object){\n");
			_code_temp =
				_code_temp.concat(
					"String id_obj = other_object.ref_mof_id();\n");
			_code_temp =
				_code_temp.concat(
					"if (ref_mof_id().equals(id_obj)) return true;\n");
			_code_temp = _code_temp.concat("else return false;\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_immediate_package\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective._RefPackage ref_immediate_package(){\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_immediate_package method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_outermost_package\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective._RefPackage ref_outermost_package(){\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_moutermost_package method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp = _code_temp.concat("// Operation ref_delete\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_delete() throws org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_delete method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			//Stocker
			_code_ref_base_object = _code_temp;
			return _code_ref_base_object;
		}

	}

	//Ajout de la partie RefAssociation
	private String _code_ref_association = null;
	public String refAssociationTemplate() {

		// voir si c'ets dja Calcul
		if (_code_ref_association != null)
			return _code_ref_association;

		else {
			String _code_temp = "";
			_code_temp = _code_temp.concat("// Operation ref_all_links\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective.RefObject[][] ref_all_links(){\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_all_links method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_link_exists\n");
			_code_temp =
				_code_temp.concat(
					"public boolean ref_link_exists(org.omg.mof.Reflective.RefObject[] some_link)\n");
			_code_temp =
				_code_temp.concat(
					"    throws org.omg.mof.Reflective.MofError{\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_link_exists method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp = _code_temp.concat("// Operation ref_query\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective.RefObject[] ref_query(org.omg.mof.Reflective.RefObject query_end, org.omg.mof.Reflective.RefObject query_object)\n");
			_code_temp =
				_code_temp.concat(
					"    throws org.omg.mof.Reflective.MofError{\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_query method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp = _code_temp.concat("// Operation ref_add_link\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_add_link(org.omg.mof.Reflective.RefObject[] new_link)\n");
			_code_temp =
				_code_temp.concat(
					"    throws org.omg.mof.Reflective.MofError{\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_add_link method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_add_link_before\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_add_link_before(org.omg.mof.Reflective.RefObject[] new_link, org.omg.mof.Reflective.RefObject position_end, org.omg.mof.Reflective.RefObject before)\n");
			_code_temp =
				_code_temp.concat(
					"    throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_add_link_before method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_modify_link\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_modify_link(org.omg.mof.Reflective.RefObject[] old_link, org.omg.mof.Reflective.RefObject position_end, org.omg.mof.Reflective.RefObject new_object)\n");
			_code_temp =
				_code_temp.concat(
					"    throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError{\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_modify_link method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_remove_link\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_remove_link(org.omg.mof.Reflective.RefObject[] old_link)\n");
			_code_temp =
				_code_temp.concat(
					"    throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_remove_link method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");
			//Stocker
			_code_ref_association = _code_temp;
			return _code_ref_association;
		}
	}

	//Ajout de la partie RefObject
	private String _code_ref_object = null;
	public String refObjectTemplate() {

		// voir si c'ets dja Calcul
		if (_code_ref_object != null)
			return _code_ref_object;

		else {
			String _code_temp = "";
			_code_temp =
				_code_temp.concat("//Ajout de la partie  RefObject\n");
			_code_temp =
				_code_temp.concat("// Operation ref_is_instance_of\n");
			_code_temp =
				_code_temp.concat(
					"public boolean ref_is_instance_of(org.omg.mof.Reflective.RefObject some_class, boolean consider_subtypes) {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_is_instance_of method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_create_instance\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective.RefObject ref_create_instance(org.omg.CORBA.Any[] args) throws org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_create_instance method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_all_objects\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective.RefObject[] ref_all_objects(boolean include_subtypes){\n");
			_code_temp =
				_code_temp.concat(
					" throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_all_objects method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp = _code_temp.concat("// Operation ref_set_value\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_set_value(org.omg.mof.Reflective.RefObject feature, org.omg.CORBA.Any new_value) throws org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_set_value method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp = _code_temp.concat("// Operation ref_value\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.CORBA.Any ref_value(org.omg.mof.Reflective.RefObject feature) throws org.omg.mof.Reflective.NotSet, org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_value method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_unset_value\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_unset_value() throws org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_unset_value method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp = _code_temp.concat("// Operation ref_add_value\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_add_value(org.omg.mof.Reflective.RefObject feature, org.omg.CORBA.Any new_element) throws org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_add_value method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_add_value_before\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_add_value_before(org.omg.mof.Reflective.RefObject feature, org.omg.CORBA.Any new_element, org.omg.CORBA.Any before_element)\n");
			_code_temp =
				_code_temp.concat(
					"	 throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_add_value_before method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_add_value_at\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_add_value_at(org.omg.mof.Reflective.RefObject feature, org.omg.CORBA.Any new_element, int position)\n");
			_code_temp =
				_code_temp.concat(
					"	 throws org.omg.mof.Reflective.BadPosition, org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_add_value_at method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_modify_value\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_modify_value(org.omg.mof.Reflective.RefObject feature, org.omg.CORBA.Any old_element, org.omg.CORBA.Any new_element)\n");
			_code_temp =
				_code_temp.concat(
					"	 throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_modify_value method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_modify_value_at\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_modify_value_at(org.omg.mof.Reflective.RefObject feature, org.omg.CORBA.Any new_element, int position)\n");
			_code_temp =
				_code_temp.concat(
					"	 throws org.omg.mof.Reflective.BadPosition, org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_modify_value_at method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_remove_value\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_remove_value(org.omg.mof.Reflective.RefObject feature, org.omg.CORBA.Any existing_element)\n");
			_code_temp =
				_code_temp.concat(
					"	 throws org.omg.mof.Reflective.NotFound, org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_remove_value method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_remove_value_at\n");
			_code_temp =
				_code_temp.concat(
					"public void ref_remove_value_at(org.omg.mof.Reflective.RefObject feature, int position)\n");
			_code_temp =
				_code_temp.concat(
					"	 throws org.omg.mof.Reflective.BadPosition, org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_remove_value_at method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_immediate_composite\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective.RefObject ref_immediate_composite() {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_remove_value method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_outermost_composite\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective.RefObject ref_outermost_composite() {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_outermost_composite method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_invoke_operation(\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.CORBA.Any ref_invoke_operation(org.omg.mof.Reflective.RefObject requested_temp, org.omg.mof.Reflective.ValueTypeListHolder args)\n");
			_code_temp =
				_code_temp.concat(
					"	 throws org.omg.mof.Reflective.OtherException, org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_invoke_operation( method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			//Stocker
			_code_ref_object = _code_temp;
			return _code_ref_object;
		}
	}

	private String _code_ref_package = null;
	/**
	 * @see fr.lip6.modfact.helper.JavaCommon#refPackageTemplate()
	 */
	public String refPackageTemplate() {
		// voir si c'ets dja Calcul
		if (_code_ref_package != null)
			return _code_ref_package;

		else {
			String _code_temp = "";
			_code_temp = _code_temp.concat("// Operation ref_class_ref\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective.RefObject ref_class_ref(org.omg.mof.Reflective.RefObject _class)\n");
			_code_temp =
				_code_temp.concat(
					"  throws org.omg.mof.Reflective.MofError{\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_class_ref method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("  }\n");

			_code_temp =
				_code_temp.concat("// Operation ref_association_ref\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective.RefAssociation ref_association_ref(org.omg.mof.Reflective.RefObject association)\n");
			_code_temp =
				_code_temp.concat(
					"  throws org.omg.mof.Reflective.MofError {\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_association_ref method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			_code_temp =
				_code_temp.concat("// Operation ref_package_ref\n");
			_code_temp =
				_code_temp.concat(
					"public org.omg.mof.Reflective._RefPackage ref_package_ref(org.omg.mof.Reflective.RefObject _package)\n");
			_code_temp =
				_code_temp.concat(
					"  throws org.omg.mof.Reflective.MofError{\n");
			_code_temp =
				_code_temp.concat(
					"throw new org.omg.CORBA.NO_IMPLEMENT(\"ref_package_ref method not implemented\",1 ,org.omg.CORBA.CompletionStatus.COMPLETED_NO);\n");
			_code_temp = _code_temp.concat("}\n");

			//Stocker
			_code_ref_package = _code_temp;
			return _code_ref_package;
		}

	}

	/**
	 * Invers prefix for Java Impl
	 */
	public String idlPrefix2Java(String prefix) {
		StringTokenizer st = new StringTokenizer(prefix, ".");
		String resu = "";
		while (st.hasMoreTokens()) {
			resu = resu + st.nextToken();
			if (st.hasMoreTokens())
				resu = resu + ".";
		}
		return resu;
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

}
