/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.provider;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;
import org.omg.mof.Model.*;
import org.omg.mof.Reflective.*;

import org.objectweb.modfact.corba.helper.MOFCommon;

/**
 * 
 */
public class MOFCachedProvider implements MOFCommon {

	/**
	 * Give the value of a Constant 
	 */
	public String valueOfConstant(org.omg.CORBA.Any _value)
		throws org.omg.CORBA.TypeCodePackage.BadKind {
		org.omg.CORBA.TypeCode _typecode = _value.type();
		switch (_typecode.kind().value()) {
			case org.omg.CORBA.TCKind._tk_boolean :
				return String.valueOf(_value.extract_boolean());

			case org.omg.CORBA.TCKind._tk_octet :
				return String.valueOf(
					(new Byte(_value.extract_octet())).toString());

			case org.omg.CORBA.TCKind._tk_char :
				return String.valueOf(_value.extract_char());

			case org.omg.CORBA.TCKind._tk_wchar :
				return String.valueOf(_value.extract_wchar());

			case org.omg.CORBA.TCKind._tk_short :
				return String.valueOf(
					(new Short(_value.extract_short())).toString());

			case org.omg.CORBA.TCKind._tk_ushort :
				return String.valueOf(
					(new Short(_value.extract_ushort())).toString());

			case org.omg.CORBA.TCKind._tk_long :
				return String.valueOf(_value.extract_long());

			case org.omg.CORBA.TCKind._tk_ulong :
				return String.valueOf(_value.extract_ulong());

			case org.omg.CORBA.TCKind._tk_longlong :
				return String.valueOf(_value.extract_longlong());

			case org.omg.CORBA.TCKind._tk_ulonglong :
				return String.valueOf(_value.extract_ulonglong());

			case org.omg.CORBA.TCKind._tk_double :
				return String.valueOf(_value.extract_double());

			case org.omg.CORBA.TCKind._tk_float :
				return String.valueOf(_value.extract_float());

			case org.omg.CORBA.TCKind._tk_string :
				return _value.extract_string();

			case org.omg.CORBA.TCKind._tk_wstring :
				return _value.extract_wstring();

			default :
				throw new org.omg.CORBA.TypeCodePackage.BadKind();
		}
	}

	/**
	 * Give the attributes of a Class (included the inherited ones)
	 */
	public org.omg.mof.Model.MofAttribute[] attributesOfClass(
		org.omg.mof.Model.Class _class,
		org.omg.mof.Model.ScopeKind _scope,
		boolean _test_of_derived)
		throws org.omg.mof.Reflective.MofError {
		String key = _class.ref_mof_id();
		MOFClassCached classCached =
			(MOFClassCached) _contentsOfClasses.get(key);
		Vector res;
		if (_scope.equals(org.omg.mof.Model.ScopeKind.classifier_level)) {
			if (_test_of_derived)
				res = classCached.attributesClass;
			else
				res = classCached.attributesClassNd;
		} else {
			if (_test_of_derived)
				res = classCached.attributesInst;
			else
				res = classCached.attributesInstNd;
		}
		org.omg.mof.Model.MofAttribute[] resT =
			new org.omg.mof.Model.MofAttribute[res.size()];
		for (int i = 0; i < resT.length; i++)
			resT[i] = (org.omg.mof.Model.MofAttribute) res.elementAt(i);
		return resT;

	}
	
	/**
	 * Adds an element into a vector if it is not present
	 * (Test with the _is_equivalent method)
	 * @param vector The vector which contains the elements
	 * @param object The element to add
	 */
	private void addElement(Vector vector, org.omg.CORBA.Object object){
		boolean find = false;
		java.util.Iterator it = vector.iterator();
		while (it.hasNext() && !find) {
			org.omg.CORBA.Object obj = (org.omg.CORBA.Object) it.next();
			if (obj._is_equivalent(object))
				find = true;
		}
		if (!find)
			vector.addElement(object);
	}

	/**
	 * Give all the attributes of a Class
	 */
	public org.omg.mof.Model.MofAttribute[] allAttributesOfClass(
		org.omg.mof.Model.Class _class,
		org.omg.mof.Model.ScopeKind _scope,
		boolean isDerived)
		throws org.omg.mof.Reflective.MofError {
		//avoir toutes les classes
		Vector allSuperClasses = superClassesOrdered(_class);

		//crer le vecteur conteneur
		Vector attributesVector = new Vector();

		for (int i = 0; i < allSuperClasses.size(); i++) {

			//claculer la cl
			String classId =
				((org.omg.mof.Model.Class) allSuperClasses.elementAt(i))
					.ref_mof_id();

			if (_scope.equals(ScopeKind.classifier_level)) {
				if (!isDerived) {
					Vector _temp =
						(
							(MOFClassCached) _contentsOfClasses.get(
								classId)).attributesClassNd;
					for (int j = 0; j < _temp.size(); j++)
						addElement(attributesVector,
							(org.omg.mof.Model.MofAttribute) _temp.elementAt(
								j));
				} else {
					Vector _temp =
						(
							(MOFClassCached) _contentsOfClasses.get(
								classId)).attributesClass;
					for (int j = 0; j < _temp.size(); j++)
						addElement(attributesVector,
							(org.omg.mof.Model.MofAttribute) _temp.elementAt(
								j));
				}
			} else {
				if (!isDerived) {
					Vector _temp =
						(
							(MOFClassCached) _contentsOfClasses.get(
								classId)).attributesInstNd;
					for (int j = 0; j < _temp.size(); j++)
						addElement(attributesVector,
							(org.omg.mof.Model.MofAttribute) _temp.elementAt(
								j));
				} else {
					Vector _temp =
						(
							(MOFClassCached) _contentsOfClasses.get(
								classId)).attributesInst;
					for (int j = 0; j < _temp.size(); j++)
						addElement(attributesVector,
							(org.omg.mof.Model.MofAttribute) _temp.elementAt(
								j));
				}
			}
		}
		org.omg.mof.Model.MofAttribute[] res =
			new org.omg.mof.Model.MofAttribute[attributesVector.size()];
		for (int i = 0; i < res.length; i++)
			res[i] =
				(org.omg.mof.Model.MofAttribute) attributesVector.elementAt(i);
		return res;

	}

	/**
	 * Give the operations of a Class
	 */
	public org.omg.mof.Model.Operation[] operationsOfClass(
		org.omg.mof.Model.Class _class,
		org.omg.mof.Model.ScopeKind _scope)
		throws org.omg.mof.Reflective.MofError {
		Vector operationVector;
		MOFClassCached classCached =
			(MOFClassCached) _contentsOfClasses.get(_class.ref_mof_id());
		if (_scope.equals(org.omg.mof.Model.ScopeKind.classifier_level))
			operationVector = classCached.operationsClass;
		else
			operationVector = classCached.operationsInst;
		org.omg.mof.Model.Operation[] res =
			new org.omg.mof.Model.Operation[operationVector.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Operation) operationVector.elementAt(i);
		return res;

	}

	/**
	 * Give all the operation of a Class (included the inherited ones)
	 */
	public org.omg.mof.Model.Operation[] allOperationsOfClass(
		org.omg.mof.Model.Class _class,
		org.omg.mof.Model.ScopeKind _scope)
		throws org.omg.mof.Reflective.MofError {
		//avoir toutes les classes
		Vector allSuperclasses = superClassesOrdered(_class);

		//crer le vecteur conteneur
		Vector operationsVector = new Vector();

		for (int i = 0; i < allSuperclasses.size(); i++) {

			//claculer la cl
			String classId =
				((org.omg.mof.Model.Class) allSuperclasses.elementAt(i))
					.ref_mof_id();
			if (_scope.equals(ScopeKind.classifier_level)) {
				Vector _temp =
					(
						(MOFClassCached) _contentsOfClasses.get(
							classId)).operationsClass;
				for (int j = 0; j < _temp.size(); j++)
					addElement(operationsVector, (Operation) _temp.elementAt(j));
			} else {
				Vector _temp =
					(
						(MOFClassCached) _contentsOfClasses.get(
							classId)).operationsInst;
				for (int j = 0; j < _temp.size(); j++)
					addElement(operationsVector, (Operation) _temp.elementAt(j));
			}
		}
		org.omg.mof.Model.Operation[] res =
			new org.omg.mof.Model.Operation[operationsVector.size()];
		for (int i = 0; i < res.length; i++)
			res[i] =
				(org.omg.mof.Model.Operation) operationsVector.elementAt(i);
		return res;

	}

	/**
	 * Give the references of a Class
	 */
	public org.omg.mof.Model.Reference[] referencesOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError {
		MOFClassCached classCached =
			(MOFClassCached) _contentsOfClasses.get(_class.ref_mof_id());
		Vector references = classCached.refrences;
		org.omg.mof.Model.Reference[] res =
			new org.omg.mof.Model.Reference[references.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Reference) references.elementAt(i);
		return res;

	}

	/**
	 * Give all the references of a Class (included the inherited ones)
	 */
	public org.omg.mof.Model.Reference[] allReferencesOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError {
		//avoir toutes les classes
		Vector allSuperClasses = superClassesOrdered(_class);

		//crer le vecteur conteneur
		Vector referencesVector = new Vector();

		for (int i = 0; i < allSuperClasses.size(); i++) {
			//claculer la cl
			String classId =
				((org.omg.mof.Model.Class) allSuperClasses.elementAt(i))
					.ref_mof_id();

			Vector _temp =
				((MOFClassCached) _contentsOfClasses.get(classId)).refrences;
			for (int j = 0; j < _temp.size(); j++)
				addElement(referencesVector, (Reference) _temp.elementAt(j));
		}
		org.omg.mof.Model.Reference[] res =
			new org.omg.mof.Model.Reference[referencesVector.size()];
		for (int i = 0; i < res.length; i++)
			res[i] =
				(org.omg.mof.Model.Reference) referencesVector.elementAt(i);
		return res;
	}

	/**
	 * Give the constants of a Class
	 */
	public org.omg.mof.Model.Constant[] constantsOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError {
		MOFClassCached classCached =
			(MOFClassCached) _contentsOfClasses.get(_class.ref_mof_id());
		Vector constants = classCached.mofConstants;
		org.omg.mof.Model.Constant[] res =
			new org.omg.mof.Model.Constant[constants.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Constant) constants.elementAt(i);
		return res;
	}

	/**
	 * Give the datatypes of a Class
	 */
	public org.omg.mof.Model.DataType[] datatypesOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError {
		MOFClassCached classCached =
			(MOFClassCached) _contentsOfClasses.get(_class.ref_mof_id());
		Vector datatypes = classCached.mofDatatypes;
		org.omg.mof.Model.DataType[] res =
			new org.omg.mof.Model.DataType[datatypes.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.DataType) datatypes.elementAt(i);
		return res;
	}

	/**
	 * Give the exceptions of a Class
	 */
	public org.omg.mof.Model.MofException[] exceptionsOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError {
		MOFClassCached classCached =
			(MOFClassCached) _contentsOfClasses.get(_class.ref_mof_id());
		Vector exceptions = classCached.mofExcepion;
		org.omg.mof.Model.MofException[] res =
			new org.omg.mof.Model.MofException[exceptions.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.MofException) exceptions.elementAt(i);
		return res;
	}

	/**
	 * Give the constrains of a Class
	 */
	public org.omg.mof.Model.Constraint[] constraintsOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError {
		MOFClassCached classCached =
			(MOFClassCached) _contentsOfClasses.get(_class.ref_mof_id());
		Vector constraints = classCached.mofConstraints;
		org.omg.mof.Model.Constraint[] res =
			new org.omg.mof.Model.Constraint[constraints.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Constraint) constraints.elementAt(i);
		return res;
	}

	/**
	 * Give the associations ends of an Association
	 */
	public org.omg.mof.Model.AssociationEnd[] associationEndsOfAssociation(
		org.omg.mof.Model.Association _association)
		throws org.omg.mof.Reflective.MofError {
		AssociationEnd[] associationEnds = new AssociationEnd[2];
		ModelElement[] contents = _association.contents();

		for (int i = 0; i < contents.length; i++) {
			if (contents[i]._is_a(AssociationEndHelper.id()))
				associationEnds[i] =
					(org
						.omg
						.mof
						.Model
						.AssociationEndHelper
						.narrow(contents[i]));
		}
		return associationEnds;
	}
	
	/**
	 * Give the opposite association end of an association end
	 */
	public org.omg.mof.Model.AssociationEnd oppositeAssociationEnd(
		org.omg.mof.Model.AssociationEnd _association_end)
		throws org.omg.mof.Reflective.MofError {
		try {
			org.omg.mof.Model.Association asso = org.omg.mof.Model.AssociationHelper.narrow(_association_end.container());
			org.omg.mof.Model.AssociationEnd[] ends = associationEndsOfAssociation(asso);
			if (ends[0]._is_equivalent(_association_end))
				return ends[1];
			else
				return ends[0];
		} catch (org.omg.mof.Reflective.NotSet notSet) {
			return null;
		}
	}

	/**
	 * Give the classes of a Package
	 */
	public org.omg.mof.Model.Class[] classesOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		// vecteur contenuer 
		Hashtable disorderedClasses = new Hashtable();

		//le contenu 
		ModelElement[] contents = _package.contents();
		for (int i = 0; i < contents.length; i++) {
			ModelElement current = contents[i];
			if (current._is_a(ClassHelper.id())) {
				org.omg.mof.Model.Class _class = ClassHelper.narrow(current);
				if (!disorderedClasses.containsKey(_class.ref_mof_id()))
					disorderedClasses.put(_class.ref_mof_id(), _class);
			}
		}
		org.omg.mof.Model.Class[] res =
			new org.omg.mof.Model.Class[disorderedClasses.size()];
		int i = 0;
		for (Enumeration e = disorderedClasses.elements();
			e.hasMoreElements();
			) {
			res[i++] = (org.omg.mof.Model.Class) e.nextElement();
		}
		return res;

	}

	/**
	 * Give the classes of a Package
	 */
	public org.omg.mof.Model.Class[] classesOfPackageOrderedByInheritance(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		org.omg.mof.Model.Class[] classesUnordered = classesOfPackage(_package);
		
		Vector classesUnorderedV = new Vector();
		Vector classesOrderedV = new Vector();
		for (int i = 0; i < classesUnordered.length; i++)
			classesUnorderedV.addElement(classesUnordered[i]);

		int courant = 0;
		while (classesUnorderedV.size() != 0) {
			org.omg.mof.Model.Class _courant = null;
			_courant =
				(org.omg.mof.Model.Class) classesUnorderedV.elementAt(courant);
			org.omg.mof.Model.GeneralizableElement[] supertypes =
				_courant.supertypes();
				
			boolean _is_top = true;
			int j = 0;
			while (_is_top && j < supertypes.length) {
				// Use _is_equivalent method instead of calling equals method (by Vector.contains())
				// if (classesUnorderedV.contains(supertypes[j]))
				//     _is_top = false;
				for (int i=0 ; i<classesUnorderedV.size() ; i++) {
					org.omg.CORBA.Object o = (org.omg.CORBA.Object) classesUnorderedV.elementAt(i);
					if (o._is_equivalent(supertypes[j]))
						_is_top = false;
				}
				
				j++;
			}
			if (_is_top) {
				classesOrderedV.addElement(_courant);
				classesUnorderedV.removeElement(_courant);
				courant = 0;
			} else
				courant++;
		}
		org.omg.mof.Model.Class[] resu =
			new org.omg.mof.Model.Class[classesOrderedV.size()];
		for (int i = 0; i < resu.length; i++)
			resu[i] = (org.omg.mof.Model.Class) classesOrderedV.elementAt(i);
		return resu;
	}

	/**
	 * Give the classes of a Package
	 */
	public org.omg.mof.Model.Class[] allClassesOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		// vecteur contenuer 
		Hashtable disorderedClasses = new Hashtable();

		//le contenu 
		ModelElement[] contents = _package.contents();
		for (int i = 0; i < contents.length; i++) {
			ModelElement current = contents[i];
			if (current._is_a(ClassHelper.id())) {
				org.omg.mof.Model.Class _class = ClassHelper.narrow(current);
				if (!disorderedClasses.containsKey(_class.ref_mof_id()))
					disorderedClasses.put(_class.ref_mof_id(), _class);
			} else if (current._is_a(ImportHelper.id())) {
				Import _import = ImportHelper.narrow(current);

				//rcupration des caractristiques du namesapce rcupr
				boolean isClustered = _import.is_clustered();
				Namespace namespace = _import.imported_namespace();

				//traitement
				if (isClustered) {
					if (namespace._is_a(ClassHelper.id())) {
						org.omg.mof.Model.Class _class =
							ClassHelper.narrow(namespace);
						if (!disorderedClasses
							.containsKey(_class.ref_mof_id()))
							disorderedClasses.put(_class.ref_mof_id(), _class);
					} else if (namespace._is_a(PackageHelper.id())) {
						// c'est un package 
						org.omg.mof.Model.Package clusteredPacakge =
							PackageHelper.narrow(namespace);

						org.omg.mof.Model.Class[] contentsCluster =
							classesOfPackage(clusteredPacakge);
						for (int j = 0; j < contentsCluster.length; j++) {
							String key =
								(String) contentsCluster[j].ref_mof_id();
							if (!disorderedClasses.containsKey(key))
								disorderedClasses.put(key, contentsCluster[j]);

						}
					}
				}
			} else
				//nested Package 
				if (contents[i]._is_a(PackageHelper.id())) {
					org.omg.mof.Model.Package nestedPackage =
						PackageHelper.narrow(contents[i]);

					org.omg.mof.Model.Class[] contentsNest =
						classesOfPackage(nestedPackage);
					for (int j = 0; j < contentsNest.length; j++) {
						String key = (String) contentsNest[j].ref_mof_id();
						if (!disorderedClasses.containsKey(key))
							disorderedClasses.put(key, contentsNest[j]);
					}
				}
		}
		org.omg.mof.Model.Class[] res =
			new org.omg.mof.Model.Class[disorderedClasses.size()];
		int i = 0;
		for (Enumeration e = disorderedClasses.elements();
			e.hasMoreElements();
			) {
			res[i++] = (org.omg.mof.Model.Class) e.nextElement();
		}
		return res;
	}

	/**
	 * Give the all classes of a Package
	 */
	public org.omg.mof.Model.Class[] allClassesOfPackageOrderedByInheritance(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		org.omg.mof.Model.Class[] classesUnordered =
			allClassesOfPackage(_package);

		Vector classesUnorderedV = new Vector();
		Vector classesOrderedV = new Vector();
		for (int i = 0; i < classesUnordered.length; i++)
			classesUnorderedV.addElement(classesUnordered[i]);

		int courant = 0;
		while (classesUnorderedV.size() != 0) {
			org.omg.mof.Model.Class _courant = null;
			_courant =
				(org.omg.mof.Model.Class) classesUnorderedV.elementAt(courant);
			org.omg.mof.Model.GeneralizableElement[] supertypes =
				_courant.supertypes();
			boolean _is_top = true;
			int j = 0;
			while (_is_top && j < supertypes.length) {
				if (classesUnorderedV.contains(supertypes[j]))
					_is_top = false;
				j++;
			}
			if (_is_top) {
				classesOrderedV.addElement(_courant);
				classesUnorderedV.removeElement(_courant);
				courant = 0;
			} else
				courant++;
		}
		org.omg.mof.Model.Class[] resu =
			new org.omg.mof.Model.Class[classesOrderedV.size()];
		for (int i = 0; i < resu.length; i++)
			resu[i] = (org.omg.mof.Model.Class) classesOrderedV.elementAt(i);
		return resu;
	}

	/**
	 * Give the sub-classes of a Package
	 */
	public org.omg.mof.Model.Class[] subClassesOfClass(
		org.omg.mof.Model.Classifier _class)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		//calcules les sous classes
		Vector subClasses = new Vector();

		//Rechercher dans toutes les classes du meta modele
		Enumeration keysSet = _contentsOfPackages.keys();

		while (keysSet.hasMoreElements()) {

			//id du package
			String packageId = (String) keysSet.nextElement();
			Vector classesPackage =
				(
					(MOFPackageCached) _contentsOfPackages.get(
						packageId)).mofClasses;

			for (int i = 0; i < classesPackage.size(); i++) {
				org.omg.mof.Model.Class testedClass =
					(org.omg.mof.Model.Class) classesPackage.elementAt(i);

				GeneralizableElement[] superClasses =
					testedClass.all_supertypes();

				int j = 0;
				boolean sub = false;

				while ((j < superClasses.length) && !sub) {
					if (superClasses[j]
						.ref_mof_id()
						.equals(_class.ref_mof_id())) {
						sub = true;

						//test de la duplication 
						//if (!subClasses.contains(testedClass))
						//	subClasses.addElement(testedClass);
						addElement(subClasses, testedClass);
					}
					j++;
				}
			}
		}

		org.omg.mof.Model.Class[] subTable =
			new org.omg.mof.Model.Class[subClasses.size()];
		for (int i = 0; i < subClasses.size(); i++)
			subTable[i] = (org.omg.mof.Model.Class) subClasses.elementAt(i);
		return subTable;

	}

	/**
	 * Give the packages of a Package
	 */
	public org.omg.mof.Model.Package[] packagesOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		MOFPackageCached packageCached =
			(MOFPackageCached) _contentsOfPackages.get(_package.ref_mof_id());
		Vector packages = packageCached.mofPackages;
		org.omg.mof.Model.Package[] res =
			new org.omg.mof.Model.Package[packages.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Package) packages.elementAt(i);
		return res;
	}

	/**
	 * Give the associations of a Package
	 */
	public org.omg.mof.Model.Association[] allAssociationsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		// vecteur contenuer 
		Hashtable disorderedAssociations = new Hashtable();

		//le contenu 
		ModelElement[] contents = _package.contents();
		for (int i = 0; i < contents.length; i++) {
			ModelElement current = contents[i];
			if (current._is_a(AssociationHelper.id())) {
				org.omg.mof.Model.Association _association = AssociationHelper.narrow(current);
				if (!disorderedAssociations.containsKey(_association.ref_mof_id()))
					disorderedAssociations.put(_association.ref_mof_id(), _association);
			} else if (current._is_a(ImportHelper.id())) {
				Import _import = ImportHelper.narrow(current);
				//rcupration des caractristiques du namespace rcupr
				boolean isClustered = _import.is_clustered();
				Namespace namespace = _import.imported_namespace();
				//traitement
				if (isClustered) {
					if (namespace._is_a(AssociationHelper.id())) {
						org.omg.mof.Model.Association _association =
							AssociationHelper.narrow(namespace);
						if (!disorderedAssociations
							.containsKey(_association.ref_mof_id()))
							disorderedAssociations.put(_association.ref_mof_id(), _association);
					} else if (namespace._is_a(PackageHelper.id())) {
						// c'est un package 
						org.omg.mof.Model.Package clusteredPacakge =
							PackageHelper.narrow(namespace);
						org.omg.mof.Model.Association[] contentsCluster =
							associationsOfPackage(clusteredPacakge);
						for (int j = 0; j < contentsCluster.length; j++) {
							String key =
								(String) contentsCluster[j].ref_mof_id();
							if (!disorderedAssociations.containsKey(key))
								disorderedAssociations.put(key, contentsCluster[j]);
						}
					}
				}
			} else
				//nested Package 
				if (contents[i]._is_a(PackageHelper.id())) {
					org.omg.mof.Model.Package nestedPackage =
						PackageHelper.narrow(contents[i]);
					org.omg.mof.Model.Association[] contentsNest =
						associationsOfPackage(nestedPackage);
					for (int j = 0; j < contentsNest.length; j++) {
						String key = (String) contentsNest[j].ref_mof_id();
						if (!disorderedAssociations.containsKey(key))
							disorderedAssociations.put(key, contentsNest[j]);
					}
				}
		}
		org.omg.mof.Model.Association[] res =
			new org.omg.mof.Model.Association[disorderedAssociations.size()];
		int i = 0;
		for (Enumeration e = disorderedAssociations.elements();
			e.hasMoreElements();
			) {
			res[i++] = (org.omg.mof.Model.Association) e.nextElement();
		}
		return res;
	}

	/**
	 * Give the associations of a Package
	 */
	public org.omg.mof.Model.Association[] associationsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		MOFPackageCached packageCached =
			(MOFPackageCached) _contentsOfPackages.get(_package.ref_mof_id());
		Vector associations = packageCached.mofAssociations;
		org.omg.mof.Model.Association[] res =
			new org.omg.mof.Model.Association[associations.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Association) associations.elementAt(i);
		return res;
	}

	/**
	 * Give the imports of a Package
	 */
	public org.omg.mof.Model.Import[] importsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		MOFPackageCached packageCached =
			(MOFPackageCached) _contentsOfPackages.get(_package.ref_mof_id());
		Vector imports = packageCached.mofImports;
		org.omg.mof.Model.Import[] res =
			new org.omg.mof.Model.Import[imports.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Import) imports.elementAt(i);
		return res;
	}

	/**
	 * Give the tags of a Package
	 */
	public org.omg.mof.Model.Tag[] tagsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		MOFPackageCached packageCached =
			(MOFPackageCached) _contentsOfPackages.get(_package.ref_mof_id());
		Vector tags = packageCached.mofTags;
		org.omg.mof.Model.Tag[] res = new org.omg.mof.Model.Tag[tags.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Tag) tags.elementAt(i);
		return res;
	}

	/**
	 * Give the exception of a Package
	 */
	public org.omg.mof.Model.MofException[] exceptionsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		MOFPackageCached packageCached =
			(MOFPackageCached) _contentsOfPackages.get(_package.ref_mof_id());
		Vector exceptions = packageCached.mofExceptions;
		org.omg.mof.Model.MofException[] res =
			new org.omg.mof.Model.MofException[exceptions.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.MofException) exceptions.elementAt(i);
		return res;
	}

	/**
	 * Give the constraints of a Package
	 */
	public org.omg.mof.Model.Constraint[] constraintsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		MOFPackageCached packageCached =
			(MOFPackageCached) _contentsOfPackages.get(_package.ref_mof_id());
		Vector constraints = packageCached.mofConstraints;
		org.omg.mof.Model.Constraint[] res =
			new org.omg.mof.Model.Constraint[constraints.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Constraint) constraints.elementAt(i);
		return res;
	}

	/**
	 * Give the constrants of a Package
	 */
	public org.omg.mof.Model.Constant[] constantsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		MOFPackageCached packageCached =
			(MOFPackageCached) _contentsOfPackages.get(_package.ref_mof_id());
		Vector constants = packageCached.mofConstants;
		org.omg.mof.Model.Constant[] res =
			new org.omg.mof.Model.Constant[constants.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.Constant) constants.elementAt(i);
		return res;
	}

	/**
	 * Give the datatypes of a Package
	 */
	public org.omg.mof.Model.DataType[] datatypesOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError {
		MOFPackageCached packageCached =
			(MOFPackageCached) _contentsOfPackages.get(_package.ref_mof_id());
		Vector datatypes = packageCached.mofDataType;
		org.omg.mof.Model.DataType[] res =
			new org.omg.mof.Model.DataType[datatypes.size()];
		for (int i = 0; i < res.length; i++)
			res[i] = (org.omg.mof.Model.DataType) datatypes.elementAt(i);
		return res;
	}

	/**
	 * Give the references of an Association
	 */
	public org.omg.mof.Model.Reference[] referencesOfAssociation(
		org.omg.mof.Model.Association _association)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		Vector referencesVector = new Vector();

		//rcuprer le package conteneur de l'association
		org.omg.mof.Model.Package containerPackage =
			PackageHelper.narrow(_association.container());

		//rcuprer toutes les classes dans le package conteneur
		MOFPackageCached packageTuple =
			(MOFPackageCached) _contentsOfPackages.get(
				containerPackage.ref_mof_id());

		Vector classesOfPackage = packageTuple.mofClasses;

		//rcuprer les AssociationEnd
		AssociationEnd[] association_ends =
			associationEndsOfAssociation(_association);

		//commencer la vrification
		for (int i = 0; i < classesOfPackage.size(); i++) {

			org.omg.mof.Model.Class _class =
				(org.omg.mof.Model.Class) classesOfPackage.elementAt(i);

			//rcuprer les rfrences contenues dans une classe
			MOFClassCached classTuple =
				(MOFClassCached) _contentsOfClasses.get(_class.ref_mof_id());
			Vector references = classTuple.refrences;

			for (int j = 0; j < references.size(); j++) {
				Reference _reference = (Reference) references.elementAt(j);

				AssociationEnd _association_end = _reference.referenced_end();

				// is_equivalent marche juste sur le Builder sinon sur le dos camarche equals
				if (_association_end.equals(association_ends[0])
					|| _association_end._is_equivalent(association_ends[1])) {
					referencesVector.addElement(_reference);
				}
			}
		}

		Reference[] referencesTable = new Reference[referencesVector.size()];
		for (int i = 0; i < referencesTable.length; i++)
			referencesTable[i] = (Reference) referencesVector.elementAt(i);
		return referencesTable;

	}

	private boolean contains(
		java.util.Vector _vector,
		org.omg.mof.Reflective.RefBaseObject _object) {
		if (_vector.size() == 0)
			return false;
		int i = 0;
		boolean _is_contained = false;
		while (!_is_contained && (i < _vector.size())) {
			org.omg.mof.Reflective.RefBaseObject _current =
				(org.omg.mof.Reflective.RefBaseObject) _vector.elementAt(i);
			if (_current.ref_itself(_object))
				_is_contained = true;
			else
				i++;
		}
		return _is_contained;
	}

	/**
	 * Give the outermost package that includes the ModelElement
	 */
	public org.omg.mof.Model.Package outermostPackageOfModelElement(
		org.omg.mof.Model.ModelElement _element)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		org.omg.mof.Model.Package _package = null;

		ModelElement container = _element;

		boolean find = false;
		while (!find) {
			if (container._is_a(PackageHelper.id())) {
				_package = PackageHelper.narrow(container);
				if (isOutermostPackage(_package))
					find = true;
				else
					container = container.container();

			} else
				container = container.container();
		}
		return _package;
	}

	/**
	* Return true is Package is an outermostPackage
	*/
	public boolean isOutermostPackage(org.omg.mof.Model.Namespace _package) {
		boolean isOutermost = false;
		if (_outermostPackages.containsKey(_package.ref_mof_id()))
			isOutermost = true;
		return isOutermost;
	}

	/**
	 * Give the value of the Prefix Tag of a Namespace
	 */
	public String idlPrefixForPackage(org.omg.mof.Model.Package _package)
		throws MofError {

		String _prefix_mof = "";

		MOFPackageCached _packageTuple =
			(MOFPackageCached) _contentsOfPackages.get(_package.ref_mof_id());

		Vector tags = _packageTuple.mofTags;

		boolean end = false;
		int index = 0;
		while (!end && (index < tags.size())) {
			Tag _tag = (Tag) tags.elementAt(index);
			ModelElement[] _elements = _tag.elements();
			if (_elements[0].ref_itself(_package))
				end = true;
			else
				index++;
		}
		if (end) {
			Tag _the_tag = (Tag) tags.elementAt(index);
			if (_the_tag.tag_id().equals("org.omg.mof.idl_prefix")) {
				StringTokenizer _prefix_inv =
					new StringTokenizer(
						_the_tag.values()[0].extract_string(),
						".");
				String[] _temp = new String[_prefix_inv.countTokens()];
				for (int i = 0; i < _temp.length; i++)
					_temp[i] = _prefix_inv.nextToken();
				String _prefix = "";
				for (int i = _temp.length - 1; i > -1; i--) {
					_prefix = _prefix + _temp[i];
					if (i != 0)
						_prefix = _prefix + ".";
				}
				_prefix_mof = _prefix;
			}
		}
		return _prefix_mof;
	}

	/**
	 * Return the idl_prefix assigned to the package if any (return "" else)
	 */
	public String idlSubstituteIdentifierForClass(
		org.omg.mof.Model.Classifier clazz)
		throws MofError, NotSet {
		org.omg.mof.Model.Package _package =
			PackageHelper.narrow(clazz.container());
		Tag[] tags = tagsOfPackage(_package);
		boolean idlSubstituteB = false;
		String idlSubstitute = "";
		int i = 0;
		while (!idlSubstituteB
			&& i < tags.length) { //org.omg.mof.idl_substitute_name
			if (tags[i].tag_id().compareTo("org.omg.mof.idl_substitute_name")
				== 0) {
					String name = tags[i].elements().toString();
					String name2 = tags[i].name();
				//String name = tags[i].elements()[0].name();
				if (tags[i].elements().length > 0) {
					if ((tags[i].elements()[0]!=null) && (tags[i].elements()[0]._is_a(org.omg.mof.Model.ClassifierHelper.id()))) {
					if (tags[i].elements()[0].ref_itself(clazz)) {
						idlSubstitute = tags[i].values()[0].extract_string();
						idlSubstituteB = true;
					}}
				}
			}
			i++;
		}
		return idlSubstitute;
	}

	/**
	 * Return the jmi_prefix assigned to the package if any (return "" else)
	 */
	public String jmiPrefixForPackage(org.omg.mof.Model.Package _package)
		throws MofError {
		Tag[] tags = tagsOfPackage(_package);
		boolean jmiPrefix = false;
		String jmi_prefix = "";
		int i = 0;
		while (!jmiPrefix && i < tags.length) { //com.urep.jcr.java_prefix
			if ((tags[i].tag_id().compareTo("javax.jmi.packagePrefix") == 0
				|| tags[i].tag_id().compareTo("com.urep.jcr.java_prefix") == 0)
				&& tags[i].elements()[0].ref_itself(_package)) {
				jmi_prefix = tags[i].values()[0].extract_string();
				jmiPrefix = true;
			} else
				i++;
		}
		return jmi_prefix;
	}

	/**
	 * 
	 */
	public String jmiSubstituteIdentifierForClass(
		org.omg.mof.Model.Class clazz)
		throws MofError, NotSet {
		org.omg.mof.Model.Package _package =
			PackageHelper.narrow(clazz.container());
		Tag[] tags = tagsOfPackage(_package);
		boolean jmiSubstituteB = false;
		String jmiSubstitute = "";
		int i = 0;
		while (!jmiSubstituteB && i < tags.length) { //com.urep.jcr.java_prefix
			if (tags[i].tag_id().compareTo("javax.jmi.substituteName") == 0
				|| tags[i].tag_id().compareTo("com.urep.jcr.java_substitute_name")
					== 0) {
				String name = tags[i].elements()[0].name();
				if (tags[i].elements()[0].ref_itself(clazz)) {
					jmiSubstitute = tags[i].values()[0].extract_string();
					jmiSubstituteB = true;
				}
			}
			i++;
		}
		return jmiSubstitute;
	}

	/**
	 * Return true if sub class is in the same extent of super class
	 */
	public boolean isInSameExtent(
		org.omg.mof.Model.Class innerExtent,
		org.omg.mof.Model.Class outerExtent)
		throws NotSet, MofError {
		boolean isInSameExtent = false;
		org.omg.mof.Model.Package packageOfOuterClass =
			PackageHelper.narrow(innerExtent.container());
		org.omg.mof.Model.Package packageOfInnerClass =
			PackageHelper.narrow(outerExtent.container());

		//Same package
		if (packageOfOuterClass.ref_itself(packageOfInnerClass))
			return true;

		//Nested package
		if (isNested(packageOfOuterClass, packageOfInnerClass))
			return true;

		//Clustered package
		Import[] imports = importsOfPackage(packageOfInnerClass);
		int i = 0;
		while (!isInSameExtent && i < imports.length) {
			if (imports[i].is_clustered()
				&& packageOfOuterClass.ref_itself(
					imports[i].imported_namespace()))
				isInSameExtent = true;
			else
				i++;
		}

		return isInSameExtent;
	}

	public boolean isNested(
		org.omg.mof.Model.Package nestedPackage,
		org.omg.mof.Model.Package nestingPackage)
		throws MofError {
		boolean isNested = false;
		org.omg.mof.Model.Package nestedPackages[] =
			packagesOfPackage(nestingPackage);
		int i = 0;
		while (!isNested && i < nestedPackages.length) {
			if (nestedPackage.ref_itself(nestedPackages[i]))
				isNested = true;
			else {
				isNested = isNested(nestedPackage, nestedPackages[i]);
				i++;
			}
		}
		return isNested;
	}

	////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////
	// ADDED METHOD
	////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////

	// Here are the caches
	private Hashtable _outermostPackages = new Hashtable();
	private Hashtable _packagesOfMetamodel = new Hashtable();
	private Hashtable _contentsOfPackages = new Hashtable();
	private Hashtable _contentsOfClasses = new Hashtable();

	/**
	 * Fill all the caches
	 */
	public void init(org.omg.mof.Model.Package[] outermostPackages)
		throws NotSet, MofError {
		for (int i = 0; i < outermostPackages.length; i++) {
			org.omg.mof.Model.Package _package = outermostPackages[i];
			String key = _package.ref_mof_id();
			_packagesOfMetamodel.put(key, _package);
			_outermostPackages.put(key, "");
			contentsOfPackage(_package);
		}
	}

	/**
	 * Fill all the caches for a package
	 */
	private void contentsOfPackage(org.omg.mof.Model.Package _package)
		throws MofError {

		MOFPackageCached current = new MOFPackageCached();

		//traitement du contenu
		ModelElement[] _contents = _package.contents();

		for (int i = 0; i < _contents.length; i++) {
			if (_contents[i]._is_a(ClassHelper.id())) {
				org.omg.mof.Model.Class _class =
					ClassHelper.narrow(_contents[i]);
				current.mofClasses.addElement(_class);
				contentsOfClass(_class);
			} else if (_contents[i]._is_a(AssociationHelper.id())) {
				Association _association =
					AssociationHelper.narrow(_contents[i]);
				current.mofAssociations.addElement(_association);
			} else if (_contents[i]._is_a(TagHelper.id())) {
				Tag _tag = TagHelper.narrow(_contents[i]);
				current.mofTags.addElement(_tag);
			} else if (_contents[i]._is_a(ImportHelper.id())) {
				Import _import = ImportHelper.narrow(_contents[i]);
				current.mofImports.addElement(_import);
			} else if (_contents[i]._is_a(ConstantHelper.id())) {
				Constant _constant = ConstantHelper.narrow(_contents[i]);
				current.mofConstants.addElement(_constant);
			} else if (_contents[i]._is_a(DataTypeHelper.id())) {
				DataType _datatype = DataTypeHelper.narrow(_contents[i]);
				current.mofDataType.addElement(_datatype);
			} else if (_contents[i]._is_a(ConstraintHelper.id())) {
				Constraint _constraint = ConstraintHelper.narrow(_contents[i]);
				current.mofConstraints.addElement(_constraint);
			} else if (_contents[i]._is_a(MofExceptionHelper.id())) {
				MofException _exception =
					MofExceptionHelper.narrow(_contents[i]);
				current.mofExceptions.addElement(_exception);
			} else if (_contents[i]._is_a(PackageHelper.id())) {
				org.omg.mof.Model.Package _nestedPackage =
					PackageHelper.narrow(_contents[i]);
				current.mofPackages.addElement(_nestedPackage);
				contentsOfPackage(_nestedPackage);
			}
		}

		//stocker le contenu de chaque packages 
		String key = _package.ref_mof_id();
		_contentsOfPackages.put(key, current);

	}

	/**
	 * Fill the caches for a class
	 */
	private void contentsOfClass(org.omg.mof.Model.Class _class)
		throws MofError {

		MOFClassCached current = new MOFClassCached();

		//traitement du contenu

		ModelElement[] _contents = _class.contents();

		for (int i = 0; i < _contents.length; i++) {

			//attributes
			if (_contents[i]._is_a(MofAttributeHelper.id())) {
				MofAttribute _attribute =
					MofAttributeHelper.narrow(_contents[i]);
				if (_attribute.visibility() == VisibilityKind.public_vis) {

					if (_attribute.scope() == ScopeKind.classifier_level) {
						if (!_attribute.is_derived() == true)
							current.attributesClassNd.addElement(_attribute);
						current.attributesClass.addElement(_attribute);
					} else {
						if (!_attribute.is_derived() == true)
							current.attributesInstNd.addElement(_attribute);
						current.attributesInst.addElement(_attribute);
					}
				}
			}
			//operations
			else if (_contents[i]._is_a(OperationHelper.id())) {
				Operation _operation = OperationHelper.narrow(_contents[i]);

				if (_operation.visibility() == VisibilityKind.public_vis) {
					if (_operation.scope() == ScopeKind.classifier_level)
						current.operationsClass.addElement(_operation);
					else
						current.operationsInst.addElement(_operation);
				}
			}

			//les rfrences
			else if (_contents[i]._is_a(ReferenceHelper.id())) {
				Reference _reference = ReferenceHelper.narrow(_contents[i]);

				if (_reference.visibility() == VisibilityKind.public_vis)
					current.refrences.addElement(_reference);
			}

			//constants
			else if (_contents[i]._is_a(ConstantHelper.id())) {
				Constant _constant = ConstantHelper.narrow(_contents[i]);

				current.mofConstants.addElement(_constant);
			}
			//contraintes 
			else if (_contents[i]._is_a(ConstraintHelper.id())) {
				Constraint _constraint = ConstraintHelper.narrow(_contents[i]);

				current.mofConstraints.addElement(_constraint);
			}
			//tags
			else if (_contents[i]._is_a(TagHelper.id())) {
				Tag _tag = TagHelper.narrow(_contents[i]);

				current.mofTags.addElement(_tag);
			}
			//exceptions
			else if (_contents[i]._is_a(MofExceptionHelper.id())) {
				MofException _exception =
					MofExceptionHelper.narrow(_contents[i]);

				current.mofExcepion.addElement(_exception);
			}

			//datatypes
			else if (_contents[i]._is_a(DataTypeHelper.id())) {
				DataType _datatype = DataTypeHelper.narrow(_contents[i]);

				current.mofDatatypes.addElement(_datatype);
			}

		}

		//calcul de la cl
		String classId = _class.ref_mof_id();

		_contentsOfClasses.put(classId, current);
	}

	/**
	 * Give the super-classes of a Class
	 */
	public Vector superClassesOrdered(org.omg.mof.Model.Class _class)
		throws MofError {

		Vector superClasses = new Vector();

		//rcuprer les super classes

		GeneralizableElement[] allSuperClasses = _class.all_supertypes();
		for (int i = 0; i < allSuperClasses.length; i++)
			addElement(superClasses, ClassHelper.narrow(allSuperClasses[i]));

		superClasses.addElement(_class);

		return superClasses;
	}

	public MOFCachedProvider(org.omg.mof.Model.Package[] outermost)
		throws NotSet, MofError {
		//init the caches
		init(outermost);
	}
}
