/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.server;

import org.objectweb.modfact.corba.generator.BracketGenerator;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;
import org.omg.mof.Model.ModelElement;


/**
 * This class allows the generation of the repository server main class.
 */
public class GenerationServerApplication extends BracketGenerator {

	/** The Java Helper. */
	private JavaCommon javaHelper;

    /** The logger. */
    protected ModFactLogger _logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.server.GenerationServerApplication");

    /**
     * The Server Application generation.
     * @param package_ The root package.
     * @param class_name_ The class name.
     * @param class_name_application The application class name.
     */
    public void generateServerApplication(org.omg.mof.Model.Package package_, String class_name_, String class_name_application_) throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
        _logger.log(Level.FINE, "Server - beginning of the generation");
        _logger.log(Level.INFO, class_name_application_ + " generation ...");
        head(package_, class_name_application_);
        constructors(package_.name(), class_name_application_);
        startMethod(package_.name(), class_name_);
        mainMethod(class_name_application_);
        outputln("} // end class " + class_name_);
        flushFile();
    }

    /**
     * Server Application generation for the head.
     * @param package_ The package.
     * @param class_name_application_ The application class name.
     */
    public void head(org.omg.mof.Model.Package package_, String class_name_application_)
    	throws org.omg.mof.Reflective.MofError {
        outputln("package " + javaHelper.javaPackage(package_) + ";");
        outputln();
        outputln("public class " + class_name_application_);
        outputln("     extends org.objectweb.modfact.corba.service.cmdline.common.ModfactApplicationBase {");
        outputln();
    }

    /**
     * Server Application generation for the constructors.
     * @param package_name_ The package name.
     * @param class_name_application_ The application class name.
     */
    public void constructors(String package_name_, String class_name_application_) {
        outputln("public " + class_name_application_ + "() {");
        outputln("super(new org.objectweb.util.cmdline.lib.DefaultCommandLine(new java.lang.String [] {\"" + class_name_application_ + "\"},");
        outputln("                                                            new java.lang.String [0],");
        outputln("                                                            new java.lang.String [] {\"Run the " + package_name_ + " Server.\"},");
        outputln("                                                            true));");
        outputln("}");
        outputln();
    }

    /**
     * Server Application generation for the main method.
     * @param class_name_application_ The application class name.
     */
    public void mainMethod(String class_name_application_) throws org.omg.mof.Reflective.MofError {
        outputln("/**");
        outputln(" * The main method");
        outputln(" */");
        outputln("public static void main (java.lang.String [] args) {");
        outputln(class_name_application_ + " application = new " + class_name_application_ + "();");
        outputln("application.runMain(args);");
        outputln("}");
        outputln();
    }

    /**
     * Server Application generation for the start method.
     * @param package_name_ The package name.
     * @param class_name_ The class name.
     */
    public void startMethod(String package_name_, String class_name_) {
        outputln("/**");
        outputln(" * Starts the main function");
        outputln(" */");
        outputln("public int start(java.lang.String[] args) {");
        outputln(class_name_ + " server = new " + class_name_ + "(orb_);");
        outputln("server.run();");
        outputln("return 0;");
        outputln("}");
        outputln();
    }

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#generate()
	 */
	public void generate() throws Exception {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#setInput(org.omg.mof.Model.ModelElement[])
	 */
	public void setInput(ModelElement[] elt) throws Exception {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#setLogger(org.objectweb.modfact.corba.logging.ModFactLogger)
	 */
	public void setLogger(ModFactLogger log) throws Exception {
		_logger = log;

	}

	/**
	 * @param common
	 */
	public void setJavaHelper(JavaCommon common) {
		javaHelper = common;
	}

}
