/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.server;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;

import org.omg.mof.Model.ModelElement;
import org.omg.mof.Model.Package;

import org.objectweb.modfact.corba.generator.ZipGenerator;
import org.objectweb.modfact.corba.helper.IDLCommon;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.helper.XMIImportCommon;

/**
 * An abstract implementation of an XMI Import application.
 * This class provides methods for all XMI importers.
 */
public class ServerGenerator extends ZipGenerator {

	private Package[] input;

	private ModFactLogger logger;

	private MOFCommon mofHelper;
	private JavaCommon javaHelper;
	private IDLCommon idlHelper;
	private XMIImportCommon xmiImportHelper;
	private boolean mappingValid;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new Package[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (Package) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * Default Constructor.
	 */
	public ServerGenerator() {
	}

	/**
	 * @see org.objectweb.modfact.generation.Generator#generate()
	 */
	public void generate() throws IOException {
		logger.log(Level.FINE, "Begin generation of JMI Importer");

		java.util.List redundant_packages = new java.util.ArrayList();
		try {
			for (int i = 0; i < input.length; i++) {
				org.omg.mof.Model.ModelElement contents[] = input[i].contents();
				for (int j = 0; j < contents.length; j++) {
					if (contents[j]._is_a(org.omg.mof.Model.PackageHelper.id())) {
						redundant_packages.add(contents[j].ref_mof_id());
					} else if (contents[j]._is_a(org.omg.mof.Model.ImportHelper.id())) {
						org.omg.mof.Model.Import imp = org.omg.mof.Model.ImportHelper.narrow(contents[j]);
						redundant_packages.add(imp.imported_namespace().ref_mof_id());
					}
				}
			}
		} catch (org.omg.mof.Reflective.MofError mofError) {
			// Nothing to do.
		}

		for (int p = 0; p < input.length; p++) {
			//Traitemement du Package
			if (redundant_packages.contains(input[p].ref_mof_id()))
				continue;
			Package[] param = new Package[1];
			param[0] = input[p];

			try {
				String path = javaHelper.javaPackage(input[p]);
				path = path.replace('.', '/') + "/";
				String className = idlHelper.format1(input[p].name()) + "Server";
				String baseFileName = path + className;

				GenerationServer serverGenerator = new GenerationServer();
				ByteArrayOutputStream entryOutput1 = new ByteArrayOutputStream();
				serverGenerator.setOutput(entryOutput1);
				serverGenerator.setIdlHelper(idlHelper);
				serverGenerator.setJavaHelper(javaHelper);
				serverGenerator.generateServer(param[0], className);
				writeEntry(baseFileName + ".java", entryOutput1);
				entryOutput1.close();
				
				// XMIImportApplication 
				GenerationServerApplication serverApplication = new GenerationServerApplication();
				entryOutput1 = new ByteArrayOutputStream();
				serverApplication.setOutput(entryOutput1);
				serverApplication.setJavaHelper(javaHelper);
				serverApplication.generateServerApplication(param[0], className, className + "Application");
				writeEntry(baseFileName + "Application.java", entryOutput1);
				entryOutput1.close();

				logger.log(Level.FINE, "Server is generated");
			} catch (org.omg.CORBA.UserException userException) {
				logger.log(Level.FATAL, "An error occurs during the generation (" + userException.getMessage() + ")");
			}
		}
		out.finish();
		logger.log(Level.FINE, " Generation is completed without error");
	}

	/**
	 * @param common
	 */
	public void setIdlHelper(IDLCommon common) {
		idlHelper = common;
	}

	/**
	 * @param common
	 */
	public void setJavaHelper(JavaCommon common) {
		javaHelper = common;
	}

	/**
	 * @param common
	 */
	public void setMofHelper(MOFCommon common) {
		mofHelper = common;
	}

	/**
	 * @param common
	 */
	public void setXmiImportHelper(XMIImportCommon common) {
		xmiImportHelper = common;
	}

	/**
	 * @param b
	 */
	public void setMappingValid(boolean b) {
		mappingValid = b;
	}

}
