/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.modfact.corba.service.cmdline.common;

/**
 * Class represnting a modfact base application, which can be an importer or an exporter.
 */
public abstract class ModfactApplicationBase
	extends org.objectweb.util.cmdline.lib.ApplicationBase {

	/** The ORB. */
	protected org.omg.CORBA.ORB orb_;

	/**
	 * Constructor of a ModfactApplicationBase (call to the super class constructor).
	 * @param commandLine The command line manager.
	 */
	protected ModfactApplicationBase(org.objectweb.util.cmdline.lib.DefaultCommandLine commandLine) {
		super(commandLine, false);
		getCommandLine().addOption(new OptionQuiet());
	}

	/**
	 * Obtains the mailing list associated to this development.
	 * @return The mailing list associated to this development.
	 */
	public String getMailingList() {
		return "modfact@lip6.fr";
	}

	/**
	 * Obtains the associated version information.
	 *
	 * @return The associated version information.
	 */
	public String[] getVersionInformation() {
		return new String[] {
			getIdentity(),
			"",
			"Copyright (C) 2000-2003 Laboratoire d'Informatique Paris 6 (LIP6)",
			"Contact: " + getMailingList(),
			"",
			"ModFact is free software; you can redistribute it and/or",
			"modify it under the terms of the GNU Lesser General Public",
			"License as published by the Free Software Foundation; either",
			"version 2 of the License, or any later version.",
			"",
			"ModFact is distributed in the hope that it will be useful,",
			"but WITHOUT ANY WARRANTY; without even the implied warranty of",
			"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU",
			"Lesser General Public License for more details.",
			"",
			"You should have received a copy of the GNU Lesser General Public",
			"License along with this library; if not, write to the Free Software",
			"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA" };
	}

	/**
	 * Executes the main function, i.e.
	 * 1 - Initializes the CORBA::ORB,
	 * 2 - Calls super.runMain().
	 * @param args The command line arguments.
	 */
	public void runMain(String[] args) {
		try {
			// Initializes the CORBA::ORB singleton.
			//args = TheORB.initialize(args);
			orb_ = org.omg.CORBA.ORB.init(args, System.getProperties());
		} catch (org.objectweb.util.api.ExceptionWrapper exc) {
			report_exception(exc.getException());
		} catch (java.lang.Exception exc) {
			report_exception(exc);
		}
		// Calls ApplicationBase.runMain method.
		super.runMain(removeORBArgument(args));
	}

	/**
		* Remove the ORB arguments.
		* @param args The command line arguments.
		* @return The new args without ORB arguments.
		*/
	private String[] removeORBArgument(String[] args) {
		int nb = 0;
		String[] tmp = new String[args.length];
		for (int i = 0; i < args.length; i++) {
			if (!args[i].startsWith("-ORB")) {
				tmp[nb] = args[i];
				nb++;
			} else {
				if (args[i].startsWith("-ORBInitRef"))
					i++;
				i++;
			}
		}
		String[] tmp2 = new String[nb];
		for (int i = 0; i < nb; i++)
			tmp2[i] = tmp[i];
		return tmp2;
	}

}
