/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.common;

import org.objectweb.modfact.corba.generator.BracketGenerator;
import org.omg.mof.Model.ModelElement;
import org.objectweb.modfact.corba.helper.IDLCommon;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.helper.StringHelper;
import org.objectweb.modfact.corba.helper.XMIImportCommon;
import org.objectweb.modfact.corba.logging.ModFactLogger;
import org.objectweb.modfact.corba.logging.Level;

public abstract class AbstractXMIIOGenerator extends BracketGenerator {

	/** The IDL Helper. */
	protected IDLCommon idlHelper;
	
	/** The MOF Helper. */
	protected MOFCommon mofHelper;
	
	/** The Java Helper. */
	protected JavaCommon javaHelper;
	
	/** The XMI Import Helper. */
	protected XMIImportCommon xmiImpHelper;

	/** The IDL2Java mapping compliance flag. */
	protected boolean mappingValid;

	/** The logger. */
	protected ModFactLogger _logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiimport.GenerationXMIImportApplication");

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#setInput(org.omg.mof.Model.ModelElement[])
	 */
	public void setInput(ModelElement[] elt) throws Exception {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#setLogger(java.util.logging.Logger)
	 */
	public void setLogger(ModFactLogger log) throws Exception {
		_logger = log;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#generate()
	 */
	public void generate() throws Exception {
		// TODO Auto-generated method stub
	}

	/**
	 * Annotation processing.
	 * @param annotation_ The annotation to print.
	 */
	public void annotationTemplate(String annotation_) throws org.omg.mof.Reflective.MofError {
		/* !!! Management of the carriage return, the tabulations (and the character \s)  */
		if (annotation_.trim().length() > 0) {
			String annotation = "/** \n" + StringHelper.replaceAll(annotation_.trim(), "\\\\n", "\n") + "\n/";
			annotation = StringHelper.replaceAll(annotation, "\\\\s", " ");
			StringBuffer margin = new StringBuffer();
			for (int i = 0; i < level; i++)
				margin.append(TABULATION);
			annotation = StringHelper.replaceAll(annotation, "\n", "\n" + margin.toString() + " *");
			outputln(annotation);
		}
	}

	/**
	 * Get the class name.
	 * @param package_name_ The root package name.
	 * @param suffix_ The suffix (i.e. XMIImport, XMIExport)
	 * @return The class name.
	 */
	protected String getClassName(String package_name_, String suffix_) {
		_logger.log(Level.FINE, "getClassName(" + package_name_ + ")");
		String packageName1 = idlHelper.format1(package_name_);
		String className = filename;
		if (filename.trim().length() > 0) {
			while (className.indexOf(java.io.File.separator) != -1)
				className = className.substring(className.indexOf(java.io.File.separator) + 1);
			if (className.lastIndexOf(".java") != -1)
				className = className.substring(0, className.lastIndexOf(".java"));
		} else {
			className = packageName1 + suffix_;
		}
		return className;
	}

	/**
	 * @param common
	 */
	public void setIdlHelper(IDLCommon common) {
		idlHelper = common;
	}

	/**
	 * @param common
	 */
	public void setJavaHelper(JavaCommon common) {
		javaHelper = common;
	}

	/**
	 * @param b
	 */
	public void setMappingValid(boolean b) {
		mappingValid = b;
	}

	/**
	 * @param common
	 */
	public void setMofHelper(MOFCommon common) {
		mofHelper = common;
	}

	/**
	 * @param common
	 */
	public void setXmiImpHelper(XMIImportCommon common) {
		xmiImpHelper = common;
	}

}
