/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.exporter;

/**
 * An abstract implementation of an XMI Export application.
 * This class provides methods for all XMI exporters.
 */
public abstract class XMIExportApplication
	extends org.objectweb.modfact.corba.service.cmdline.common.ModfactApplicationBase
	implements org.objectweb.modfact.corba.service.cmdline.xmiio.XMIExportApplication {

	/** The PrintIOR flag. */
	protected boolean printIOR_ = false;

	/** The IOR file. */
	protected String iorFile_ = "out.ior";

	/** The output file. */
	protected String outputFile_ = new String();

	/** The output directory. */
	protected String outputDir_ = new String();

	/** The name in the name service. */
	protected String nameService_ = new String();

	/** The DTD uri. */
	protected String dtdFile_ = new String();

	/**
	 * XMIImportApplication constructor.
	 * @param package_name The package name.
	 * @param class_name The class name.
	 * @param default_ior_file The default IOR file.
	 */
	protected XMIExportApplication(String package_name, String class_name) {
		this(
			new org.objectweb.util.cmdline.lib.DefaultCommandLine(
				new String[] { class_name },
				new String[0],
				new String[] { "Export a model (" + package_name + ") to an XMI file." },
				true));
	}

	/**
	 * XMIImportApplication constructor.
	 * @param commandLine The command line manager.
	 */
	private XMIExportApplication(org.objectweb.util.cmdline.lib.DefaultCommandLine commandLine) {
		super(commandLine);
		// Creates the XMIExportApplication options
		commandLine.addOption(new org.objectweb.modfact.corba.service.cmdline.io.OptionOutput(this));
		commandLine.addOption(new org.objectweb.modfact.corba.service.cmdline.xmiio.OptionDTDFile(this));
		commandLine.addOption(new org.objectweb.modfact.corba.service.cmdline.xmiio.OptionIORFile(this));
		commandLine.addOption(new org.objectweb.modfact.corba.service.cmdline.xmiio.OptionPrintIOR(this));
	}

	/**
	 * Set the IOR file.
	 * @param filename The IOR filename.
	 */
	public void setIORFile(String filename) {
		iorFile_ = filename;
	}

	/**
	 * Set the output file.
	 * @param filename The output filename.
	 */
	public void setOutput(String filename) {
		outputFile_ = filename;
	}

	/**
	 * Set the output directory.
	 * @param directory The output directory.
	 */
	public void setOutputDir(String directory) {
		outputDir_ = directory;
	}

	/**
	 * Set the PrintIOR flag.
	 * @param print TRUE to display the IOR on the console.
	 */
	public void setPrintIOR(boolean print) {
		printIOR_ = print;
	}

	/**
	 * Set the name in the Name Service.
	 * @param nameService The name in the Name Service.
	 */
	public void setNameService(String nameService) {
		nameService_ = nameService;
	}

	/**
	 * Set the DTD file.
	 * @param filename The DTD filename.
	 */
	public void setDTDFile(String filename) {
		dtdFile_ = filename;
	}

}
