/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.api;


import javax.jmi.model.*;

import org.objectweb.modfact.jmi.generator.BracketGenerator;
import org.objectweb.modfact.jmi.helper.*;


/**
 * @author Xavier
 *
 */
public abstract class CommonGenerator extends BracketGenerator {
	
	
	public void attributeTemplate(Attribute attribute) {
		if (attribute.getVisibility().equals(VisibilityKindEnum.forName("public_vis"))) {
			//<<ANNOTATION TEMPLATE>>
			
			//----------------------- Accessor Operations
			structuralFeatureAccessor(attribute);
		
			//-------------------------  Mutator Operations
			structuralFeatureMutator(attribute);

		}
	}

	public void referenceTemplate(Reference reference)   {
		if (reference.getVisibility().equals(VisibilityKindEnum.forName("public_vis"))) {
			//<<ANNOTATION TEMPLATE>>

			//----------------------- Accessor Operations
			structuralFeatureAccessor(reference);
		
			//-------------------------  Mutator Operations
			structuralFeatureMutator(reference);	
			
		}
	}
	
	public void structuralFeatureAccessor(StructuralFeature f) {
		
		String fType = JMIProvider.typeTemplate(f);
		String accessorName = JMIProvider.jmiAccessorName(f);
		
		outputln(
				"public "
					+ fType
					+ " "
					+ accessorName
					+ "() throws javax.jmi.reflect.JmiException;");

	}
	
	
	public void structuralFeatureMutator(StructuralFeature f) {
		
		String mutatorName = JMIProvider.jmiMutatorName(f);
		String fType = JMIProvider.typeTemplate(f);
				
		if ( f.getMultiplicity().getUpper() == 1  && f.isChangeable() ) {
			outputln(
				"public void "
					+ mutatorName
					+ "("
					+ fType
					+ " newValue) throws javax.jmi.reflect.JmiException;");
		}		
	}
	

	///////////////// Operation //////////////////////////////////
	public void operationTemplate(Operation operation) {
		if (operation.getVisibility().equals(VisibilityKindEnum.forName("public_vis"))) {
			//<<JAVADOCS TEMPLATE>>
			//Get Parameters and exceptions
			Parameter[] parameters = MofHelper.parametersOf(operation);
			MofException[] exceptions = MofHelper.exceptionsOf(operation);
			Parameter returnParameter = null;
			
			for(int i=0; i<parameters.length; i++) {
				if(parameters[i].getDirection().equals(DirectionKindEnum.RETURN_DIR)) {
					returnParameter = parameters[i];
				}
			}

			if (returnParameter==null) output("\tpublic void ");
			else output("public "
				+JMIProvider.typeTemplate(returnParameter)
				+" " );
			
			output(operation.getName()+"(");
			
			boolean comma = false;
			for (int i=0 ; i <parameters.length ; i++) {		
				DirectionKind dir = parameters[i].getDirection();
				if ( !dir.equals(DirectionKindEnum.RETURN_DIR)) {
				  if (comma) output(",");
				  output( JMIProvider.typeTemplate(parameters[i]) +" " +parameters[i].getName());
				  comma = true;
				}
			}
			output(") throws ");
			
			//for each exception
			for (int i=0 ; i < exceptions.length ; i++) {
				output(exceptions[i].getName()+",");
			}
			outputln("javax.jmi.reflect.JmiException;");
		}
	}


	
	
	
	/////////////////// DATA TYPE //////////////////
	public void dataTypeTemplates(Namespace ns) {
		//for each StructType directly contained by the package/class
		DataType[] containedDataTypes =
			MofHelper.datatypesOf(ns, false);
		for (int i = 0; i < containedDataTypes.length; i++) {
			if ( containedDataTypes[i] instanceof StructureType) {
				structTemplate((StructureType)containedDataTypes[i]);
			} 
		}	
	}
	
	public void structTemplate(StructureType t) {

			output(
				"public "
					+ JMIProvider.jmiDataTypeQualifiedName(t)
					+ " create"
					+ t.getName()
					+ " (");
									
			StructureField[] fields = MofHelper.structureFieldsOf(t);			
			for (int j = 0; j < fields.length; j++) {
				output( JMIProvider.typeTemplate(fields[j]) );
				//attribute type
				output(" ");
				output(fields[j].getName()); //attribute name
				if (j != fields.length - 1)
					output(" , ");
			}
			outputln(") throws javax.jmi.reflect.JmiException;");	
	
	}
	
	public void constantTemplate(Constant constant) {
		//<<ANNOTATION TEMPLATE>>
		// TODO 
		outputln(
			"public final "
				+ "String"
				+ " "
				+ constant.getName()
				+ "= \""
				+ constant.getValue()
				+ "\";");
	}
	
	
}
