/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.api;

import javax.jmi.model.Association;
import javax.jmi.model.GeneralizableElement;
import javax.jmi.model.Import;
import javax.jmi.model.ModelElement;
import javax.jmi.model.MofClass;
import javax.jmi.model.MofPackage;
import javax.jmi.model.VisibilityKindEnum;

import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.logging.ModFactLogger;


/**
 * @author Xavier
 *
 */
public class PackageInterfaceGenerator extends CommonGenerator {

	MofPackage[] input;

    ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofPackage[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofPackage) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * @see fr.lip6.modfact.generation.Generator#generate()
	 */
	public void generate() {
		//Tag Import todo
		MofPackage _package = input[0];
		//javadocsTemplate(_package);


        outputln(
			"package " + JMIProvider.qualifierOf(_package) + ";");

        output(
			"public interface "
				+ JMIProvider.jmiPackageExtentName(_package)
				+ "Package extends ");
		GeneralizableElement[] generalizableElement = (GeneralizableElement[])_package.getSupertypes().toArray(new GeneralizableElement[0]);
		if (generalizableElement.length == 0)
            outputln("javax.jmi.reflect.RefPackage");
		else {
			for (int i = 0; i < generalizableElement.length; i++) {
                output(
					JMIProvider.jmiPackageQualifiedName(
						(MofPackage)generalizableElement[i] )
						+ "Package ");
				if (i != generalizableElement.length)
                    output(",");
			}
		}
        outputln(" { ");
		
		//for each StructType 
		dataTypeTemplates(_package);		

		////////////////  Package Import /////////////////
		Import[] imports = MofHelper.importsOfPackage(_package);
		for (int i = 0; i < imports.length; i++) {
			if ( VisibilityKindEnum.forName("public_vis").equals(imports[i].getVisibility())
				&& imports[i].isClustered() 
				&& imports[i].getImportedNamespace() instanceof MofPackage
			) { 
				MofPackage importedPackage = (MofPackage) imports[i].getImportedNamespace();
                    outputln(
						"\tpublic "
							+ JMIProvider.jmiPackageQualifiedName(importedPackage)
							+ "Package get"
							+ JMIProvider.jmiImportName(imports[i])
							+ "();");
			}
		}

		//for each contained package where visibility = public_vis
		MofPackage[] containedPackages =
			MofHelper.packagesOfPackage(_package);
		for (int i = 0; i < containedPackages.length; i++) {
			if (containedPackages[i]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))
			 	) {
					
                outputln(
					"\tpublic "
						+ JMIProvider.jmiPackageQualifiedName(containedPackages[i])
						+ "Package get"
						+ JMIProvider.jmiPackageExtentName(containedPackages[i])
						+ "();");
			}
		}

		//for each contained class where visibility = public_vis
		MofClass[] containedClasses =
			MofHelper.classesOfPackage(_package);
		for (int i = 0; i < containedClasses.length; i++) {
			if (containedClasses[i]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))) {
                outputln(
					"\tpublic "
						+ JMIProvider.jmiClassQualifiedName(containedClasses[i])
						+ "Class get"
						+ JMIProvider.jmiClassName(containedClasses[i])
						+ "();");
			}
		}

		//for each contained association where visibility = public_vis
		Association[] containedAssociations =
			MofHelper.associationsOfPackage(_package);
		for (int i = 0; i < containedAssociations.length; i++) {
			if (containedAssociations[i]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis")) ) {
                outputln(
					"\tpublic "
						+ JMIProvider.jmiAssociationQualifiedName(containedAssociations[i])
						+ " get"
						+ JMIProvider.jmiAssociationName(containedAssociations[i])
						+ "();");
			}
		}

        outputln("}");
		flushFile();
	}


}
