/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.helper;

import javax.jmi.model.*;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 */
/**
 *
 */
/**
 * @author Xavier, Prawee
 *  
 */
public class JMIProvider {

	public static boolean considers_idl_substitute_name = true;

	// ignore package prefix
	public static String shortQualifierOf(ModelElement _elem) {
		Namespace ns = null;

		if (_elem instanceof Namespace) {
			ns = (Namespace) _elem;
		} else {
			ns = _elem.getContainer();
		}
		if (ns == null)
			return "";

		String res = null;

		while (true) {
			if (ns instanceof MofPackage) {
				// ignore classes that contain datatypes
				res =
					 packageNameFormat(ns.getName())
						+ ((res == null) ? "" : ("." + res));
			}
			Namespace ns2 = ns.getContainer();
			if (ns2 == null)
				break;
			ns = ns2;
		}
		return res;
	}

	// include package prefix
	public static String qualifierOf(ModelElement _elem) {

		return packagePrefixOf(_elem) + shortQualifierOf(_elem);

	}

	// return package prefix with dot
	public static String packagePrefixOf(ModelElement _elem) {
		Namespace ns = null;

		if (_elem instanceof MofPackage) {
			ns = (Namespace) _elem;
		} else {
			ns = _elem.getContainer();
		}
		if (ns == null)
			return "";

		while (true) {
			Namespace ns2 = ns.getContainer();
			if (ns2 == null)
				break;
			ns = ns2;
		}

		Iterator it =
			((ModelPackage) ns.refImmediatePackage())
				.getTag()
				.refAllOfClass()
				.iterator();
		while (it.hasNext()) {
			Tag t = (Tag) it.next();
			if ((t.getTagId().equals("javax.jmi.packagePrefix")
				|| t.getTagId().equals("com.urep.jcr.java_prefix"))
				&& t.getElements().contains(ns)) {
				return t.getValues().get(0) + ".";
			}
		}
		return "";
	}

	public static String jmiPackageExtentName(MofPackage p) {
		String stName = findSubstituteName(p);
		if (stName != null)
			return stName;
		return classNameFormat(p.getName());
	}
	
	/*
	 * Alias for PackageExtentName
	 * */
	public static String jmiPackageName(MofPackage p) {
		return jmiPackageExtentName(p);
	}	

	public static String jmiPackageQualifiedName(MofPackage p) {
		return qualifierOf(p) + "." + jmiPackageExtentName(p);
	}

	public static String jmiClassName(MofClass clazz) {
		String stName = findSubstituteName(clazz);
		if (stName != null)
			return stName;
		return classNameFormat(clazz.getName());
	}

	// if not found, return null
	public static String findSubstituteName(ModelElement m) {
		Iterator it =
			((ModelPackage) m.refImmediatePackage())
				.getTag()
				.refAllOfClass()
				.iterator();
		while (it.hasNext()) {
			Tag t = (Tag) it.next();
			if ((t.getTagId().equals("javax.jmi.substituteName")
				|| t.getTagId().equals("com.urep.jcr.java_substitute_name")
				|| (considers_idl_substitute_name
					&& t.getTagId().equals("org.omg.mof.idl_substitute_name")))
				&& t.getElements().contains(m)) {
				return (String) t.getValues().get(0);
			}
		}
		return null;
	}

	public static String jmiClassQualifiedName(MofClass clazz) {
		return qualifierOf(clazz) + "." + jmiClassName(clazz);
	}

	public static String jmiAssociationName(Association a) {
		String stName = findSubstituteName(a);
		if (stName != null)
			return stName;
		return classNameFormat(a.getName());
	}

	public static String jmiAssociationQualifiedName(Association association) {
		return qualifierOf(association) + "." + jmiAssociationName(association);
	}

	public static String jmiPrimitiveTypeName(PrimitiveType _type) {
		String typeName = _type.getName().toLowerCase();
		if (typeName.equals("string")) {
			return "String";
		} else if (typeName.equals("boolean")) {
			return "boolean";
		} else if (typeName.equals("integer") || typeName.equals("long")) {
			return "int";
		} else if (typeName.equals("float")) {
			return "float";
		} else if (typeName.equals("double")) {
			return "double";
		} else {
			throw new RuntimeException(
				"unknown primitive type: " + _type.getName());
		}
	}

	public static String jmiNullablePrimitiveTypeName(PrimitiveType _type) {
		String typeName = _type.getName().toLowerCase();
		if (typeName.equals("string")) {
			return "java.lang.String";
		} else if (typeName.equals("boolean")) {
			return "java.lang.Boolean";
		} else if (typeName.equals("integer") || typeName.equals("long")) {
			return "java.lang.Integer";
		} else if (typeName.equals("float")) {
			return "java.lang.Float";
		} else if (typeName.equals("double")) {
			return "java.lang.Double";
		} else {
			throw new RuntimeException(
				"unknown primitive type: " + _type.getName());
		}
	}

	public static String jmiDataTypeQualifiedName(DataType _type) {
		if (_type instanceof PrimitiveType) {
			return jmiNullablePrimitiveTypeName((PrimitiveType) _type);
		}
		return qualifierOf(_type) + "." + _type.getName();
	}

	public static String jmiImportName(Import _import) {
		MofPackage p = (MofPackage) _import.getImportedNamespace();
		return jmiPackageExtentName(p);
	}

	public static String jmiClassifierQualifiedName(Classifier _classifier) {
		if (_classifier instanceof MofClass)
			return jmiClassQualifiedName((MofClass) _classifier);

		if (_classifier instanceof DataType)
			return jmiDataTypeQualifiedName((DataType) _classifier);

		throw new RuntimeException("invalide parameter:" + _classifier);
	}

	public static String jmiFormat1(String name) {
		return noUnderscoreName(name, true);
	}
    
    public static java.util.Vector splitAndDelete(String _input) {
        java.util.Vector parts = new java.util.Vector();
        String delim = "-_    .";
        java.util.StringTokenizer _sstring = new java.util.StringTokenizer(_input, delim);
        while (_sstring.hasMoreTokens()) {
            String part = _sstring.nextToken();
            int i = 0;
            int j = 0;
            boolean isPart = false;
            boolean beginPart = true;
            while (i < part.length()) {
                if (beginPart) {
                    if ((Character.isDigit(part.charAt(i))) || (Character.isUpperCase(part.charAt(i))))
                        i++;
                    else
                        beginPart = false;
                } else {
                    if ((Character.isDigit(part.charAt(i))) || (Character.isLowerCase(part.charAt(i))))
                        i++;
                    else
                        isPart = true;
                }
                if (isPart) {
                    parts.addElement(part.substring(j, i));
                    j = i;
                    isPart = false;
                    beginPart = true;
                }
                if (i == j)
                    i++; // if the character is not specified in the filter
            }
            if (i != j)
                parts.addElement(part.substring(j, i));
        }
        return parts;
    }
    
    public static String jmiFormat2(String name) {
        java.util.Vector s1 = splitAndDelete(name);
        String buffer = " ";
        for (int i = 0; i < s1.size(); i++) {
            String s2 = s1.elementAt(i).toString().toLowerCase();
            buffer = buffer.concat(s2);
            if (i != (s1.size() - 1))
                buffer = buffer.concat("_");
        }
        return buffer.trim();
    }

	// StructureField, Attribute, or Reference
	public static String jmiAccessorName(TypedElement attribute) {

		String accessorName = jmiFormat1(attribute.getName());
		Classifier t = attribute.getType();

		if ((t instanceof PrimitiveType)
			&& t.getName().equalsIgnoreCase("boolean")
			&& ((attribute instanceof StructureField)
				|| ((Attribute) attribute).getMultiplicity().getUpper() == 1)) {

			if (attribute.getName().toLowerCase().startsWith("is"))
				return attribute.getName();
			else
				return "is" + accessorName;
		} else
			return "get" + accessorName;
	}

	// Attribute or Reference
	public static String jmiMutatorName(StructuralFeature attribute) {

		Classifier t = attribute.getType();

		if ((t instanceof PrimitiveType)
			&& t.getName().equalsIgnoreCase("boolean")
			&& attribute.getMultiplicity().getUpper() == 1
			&& attribute.getName().toLowerCase().startsWith("is")) {

			return "set" + jmiFormat1(attribute.getName().substring(2));

		} else
			return "set" + jmiFormat1(attribute.getName());
	}

	public static String typeTemplate(TypedElement elem) {
		String javaType = null;
		Classifier type = elem.getType();

		if (MofHelper.isSinglePrimitive(elem)) {
			javaType = JMIProvider.jmiPrimitiveTypeName((PrimitiveType) type);

		} else if (
			(elem instanceof StructuralFeature)
				&& ((StructuralFeature) elem).getMultiplicity().getUpper() != 1) {

			javaType =
				(((StructuralFeature) elem).getMultiplicity().isOrdered())
					? "java.util.List"
					: "java.util.Collection";

		} else if (
			(elem instanceof Parameter)
				&& ((Parameter) elem).getMultiplicity().getUpper() != 1) {

			javaType =
				(((Parameter) elem).getMultiplicity().isOrdered())
					? "java.util.List"
					: "java.util.Collection";

		} else {
			javaType = JMIProvider.jmiClassifierQualifiedName(type);
		}

		if ((elem instanceof Parameter)
			&& (((Parameter) elem)
				.getDirection()
				.equals(DirectionKindEnum.INOUT_DIR)
				|| ((Parameter) elem).getDirection().equals(
					DirectionKindEnum.OUT_DIR))) {

			javaType = javaType + "[]";
		}
		return javaType;
	}

	/**
	 * 4.7.1 rule for generating identifier
	 * 
	 * @param name
	 * @return
	 */
	public static String packageNameFormat(String name) {
		return name.replaceAll("[^a-zA-Z0-9]", "").toLowerCase();
	}

	/**
	 * 4.7.1 rule for generating identifier
	 * 
	 * @param name
	 * @return
	 */
	public static String classNameFormat(String name) {
		return noUnderscoreName(name, true);
	}
	

	/**
	 * 4.7.1 rule for generating identifier
	 * 
	 * @param name
	 * @return
	 */
	public static String attributNameFormat(String name) {
		return noUnderscoreName(name, false);
	}

	/**
	 * 4.7.1 rule for generating identifier
	 * 
	 * @param name
	 * @return
	 */
	public static String operationNameFormat(String name) {
		return noUnderscoreName(name, false);
	}

	/**
	 * 4.7.1 rule for generating identifier
	 * 
	 * @param name
	 * @return
	 */
	public static String constantNameFormat(String name) {
		return underscoreName(name);
	}

	/**
	 * 4.7.1 rule for generating identifier
	 * 
	 * @param name
	 * @return
	 */
	public static String enumerationNameFormat(String name) {
		return underscoreName(name);
	}
	/*
	 *  example
	 *    input:  "This is a     CAT!!!"
	 *    output: "THIS_IS_A_CAT"
	 * */
	public static String underscoreName(String name) {
		String res = "";
		String decomposed = decompose(name);
		StringTokenizer st = new StringTokenizer(decomposed);
		if(st.countTokens()==0) {
			throw new RuntimeException("Invalide name '" +name +"'");
		}
		res = st.nextToken().toUpperCase();		
		while (st.hasMoreElements()) {			
			res = res +"_" +st.nextToken().toUpperCase();
		}
		return res;		
	}
	
	public static String noUnderscoreName(String name, boolean beginsWithUpper) {
		String res = "";
		String decomposed = decompose(name);
		StringTokenizer st = new StringTokenizer(decomposed);
		boolean isFirstToken = true;
		while (st.hasMoreElements()) {
			char[] tmp =	st.nextToken()
							  .toLowerCase()
							  .toCharArray();
			if (   (!isFirstToken) 
				|| beginsWithUpper ) {
				tmp[0] = Character.toUpperCase(tmp[0]);
				isFirstToken = false;
			}				
			res = res + new String(tmp);
		}
		if(res.length()==0) {
			throw new RuntimeException("Invalide name '" +name +"'");
		}
		return res;		
	}

	/* example
		input:  "abcDEFGhi23JKL45"
		output: "abc DEF Ghi23 JKL45"
	 * */
	public static String decompose(String input) {
		// eliminate special chars	
		String r = input.replaceAll("[^a-zA-Z0-9]", " ");
		// detect upper-lower transition : beginning of word
		r = r.replaceAll("[A-Z][a-z]", " $0");		
		// detect lower-upper transition : beginning of ALL-CAPITAL word 
		Matcher m = Pattern.compile("[a-z0-9][A-Z]").matcher(r);
		StringBuffer sb = new StringBuffer();
		while (m.find()) {
			String matched = m.group();
			m.appendReplacement(sb
				, matched.charAt(0) +" " +matched.charAt(1) );
		}
		m.appendTail(sb);		
		r = sb.toString();		
		return r;
	}	
	
	
}
