package org.objectweb.modfact.jmi.helper;

import java.util.StringTokenizer;

/**
 * This class provides several functions to manipulate Strings.
 */
public class StringHelper {

    /**
     * Replace the first occurence of the given string with the given replacement.
     * @param original The original String to modify.
     * @param to_replace The substring of the string to search and replace.
     * @param new_value The replacement String.
     * @return The modified String.
     */
    public static String replaceFirst(String original, String to_replace, String new_value) {
        int index = original.indexOf(to_replace);
        if (index != -1) {
            return original.substring(0, index) + new_value + original.substring(index+to_replace.length());
        } else {
            return original;
        }
    }

    /**
     * Replace all the occurences of the given string with the given replacement.
     * @param original The original String to modify.
     * @param to_replace The substring of the string to search and replace.
     * @param new_value The replacement String.
     * @return The modified String.
     */
    public static String replaceAll(String original, String to_replace, String new_value) {
        StringBuffer newString = new StringBuffer();
        int indexBegin = 0;
        int index = original.indexOf(to_replace, indexBegin);
        while (index != -1) {
            newString.append(original.substring(indexBegin, index));
            newString.append(new_value);
            indexBegin = index + to_replace.length();
            index = original.indexOf(to_replace, indexBegin);
        }
        newString.append(original.substring(indexBegin));
        return newString.toString();
    }
    
    /**
     * Parse a String and test if it corresponds to the value. This value can be multiple and separate by '|', for example,
     * matchesValues(to_parse, new String("value1|value2|value3")).
     * @param to_parse The substring of the string to search and replace.
     * @param value The replacement String.
     * @return TRUE if the parsed string corresponds to one of the values.
     */
    public static boolean matchesValues(String to_parse, String value) {
        StringTokenizer token = new StringTokenizer(value, "|");
        while (token.hasMoreTokens()) {
            if (to_parse.equals(token.nextToken()))
                return true;
        }
        return false;
    }
    
    /**
      * Parse a String and test if it ends with the value. This value can be multiple and separate by '|', for example,
      * matchesValues(to_parse, new String("end1|end2|end3")).
      * @param to_parse The substring of the string to search and replace.
      * @param value The replacement String.
      * @return TRUE if the parsed string ends with on of the values.
      */
    public static boolean endsWithValues(String to_parse, String value) {
        StringTokenizer token = new StringTokenizer(value, "|");
        while (token.hasMoreTokens()) {
            if (to_parse.endsWith(token.nextToken()))
                return true;
        }
        return false;
    }

    /**
     * Remove all the occurences which match the pattern.
     * @param original The original String to modify.
     * @param pattern The pattern string to search and remove.
     *     This pattern could be composed by character, range (a-z),
     *     and start with "[" or "[^" (negation) and stop with "]".
     * @return The modified String.
     */
    public static String removeAll(String original, String pattern) {
        // Builds pattern to search.
        boolean negation = false;
        String toSearch = pattern;
        if (toSearch.startsWith("[") && toSearch.endsWith("]")) {
            toSearch = toSearch.substring(1, toSearch.length()-1);
            if (toSearch.startsWith("^")) {
                toSearch = toSearch.substring(1);
                negation = true;
            }
        }
        // Parses pattern.
        java.util.List characters = new java.util.ArrayList();
        for (int i=0 ; i<toSearch.length() ; i++) {
            if (i>1 && toSearch.charAt(i-1)=='-') {
                for (char c=toSearch.charAt(i-2) ; c<=toSearch.charAt(i) ; c++) {
                    characters.add(new Character(c));
                }
            } else if (toSearch.charAt(i) != '-')
                characters.add(new Character(toSearch.charAt(i)));
        }
        // Parses original string.
        StringBuffer newString = new StringBuffer();
        for (int i=0 ; i<original.length() ; i++) {
            char c = original.charAt(i);
            if (!negation && !characters.contains(new Character(c)))
                newString.append(c);
            else if (negation && characters.contains(new Character(c)))
                newString.append(c);
        }
        return newString.toString();
    }

}
