/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.helper;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.jmi.model.MofClass;
import javax.jmi.model.MofPackage;

/**
 * @author Pierre Carpentier
 *
 */
public class XMIIOHelper {

	/** The hashtable which contains the subclasses for each class. */
	public static java.util.Hashtable _classes_sub_classes = new java.util.Hashtable();

	/**
	 * Gets all the subclasses of a class.
	 * @param _class The super class.
	 * @return An array which contains all the subclasses of the class.
	 */
	public static javax.jmi.model.MofClass[] subClasses(javax.jmi.model.Classifier _class) {
		if (_classes_sub_classes.size() == 0) {
			searchAllSubClasses(((MofPackage) _class.getContainer()).refClass().refAllOfType());
		}
		if (_classes_sub_classes.containsKey(JMIProvider.jmiClassifierQualifiedName(_class))) {
			java.util.Vector vector =
				(java.util.Vector) _classes_sub_classes.get(JMIProvider.jmiClassifierQualifiedName(_class));
			javax.jmi.model.MofClass[] cls = new javax.jmi.model.MofClass[vector.size()];
			for (int i = 0; i < cls.length; i++)
				cls[i] = (javax.jmi.model.MofClass) vector.elementAt(i);
			return cls;
		} else {
			return new javax.jmi.model.MofClass[0];
		}
	}

	/**
	 * Search all subclasses of a package.
	 * Parses all the classes of a package, and build an hashtable which contains the subclasses for each class.
	 * This vector is used by the subClasses method.
	 * @param _packs The packages.
	 */
	protected static void searchAllSubClasses(java.util.Collection _packs) {
		// For all packages
		Iterator it = _packs.iterator();
		while (it.hasNext()) {
			javax.jmi.model.MofClass[] classes = MofHelper.classesOfPackage((MofPackage) it.next());
			// For all classes of the package
			for (int j = 0; j < classes.length; j++) {
				javax.jmi.model.MofClass cls = (javax.jmi.model.MofClass) classes[j];
				java.util.List _sup_classes = allSupertypes(cls);
				// For all supertypes of the class
				Iterator itSup = _sup_classes.iterator();
				while (itSup.hasNext()) {
					javax.jmi.model.Classifier sup = (javax.jmi.model.Classifier) itSup.next();
					addHashtableValue(_classes_sub_classes, JMIProvider.jmiClassifierQualifiedName(sup), cls);
				}
			}
		}
	}

	/**
	 * Gets the references of an association.
	 * @param _association The association.
	 * @return The references.
	 */
	public static javax.jmi.model.Reference[] getReferences(javax.jmi.model.Association _association) {
		java.util.Vector _temp = new java.util.Vector();
		// Get the package which contains the association
		MofPackage _package = (MofPackage) _association.getContainer();
		// Get all classes
		javax.jmi.model.MofClass _classes_of_package[] = MofHelper.classesOfPackage(_package);
		// Get the AssociationEnd
		javax.jmi.model.AssociationEnd[] _association_ends = MofHelper.associationEndsOfAssociation(_association);
		for (int i = 0; i < _classes_of_package.length; i++) {
			javax.jmi.model.MofClass _class = (javax.jmi.model.MofClass) _classes_of_package[i];
			// already processed
			javax.jmi.model.Reference _references[] = MofHelper.referencesOfClass(_class, false);
			for (int j = 0; j < _references.length; j++) {
				javax.jmi.model.Reference _reference = (javax.jmi.model.Reference) _references[j];
				javax.jmi.model.AssociationEnd _association_end = _reference.getReferencedEnd();
				if (_association_end == _association_ends[0] || _association_end == _association_ends[1]) {
					_temp.addElement(_reference);
				}
			}
		}
		javax.jmi.model.Reference[] _referencees = new javax.jmi.model.Reference[_temp.size()];
		for (int i = 0; i < _referencees.length; i++)
			_referencees[i] = (javax.jmi.model.Reference) _temp.elementAt(i);
		return _referencees;
	}

	/**
	 * Gets the opposite association end of an association end.
	 * @param _association_end The association end.
	 * @return The opposite association end.
	 */
	public static javax.jmi.model.AssociationEnd otherAssociationEnd(javax.jmi.model.AssociationEnd _association_end) {
		javax.jmi.model.Association asso = (javax.jmi.model.Association) (_association_end.getContainer());
		javax.jmi.model.AssociationEnd[] ends = MofHelper.associationEndsOfAssociation(asso);
		if (ends[0] == _association_end)
			return ends[1];
		else
			return ends[0];
	}

	/**
	 * Gets all the supertype of a classifier.
	 * @param classifier The classifier.
	 * @return The supertypes.
	 */
	public static java.util.List allSupertypes(javax.jmi.model.Classifier classifier) {
		java.util.List _supertypes = classifier.getSupertypes();
		java.util.List _all_supertypes = new java.util.ArrayList(_supertypes);
		java.util.Iterator it = _supertypes.iterator();
		while (it.hasNext()) {
			javax.jmi.model.Classifier supertype = (javax.jmi.model.Classifier) it.next();
			java.util.List _super_supertypes = allSupertypes(supertype);
			java.util.Iterator itSuper = _super_supertypes.iterator();
			while (itSuper.hasNext()) {
				Object s = itSuper.next();
				if (!_all_supertypes.contains(s))
					_all_supertypes.add(s);
			}
		}
		return _all_supertypes;
	}

	/**
	 * Replace the first occurence of the given string with the given replacement.
	 * @param original The original String to modify.
	 * @param to_replace The substring of the string to search and replace.
	 * @param new_value The replacement String.
	 * @return The modified String.
	 */
	public static String replaceFirst(String original, String to_replace, String new_value) {
		int index = original.indexOf(to_replace);
		if (index != -1) {
			return original.substring(0, index) + new_value + original.substring(index + to_replace.length());
		} else {
			return original;
		}
	}

	public static String format1FirstMin(String toFormat) {
		String format1 = JMIProvider.jmiFormat1(toFormat);
		return format1.substring(0, 1).toLowerCase() + format1.substring(1);
	}

	/**
	 * Tests if the _input is a Java keyword.
	 * @param _input The word to test.
	 * @return The correct word to prevent Java conflicts.
	 */
	public static String testJavaConflict(String _input) {
		if (endsWithValues(_input, "Package|Helper|Holder|Operations|POA|POATie")
			|| matchesValues(
				_input,
				"abstract|default|if|private|throw|boolean|do|implements|protected|throws|break|double|import|public|transient|byte|else|instanceof|return|try|case|extends|int|short|void||catch|final|interface|static|volatilechar|finally|long|super|while|class|float|native|switch|const|for|new|synchronized|continue|goto|package|this|true|false|null|clone|equals|finalize|getClass|hashCode|notify|notifyAll|toString|wait"))
			return "_" + _input;
		else
			return _input;
	}

	/**
	 * Parse a String and test if it corresponds to the value. This value can be multiple and separate by '|', for example,
	 * matchesValues(to_parse, new String("value1|value2|value3")).
	 * @param to_parse The substring of the string to search and replace.
	 * @param value The replacement String.
	 * @return TRUE if the parsed string corresponds to one of the values.
	 */
	public static boolean matchesValues(String to_parse, String value) {
		java.util.StringTokenizer token = new java.util.StringTokenizer(value, "|");
		while (token.hasMoreTokens()) {
			if (to_parse.equals(token.nextToken()))
				return true;
		}
		return false;
	}

	/**
	 * Parse a String and test if it ends with the value. This value can be multiple and separate by '|', for example,
	 * matchesValues(to_parse, new String("end1|end2|end3")).
	 * @param to_parse The substring of the string to search and replace.
	 * @param value The replacement String.
	 * @return TRUE if the parsed string ends with on of the values.
	 */
	public static boolean endsWithValues(String to_parse, String value) {
		java.util.StringTokenizer token = new java.util.StringTokenizer(value, "|");
		while (token.hasMoreTokens()) {
			if (to_parse.endsWith(token.nextToken()))
				return true;
		}
		return false;
	}

	/** 
	 * Add a value to a hashtable composed by an object as key (a model) and a vector as value (models keys).
	 * @param hashtable The hashtable.
	 * @param key The key in the hastable.
	 * @param object The referenced object to add to the vector corresponding to the key of the hashtable.
	 */
	public static void addHashtableValue(java.util.Hashtable hashtable, Object key, Object object) {
		if (hashtable.containsKey(key)) {
			java.util.Vector vector = (java.util.Vector) hashtable.get(key);
			if (!vector.contains(object))
				vector.add(object);
		} else {
			java.util.Vector vector = new java.util.Vector();
			vector.add(object);
			hashtable.put(key, vector);
		}
	}
    
    /**
     * Give the all classes (ordered by inheritance) of a Package.
     */
    public static MofClass[] allClassesOfPackageOrderedByInheritance(MofPackage _package) {
        List classesUnordered = new java.util.ArrayList();
        classesUnordered = MofHelper.filterContentsByClass(_package, MofClass.class, true);
        List packages = MofHelper.filterContentsByClass(_package, MofPackage.class, false);
        for (int i=0 ; i<packages.size() ; i++) {
            List classes = MofHelper.filterContentsByClass((MofPackage)packages.get(i), MofClass.class, true);
            classesUnordered.addAll(classes);
        }

        Vector classesUnorderedV = new Vector();
        Vector classesOrderedV = new Vector();
        for (int i = 0; i < classesUnordered.size(); i++)
            classesUnorderedV.addElement(classesUnordered.get(i));

        int courant = 0;
        while (classesUnorderedV.size() != 0) {
            MofClass _courant = null;
            _courant = (MofClass) classesUnorderedV.elementAt(courant);
            List supertypes = _courant.getSupertypes();
            boolean _is_top = true;
            int j = 0;
            while (_is_top && j < supertypes.size()) {
                if (classesUnorderedV.contains(supertypes.get(j)))
                   _is_top = false;
                j++;
            }
            if (_is_top) {
                classesOrderedV.addElement(_courant);
                classesUnorderedV.removeElement(_courant);
                courant = 0;
            } else
                courant++;
        }
        MofClass[] resu = new MofClass[classesOrderedV.size()];
        for (int i = 0; i < resu.length; i++)
            resu[i] = (MofClass) classesOrderedV.elementAt(i);
        return resu;
    }

    /**
     * @see fr.lip6.modfact.helper.JavaCommon#format1Idl2JavaConflict(String)
     */
    public static String format1Idl2JavaConflict(String _input) {
        Vector s1 = JMIProvider.splitAndDelete(_input);
        String buffer = " ";
        for (int i = 0; i < s1.size(); i++) {
            char[] s2 = s1.elementAt(i).toString().toCharArray();
            s2[0] = Character.toUpperCase(s1.elementAt(i).toString().charAt(0));
            String s3 = String.valueOf(s2);
            buffer = buffer.concat(s3);
        }
        return testJavaConflict(buffer.trim());
    }
    
    /**
     * @see fr.lip6.modfact.helper.JavaCommon#format2Idl2JavaConflict(String)
     */
    public static String format2Idl2JavaConflict(String _input) {
        Vector s1 = JMIProvider.splitAndDelete(_input);
        String buffer = " ";
        for (int i = 0; i < s1.size(); i++) {
            String s2 = s1.elementAt(i).toString().toLowerCase();
            buffer = buffer.concat(s2);
            if (i != (s1.size() - 1))
                buffer = buffer.concat("_");
        }
        return testJavaConflict(buffer.trim());
    }
    
}
