package org.objectweb.modfact.jmi.impl;


import javax.jmi.model.Association;
import javax.jmi.model.AssociationEnd;
import javax.jmi.model.ModelElement;

import org.objectweb.modfact.jmi.generator.BracketGenerator;
import org.objectweb.modfact.jmi.helper.ImplHelper;
import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.logging.ModFactLogger;


/**
 * @author Xavier
 *
 */
public class AssociationImplementationGenerator extends BracketGenerator {
	Association[] input;

	ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new Association[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (Association) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}


	Association association;
	AssociationEnd[] ends;
	String classEnd0;
	String classEnd1;
	String end0;
	String end1;
	
	public void generate() {
		association = input[0];		
		outputln("package " +ImplHelper.implPrefix +JMIProvider.shortQualifierOf(association) +";");
		outputln("import " +JMIProvider.qualifierOf(association) +".*;");
		outputln("import org.objectweb.modfact.jmi.reflect.*;");
		outputln(
			"public class "
				+ JMIProvider.jmiAssociationName(association)
				+ "Impl extends RefAssociationImpl " 
				+ " implements "
				+ JMIProvider.jmiAssociationQualifiedName(association)
				+ " { \n" );
		
		ends = MofHelper.associationEndsOfAssociation(association);
		classEnd0 = JMIProvider.jmiClassifierQualifiedName(ends[0].getType());
		classEnd1 = JMIProvider.jmiClassifierQualifiedName(ends[1].getType());
		end0 = ends[0].getName();
		end1 = ends[1].getName();
		
		existsTemplate();
		addTemplate();
		removeTemplate();
		queryTemplate(ends[0], end0, classEnd0, end1, classEnd1);
		queryTemplate(ends[1], end1, classEnd1, end0, classEnd0);
											
		outputln("}");
		flushFile();
	}
	
	
	void queryTemplate(AssociationEnd end, String endName, String classEndName,
			String otherEndName, String otherClassEndName) {
		//if associationEnd0 is single valued and isNavigable
		if (end.isNavigable() 
			&& isSingleValued(end) ) {					
			outputln(
				"public "
					+ classEndName
					+ " get"
					+ JMIProvider.jmiFormat1(endName)
					+ "("
					+ otherClassEndName
					+ " "
					+ otherEndName
					+ ") throws javax.jmi.reflect.JmiException { ");

			outputln("java.util.Collection c =refQuery(\""+otherEndName +"\"," +otherEndName +");");
			outputln("if(c.isEmpty()) return null;");
			outputln("return (" +classEndName +") c.iterator().next();");					
			outputln(" } ");
		}
		
		//if associationEnd0 is multivalued, and isNavigable
		if (end.isNavigable()
			&& !isSingleValued(end) ) {
			outputln(
				"public "
					+(end.getMultiplicity().isOrdered()?
						"java.util.List" : "java.util.Collection"
					 )
					+" get"
					+ JMIProvider.jmiFormat1(endName)
					+ "("
					+ otherClassEndName
					+ " "
					+ "end"
					+ ") throws javax.jmi.reflect.JmiException { ");

			outputln("return  (java.util.List) refQuery(\""+otherEndName +"\"," +"end"+");");
			outputln("} ");
		}	
	}
	
	
	void addTemplate() {
		//	if associationEnd0 and associationEnd1 isChangeable
			  if (ends[0].isChangeable() && ends[1].isChangeable()) {
				  outputln(
					  "public boolean add("
						  + classEnd0
						  + " "
						  + "end0"
						  + ","
						  + classEnd1
						  + " "
						  + "end1"
						  + ") throws javax.jmi.reflect.JmiException { ");
				  outputln("return refAddLink("+"end0"+","+"end1"+");");			
				  outputln("} ");					
			  }				
	}
	
	void existsTemplate() {
		outputln(
			"public boolean exists("
				+ classEnd0
				+ " "
				+ "end0"
				+ ","
				+ classEnd1
				+ " "
				+ "end1"
				+ ") throws javax.jmi.reflect.JmiException { ");
				
		outputln("return refLinkExists("+ "end0"+","+"end1"+");");				
		outputln("} ");
	}
	
	void removeTemplate() {
		outputln(
			"public boolean remove("
				+ classEnd0
				+ " "
				+ "end0"
				+ ","
				+ classEnd1
				+ " "
				+ "end1"
				+ ") throws javax.jmi.reflect.JmiException "
				//+", javax.jmi.reflect.NotFoundException" 
				+" { " );
				
		outputln("return refRemoveLink("+"end0"+","+"end1"+");");
		outputln("} ");
	}
	
	

	boolean isSingleValued(AssociationEnd end) {
	  return end.getMultiplicity().getUpper() == 1;
	}

}
