package org.objectweb.modfact.jmi.impl;

import org.objectweb.modfact.jmi.helper.*;


import javax.jmi.model.*;
import java.util.*;

import org.objectweb.modfact.jmi.generator.BracketGenerator;

/**
 * @author Xavier
 *
 */
public abstract class CommonImplementationGenerator
	extends BracketGenerator {

	public void attributeTemplate(Attribute attribute) {
		if (attribute
			.getVisibility()
			.equals(VisibilityKindEnum.forName("public_vis"))) {
			//<<ANNOTATION TEMPLATE>>

			//----------------------- Accessor Operations
			structuralFeatureAccessor(attribute);

			//-------------------------  Mutator Operations
			structuralFeatureMutator(attribute);

		}
	}

	public void referenceTemplate(Reference reference) {
		if (reference
			.getVisibility()
			.equals(VisibilityKindEnum.forName("public_vis"))) {
			//<<ANNOTATION TEMPLATE>>

			//----------------------- Accessor Operations
			structuralFeatureAccessor(reference);

			//-------------------------  Mutator Operations
			structuralFeatureMutator(reference);

		}
	}

	public void structuralFeatureAccessor(StructuralFeature f) {

		String javaType = JMIProvider.typeTemplate(f);
		String accessorName = JMIProvider.jmiAccessorName(f);

		outputln(
			"public "
				+ javaType
				+ " "
				+ accessorName
				+ "() throws javax.jmi.reflect.JmiException { ");

		outputln(
			"return ("
				+ javaType
				+ ") "
				+ ImplHelper.unwrap(f, "refGetValue(\"" + f.getName() + "\")")
				+ ";");
		outputln("} ");
	}

	public void structuralFeatureMutator(StructuralFeature f) {

		String mutatorName = JMIProvider.jmiMutatorName(f);
		String javaType = JMIProvider.typeTemplate(f);

		if (f.getMultiplicity().getUpper() == 1 && f.isChangeable()) {
			outputln(
				"public void "
					+ mutatorName
					+ "("
					+ javaType
					+ " newValue) throws javax.jmi.reflect.JmiException { ");

			outputln(
				"refSetValue("
					+ "\""
					+ f.getName()
					+ "\","
					+ ImplHelper.wrap(f, "newValue")
					+ ");");
			outputln("} ");
		}
	}

	///////////////// Operation //////////////////////////////////
	public void operationTemplate(Operation operation) {
		if (operation
			.getVisibility()
			.equals(VisibilityKindEnum.forName("public_vis"))) {
			//<<JAVADOCS TEMPLATE>>
			//Get Parameters and exceptions
			Parameter[] parameters = MofHelper.parametersOf(operation);
			MofException[] exceptions = MofHelper.exceptionsOf(operation);
			Parameter returnParameter = null;

			for (int i = 0; i < parameters.length; i++) {
				if (parameters[i]
					.getDirection()
					.equals(DirectionKindEnum.RETURN_DIR)) {
					returnParameter = parameters[i];
				}
			}

			if (returnParameter == null)
				output("public void ");
			else
				output(
					"public "
						+ JMIProvider.typeTemplate(returnParameter)
						+ " ");

			output(operation.getName() + "(");

			boolean comma = false;
			for (int i = 0; i < parameters.length; i++) {
				DirectionKind dir = parameters[i].getDirection();
				if (!dir.equals(DirectionKindEnum.RETURN_DIR)) {
					if (comma)
						output(",");
					output(
						JMIProvider.typeTemplate(parameters[i])
							+ " "
							+ parameters[i].getName());
					comma = true;
				}
			}
			output(") throws ");

			//for each exception
			for (int i = 0; i < exceptions.length; i++) {
				output(exceptions[i].getName() + ",");
			}
			outputln("javax.jmi.reflect.JmiException { ");
			outputln("throw new RuntimeException(\"No Implementation\");");
			outputln("} ");
		}
	}

	/////////////////// DATA TYPE //////////////////
	public void dataTypeTemplates(Namespace ns) {
		//for each StructType directly contained by the package/class
		//for each EnumType directly contained by the package/class
		DataType[] containedDataTypes = MofHelper.datatypesOf(ns, true);
		for (int i = 0; i < containedDataTypes.length; i++) {
			if (containedDataTypes[i] instanceof StructureType) {
				structTemplate((StructureType) containedDataTypes[i]);

			}
		}
	}

	public void structTemplate(StructureType t) {

		String javaType = JMIProvider.jmiDataTypeQualifiedName(t);
		output("public " + javaType + " create" + t.getName() + " (");

		ArgList argList = new ArgList();
		StructureField[] fields = MofHelper.structureFieldsOf(t);
		for (int j = 0; j < fields.length; j++) {
			output(JMIProvider.typeTemplate(fields[j]));
			//attribute type
			output(" ");
			output(fields[j].getName()); //attribute name

			argList.add(fields[j]);

			if (j != fields.length - 1)
				output(" , ");
		}
		outputln(") throws javax.jmi.reflect.JmiException { ");
		output(argList.code);
		outputln(
			"return ("
				+ javaType
				+ ") refCreateStruct(\""
				+ t.getName()
				+ "\",list);");
		outputln("}");
	}

	void newStructTemplate(Namespace p) {
		outputln("public RefStructImpl newStruct(String n) {");
		Iterator it =
			MofHelper
				.filterContentsByClass(p, StructureType.class, true)
				.iterator();
		while (it.hasNext()) {
			StructureType content = (StructureType) it.next();
			outputln(
				"if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return new " +ImplHelper.implPrefix
					+ JMIProvider.shortQualifierOf(content)
					+ "."
					+ content.getName()
					+ "Impl"
					+ "();");
		}
		outputln("throw new RuntimeException(\"invalide StructureType: '\" + n + \"'\");");		
		outputln("}");
	}

	void newEnumTemplate(Namespace p) {
		outputln("public Class newEnum(String n) {");
		Iterator it =
			MofHelper
				.filterContentsByClass(p, EnumerationType.class, true)
				.iterator();
		while (it.hasNext()) {
			EnumerationType content = (EnumerationType) it.next();
			outputln(
				"if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return "
					+ JMIProvider.jmiDataTypeQualifiedName(content)
					+ "Enum.class;");
		}
		outputln("throw new RuntimeException(\"invalide EnumType : '\" + n + \"'\");");		
		outputln("}");
	}

	// produce code to create argument list
	static class ArgList {
		String code = "java.util.List list = new java.util.Vector(); \n";

		void add(TypedElement t) {
			code =
				code
					+ "list.add("
					+ ImplHelper.wrap(t, t.getName())
					+ ");\n";
		}

	}

}
