package org.objectweb.modfact.jmi.impl;

import java.util.Iterator;

import javax.jmi.model.Association;
import javax.jmi.model.Import;
import javax.jmi.model.ModelElement;
import javax.jmi.model.MofClass;
import javax.jmi.model.MofPackage;
import javax.jmi.model.VisibilityKindEnum;

import org.objectweb.modfact.jmi.helper.ImplHelper;
import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.logging.ModFactLogger;


/**
 * @author Xavier
 *
 */
public class PackageImplementationGenerator
	extends CommonImplementationGenerator {

	MofPackage[] input;

    ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofPackage[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofPackage) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * @see org.objectweb.modfact.generation.Generator#generate()
	 */
	public void generate() {
		//Tag Import todo
		MofPackage _package = input[0];
		//javadocsTemplate(_package);

		outputln("package " +ImplHelper.implPrefix + JMIProvider.shortQualifierOf(_package) + ";");
		outputln("import " + JMIProvider.qualifierOf(_package) + ".*;");
		
		outputln("import org.objectweb.modfact.jmi.reflect.*;");

		String pQName = JMIProvider.jmiPackageQualifiedName(_package);

		outputln(
			"public class "
				+ JMIProvider.jmiPackageExtentName(_package)
				+ "PackageImpl extends RefPackageImpl"
				+ " implements "
				+ pQName
				+ "Package { ");

		factoryTemplate(_package);

		//for each StructType 
		dataTypeTemplates(_package);

		/////////// Package import //////////////////
		Import[] imports = MofHelper.importsOfPackage(_package);
		for (int i = 0; i < imports.length; i++) {
			if (VisibilityKindEnum
				.forName("public_vis")
				.equals(imports[i].getVisibility())
				&& imports[i].isClustered()
				&& imports[i].getImportedNamespace() instanceof MofPackage) {
				MofPackage importedPackage =
					(MofPackage) imports[i].getImportedNamespace();
				outputln(
					"public "
						+ JMIProvider.jmiPackageQualifiedName(importedPackage)
						+ "Package get"
						+ JMIProvider.jmiImportName(imports[i])
						+ "() {");
				outputln(
					"return "
						+ "("
						+ JMIProvider.jmiPackageQualifiedName(importedPackage)
						+ "Package)"
						+ "refPackage(\""
						+ importedPackage.getName()
						+ "\")"
						+ ";");

				outputln("}");
			}
		}

		//for each contained package where visibility = public_vis
		MofPackage[] containedPackages = MofHelper.packagesOfPackage(_package);
		for (int i = 0; i < containedPackages.length; i++) {
			if (containedPackages[i]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))) {

				outputln(
					"public "
						+ JMIProvider.jmiPackageQualifiedName(
							containedPackages[i])
						+ "Package get"
						+ JMIProvider.jmiPackageExtentName(containedPackages[i])
						+ "() {");
				outputln(
					"return "
						+ "("
						+ JMIProvider.jmiPackageQualifiedName(
							containedPackages[i])
						+ "Package)"
						+ "refPackage(\""
						+ containedPackages[i].getName()
						+ "\")"
						+ ";");
				outputln("}");
			}
		}

		//for each contained class where visibility = public_vis
		MofClass[] containedClasses = MofHelper.classesOfPackage(_package);
		for (int i = 0; i < containedClasses.length; i++) {
			if (containedClasses[i]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))) {
				outputln(
					"public "
						+ JMIProvider.jmiClassQualifiedName(containedClasses[i])
						+ "Class get"
						+ JMIProvider.jmiClassName(containedClasses[i])
						+ "() {");
				outputln(
					"return "
						+ "("
						+ JMIProvider.jmiClassQualifiedName(containedClasses[i])
						+ "Class)"
						+ "refClass(\""
						+ containedClasses[i].getName()
						+ "\")"
						+ ";");
				outputln("}");
			}
		}

		//for each contained association where visibility = public_vis
		Association[] containedAssociations =
			MofHelper.associationsOfPackage(_package);
		for (int i = 0; i < containedAssociations.length; i++) {
			if (containedAssociations[i]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))) {
				outputln(
					"public "
						+ JMIProvider.jmiAssociationQualifiedName(
							containedAssociations[i])
						+ " get"
						+ JMIProvider.jmiAssociationName(
							containedAssociations[i])
						+ "() {");
				outputln(
					"return "
						+ "("
						+ JMIProvider.jmiAssociationQualifiedName(
							containedAssociations[i])
						+ ")"
						+ "refAssociation(\""
						+ containedAssociations[i].getName()
						+ "\")"
						+ ";");
				outputln("}");
			}
		}

		newPackageTemplate(_package);
		newClassTemplate(_package);
		newAssociationTemplate(_package);
		newEnumTemplate(_package);
		newStructTemplate(_package);

		outputln("}");
		flushFile();
	}

	void newPackageTemplate(MofPackage p) {
		outputln("public RefPackageImpl newPackage(String n) {");
		Iterator it =
			MofHelper
				.filterContentsByClass(p, MofPackage.class, true)
				.iterator();
		while (it.hasNext()) {
			MofPackage content = (MofPackage) it.next();
			outputln(
				"if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return new " +ImplHelper.implPrefix
					+ JMIProvider.shortQualifierOf(content)
					+ "."
					+ JMIProvider.jmiPackageExtentName(content)
					+ "Package"
					+ "Impl"
					+ "();");
		}
		outputln("throw new RuntimeException(\"invalide type\");");
		outputln("}");
	}

	void newClassTemplate(MofPackage p) {
		outputln("public RefClassImpl newClass(String n) {");
		Iterator it =
			MofHelper.filterContentsByClass(p, MofClass.class, true).iterator();
		while (it.hasNext()) {
			MofClass content = (MofClass) it.next();
			outputln(
				"if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return new " +ImplHelper.implPrefix
					+ JMIProvider.shortQualifierOf(content)
					+ "."
					+ JMIProvider.jmiClassName(content)
					+ "Class"
					+ "Impl"
					+ "();");
		}
		outputln("throw new RuntimeException(\"invalide Class: '\" + n + \"'\");");		
		outputln("}");
	}

	void newAssociationTemplate(MofPackage p) {
		outputln("public RefAssociationImpl newAssociation(String n) {");
		Iterator it =
			MofHelper
				.filterContentsByClass(p, Association.class, true)
				.iterator();
		while (it.hasNext()) {
			Association content = (Association) it.next();
			outputln(
				"if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return new " +ImplHelper.implPrefix
					+ JMIProvider.shortQualifierOf(content)
					+ "."
					+ JMIProvider.jmiAssociationName(content)
					+ "Impl"
					+ "();");
		}
		outputln("throw new RuntimeException(\"invalide Association: '\" + n + \"'\");");		
		outputln("}");
	}

	// factory for outer mostpackage	
	void factoryTemplate(MofPackage _package) {

		if (_package.getContainer() == null) {

			String pQName = JMIProvider.jmiPackageQualifiedName(_package);
			String pShortQName = JMIProvider.shortQualifierOf(_package) +"."
								+JMIProvider.jmiPackageExtentName(_package);

			outputln(
				"public static "
					+ pQName
					+ "Package"
					+ " create(javax.jmi.model.MofPackage meta) {");
			outputln(
				ImplHelper.implPrefix + pShortQName
					+ "PackageImpl p = new "
					+ ImplHelper.implPrefix + pShortQName
					+ "PackageImpl();");
			outputln("initRepository(p, meta);");
			outputln("return p;");
			outputln("} \n");

			outputln(
				"public static "
					+ pQName
					+ "Package"
					+ " create(String metaModelURL) throws Exception {");
			outputln(
				"javax.jmi.model.MofPackage meta=(javax.jmi.model.MofPackage)"
					+ "createMetaObject(\""
					+ _package.getName()
					+ "\", metaModelURL);");
			outputln("return create(meta);");
			outputln("}");
		}
	}

}
