/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.impl.tayloredBased;

import org.objectweb.modfact.jmi.helper.*;
import org.objectweb.modfact.jmi.logging.ModFactLogger;


import javax.jmi.model.*;

import org.objectweb.modfact.jmi.generator.PrintGenerator;


/**
 * @author Xavier
 *
 */
public class AssociationImplementationGenerator extends PrintGenerator {
	Association[] input;

	ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new Association[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (Association) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}


	Association association;
	AssociationEnd[] ends;
	String classEnd0;
	String classEnd1;
	String end0;
	String end1;
	
	public void generate() {
		association = input[0];		
		out.println("package " +ImplHelper.implPrefix +JMIProvider.shortQualifierOf(association) +";");
		out.println("import " +JMIProvider.qualifierOf(association) +".*;");
		out.println("import org.objectweb.modfact.jmi.reflect.*;");
		out.println(
			"public class "
				+ JMIProvider.jmiAssociationName(association)
				+ "Impl extends RefAssociationImpl " 
				+ " implements "
				+ JMIProvider.jmiAssociationQualifiedName(association)
				+ " { \n" );
		
		ends = MofHelper.associationEndsOfAssociation(association);
		classEnd0 = JMIProvider.jmiClassifierQualifiedName(ends[0].getType());
		classEnd1 = JMIProvider.jmiClassifierQualifiedName(ends[1].getType());
		end0 = ends[0].getName();
		end1 = ends[1].getName();
		
		existsTemplate();
		addTemplate();
		removeTemplate();
		queryTemplate(ends[0], end0, classEnd0, end1, classEnd1);
		queryTemplate(ends[1], end1, classEnd1, end0, classEnd0);
											
		out.println("}");
		out.flush();
	}
	
	
	void queryTemplate(AssociationEnd end, String endName, String classEndName,
			String otherEndName, String otherClassEndName) {
		//if associationEnd0 is single valued and isNavigable
		if (end.isNavigable() 
			&& isSingleValued(end) ) {					
			out.println(
				"\tpublic "
					+ classEndName
					+ " get"
					+ JMIProvider.jmiFormat1(endName)
					+ "("
					+ otherClassEndName
					+ " "
					+ otherEndName
					+ ") throws javax.jmi.reflect.JmiException { ");

			out.println("\t\tjava.util.Collection c =refQuery(\""+otherEndName +"\"," +otherEndName +");");
			out.println("\t\tif(c.isEmpty()) return null;");
			out.println("\t\treturn (" +classEndName +") c.iterator().next();");					
			out.println("\t } ");
		}
		
		//if associationEnd0 is multivalued, and isNavigable
		if (end.isNavigable()
			&& !isSingleValued(end) ) {
			out.println(
				"\tpublic "
					+(end.getMultiplicity().isOrdered()?
						"java.util.List" : "java.util.Collection"
					 )
					+" get"
					+ JMIProvider.jmiFormat1(endName)
					+ "("
					+ otherClassEndName
					+ " "
					+ "end"
					+ ") throws javax.jmi.reflect.JmiException { ");

			out.println("\t\treturn  (java.util.List) refQuery(\""+otherEndName +"\"," +"end"+");");
			out.println("\t } ");
		}	
	}
	
	
	void addTemplate() {
		//	if associationEnd0 and associationEnd1 isChangeable
			  if (ends[0].isChangeable() && ends[1].isChangeable()) {
				  out.println(
					  "\tpublic boolean add("
						  + classEnd0
						  + " "
						  + "end0"
						  + ","
						  + classEnd1
						  + " "
						  + "end1"
						  + ") throws javax.jmi.reflect.JmiException { ");
				  out.println("\t\treturn refAddLink("+"end0"+","+"end1"+");");			
				  out.println("\t } ");					
			  }				
	}
	
	void existsTemplate() {
		out.println(
			"\tpublic boolean exists("
				+ classEnd0
				+ " "
				+ "end0"
				+ ","
				+ classEnd1
				+ " "
				+ "end1"
				+ ") throws javax.jmi.reflect.JmiException { ");
				
		out.println("\t\treturn refLinkExists("+ "end0"+","+"end1"+");");				
		out.println("\t} ");
	}
	
	void removeTemplate() {
		out.println(
			"\tpublic boolean remove("
				+ classEnd0
				+ " "
				+ "end0"
				+ ","
				+ classEnd1
				+ " "
				+ "end1"
				+ ") throws javax.jmi.reflect.JmiException "
				//+", javax.jmi.reflect.NotFoundException" 
				+" { " );
				
		out.println("\t\treturn refRemoveLink("+"end0"+","+"end1"+");");
		out.println("\t } ");
	}
	
	

	boolean isSingleValued(AssociationEnd end) {
	  return end.getMultiplicity().getUpper() == 1;
	}

}
