/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.impl.tayloredBased;

import org.objectweb.modfact.jmi.helper.*;
import org.objectweb.modfact.jmi.logging.ModFactLogger;

import javax.jmi.model.*;

/**
 * @author Xavier
 *
 */
public class ClassProxyImplementationGenerator
	extends CommonImplementationGenerator {

	MofClass[] input;

	ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofClass[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofClass) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * @see org.objectweb.modfact.generation.Generator#generate()
	 */
	public void generate() {
		//Tag todo
		int c = 0;

		out.println("package " +ImplHelper.implPrefix + JMIProvider.shortQualifierOf(input[c]) + ";" );
		out.println("import " + JMIProvider.qualifierOf(input[c]) + ".*;" );		
		out.println("import org.objectweb.modfact.jmi.reflect.*;");
		out.println(
			"public class "
				+ JMIProvider.jmiClassName(input[c])
				+ "ClassImpl extends org.objectweb.modfact.jmi.reflect.RefClassImpl"
				+ " implements "
				+ JMIProvider.jmiClassQualifiedName(input[c]) +"Class"
				+ " {");
		
		creatorTemplate(input[c]);
		newObjectTemplate(input[c]);

		//for each StructType directly contained by the class (correction of the spec ?)
		dataTypeTemplates(input[c]);

		newEnumTemplate(input[c]);
		newStructTemplate(input[c]);

		//for each attribute and operation contained in this class or one of its
		//supertypes with "classifier-level" scope, generate the appropriate operations
		Attribute[] attributes = MofHelper.attributesOfClass(
				input[c],
				ScopeKindEnum.forName("classifier_level"), true );
		for (int i = 0; i < attributes.length; i++) {
			attributeTemplate(attributes[i]);
		}

		Operation[] operations =
			MofHelper.operationsOfClass(
				input[c],
				ScopeKindEnum.forName("classifier_level"), true );
		for (int i = 0; i < operations.length; i++) {
			operationTemplate(operations[i]);
		}

		out.println("}//end of interface");
		out.flush();
	}
	
	void creatorTemplate(MofClass cl) {
		//If isAbstract is not set to true, generate factory methods.		
		if (!cl.isAbstract()) {
			String javaType = JMIProvider.jmiClassQualifiedName(cl);
			out.println(
				"\tpublic "
					+ javaType
					+ " create"
					+ JMIProvider.jmiClassName(cl)
					+ "() throws javax.jmi.reflect.JmiException { ");
			 
			  out.println("\t\t return ("
				+javaType
				+") refCreateInstance(null);");			  		
			out.println("\t} \n");

			ArgList argList = new ArgList();
			Attribute[] attributes = MofHelper.attributesOfClass(cl,
					ScopeKindEnum.forName("instance_level"), true );
										
			java.util.List v = new java.util.Vector();
			for(int i =0; i<attributes.length; i++) {
			  if(!attributes[i].isDerived()) v.add(attributes[i]);		
			}
			attributes = (Attribute[]) v.toArray(new Attribute[0]);
								
			if (attributes.length != 0) {
				out.print(
					"\tpublic "
						+ JMIProvider.jmiClassQualifiedName(cl)
						+ " create"
						+ JMIProvider.jmiClassName(cl)
						+ "(");
				//for each non-derived direct or inherited attribute		
				for (int i = 0; i < attributes.length; i++) {	
				  	
					out.print(
						JMIProvider.typeTemplate(attributes[i]) +" " +attributes[i].getName()	
					);
					argList.add(attributes[i]);				
					if (i != attributes.length - 1)
						out.print(" , ");
				}
				out.println(") throws javax.jmi.reflect.JmiException { \n");
					out.print(argList.code);
					out.println("\t\t return ("
				  		+javaType
				  		+") refCreateInstance(list);");			  		
			  	out.println("\t} \n");
			}
		}
	}		
	
	void newObjectTemplate(MofClass cl) {
		out.println("\tpublic RefObjectImpl newObject() { ");
		  out.println("\t\treturn new " +ImplHelper.implPrefix 
		  	+JMIProvider.shortQualifierOf(cl)
		  	+"."
		  	+JMIProvider.jmiClassName(cl)+"Impl();");
		out.println("\t}");
	}

}
