/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.impl.tayloredBased;

import org.objectweb.modfact.jmi.helper.*;


import javax.jmi.model.*;
import java.util.*;

import org.objectweb.modfact.jmi.generator.PrintGenerator;

/**
 * @author Xavier
 *
 */
public abstract class CommonImplementationGenerator
	extends PrintGenerator {

	public void attributeTemplate(Attribute attribute) {
		if (attribute
			.getVisibility()
			.equals(VisibilityKindEnum.forName("public_vis"))) {
			//<<ANNOTATION TEMPLATE>>

			//----------------------- Accessor Operations
			structuralFeatureAccessor(attribute);

			//-------------------------  Mutator Operations
			structuralFeatureMutator(attribute);

		}
	}

	public void referenceTemplate(Reference reference) {
		if (reference
			.getVisibility()
			.equals(VisibilityKindEnum.forName("public_vis"))) {
			//<<ANNOTATION TEMPLATE>>

			//----------------------- Accessor Operations
			structuralFeatureAccessor(reference);

			//-------------------------  Mutator Operations
			structuralFeatureMutator(reference);

		}
	}

	public void structuralFeatureAccessor(StructuralFeature f) {

		String javaType = JMIProvider.typeTemplate(f);
		String accessorName = JMIProvider.jmiAccessorName(f);

		out.println(
			"\tpublic "
				+ javaType
				+ " "
				+ accessorName
				+ "() throws javax.jmi.reflect.JmiException { ");

		out.println(
			"\t return ("
				+ javaType
				+ ") "
				+ ImplHelper.unwrap(f, "refGetValue(\"" + f.getName() + "\")")
				+ ";");
		out.println("\t } ");
	}

	public void structuralFeatureMutator(StructuralFeature f) {

		String mutatorName = JMIProvider.jmiMutatorName(f);
		String javaType = JMIProvider.typeTemplate(f);

		if (f.getMultiplicity().getUpper() == 1 && f.isChangeable()) {
			out.println(
				"\tpublic void "
					+ mutatorName
					+ "("
					+ javaType
					+ " newValue) throws javax.jmi.reflect.JmiException { ");

			out.println(
				"\t refSetValue("
					+ "\""
					+ f.getName()
					+ "\","
					+ ImplHelper.wrap(f, "newValue")
					+ ");");
			out.println("\t } ");
		}
	}

	///////////////// Operation //////////////////////////////////
	public void operationTemplate(Operation operation) {
		if (operation
			.getVisibility()
			.equals(VisibilityKindEnum.forName("public_vis"))) {
			//<<JAVADOCS TEMPLATE>>
			//Get Parameters and exceptions
			Parameter[] parameters = MofHelper.parametersOf(operation);
			MofException[] exceptions = MofHelper.exceptionsOf(operation);
			Parameter returnParameter = null;

			for (int i = 0; i < parameters.length; i++) {
				if (parameters[i]
					.getDirection()
					.equals(DirectionKindEnum.RETURN_DIR)) {
					returnParameter = parameters[i];
				}
			}

			if (returnParameter == null)
				out.print("\tpublic void ");
			else
				out.print(
					"\tpublic "
						+ JMIProvider.typeTemplate(returnParameter)
						+ " ");

			out.print(operation.getName() + "(");

			boolean comma = false;
			for (int i = 0; i < parameters.length; i++) {
				DirectionKind dir = parameters[i].getDirection();
				if (!dir.equals(DirectionKindEnum.RETURN_DIR)) {
					if (comma)
						out.print(",");
					out.print(
						JMIProvider.typeTemplate(parameters[i])
							+ " "
							+ parameters[i].getName());
					comma = true;
				}
			}
			out.print(") throws ");

			//for each exception
			for (int i = 0; i < exceptions.length; i++) {
				out.print(exceptions[i].getName() + ",");
			}
			out.println("javax.jmi.reflect.JmiException { ");
			out.println("throw new RuntimeException(\"No Implementation\");");
			out.println("\t } ");
		}
	}

	/////////////////// DATA TYPE //////////////////
	public void dataTypeTemplates(Namespace ns) {
		//for each StructType directly contained by the package/class
		//for each EnumType directly contained by the package/class
		DataType[] containedDataTypes = MofHelper.datatypesOf(ns, true);
		for (int i = 0; i < containedDataTypes.length; i++) {
			if (containedDataTypes[i] instanceof StructureType) {
				structTemplate((StructureType) containedDataTypes[i]);

			}
		}
	}

	public void structTemplate(StructureType t) {

		String javaType = JMIProvider.jmiDataTypeQualifiedName(t);
		out.print("\tpublic " + javaType + " create" + t.getName() + " (");

		ArgList argList = new ArgList();
		StructureField[] fields = MofHelper.structureFieldsOf(t);
		for (int j = 0; j < fields.length; j++) {
			out.print(JMIProvider.typeTemplate(fields[j]));
			//attribute type
			out.print(" ");
			out.print(fields[j].getName()); //attribute name

			argList.add(fields[j]);

			if (j != fields.length - 1)
				out.print(" , ");
		}
		out.println(") throws javax.jmi.reflect.JmiException { ");
		out.print(argList.code);
		out.println(
			"\t return ("
				+ javaType
				+ ") refCreateStruct(\""
				+ t.getName()
				+ "\",list);");
		out.println("}");
	}

	void newStructTemplate(Namespace p) {
		out.println("\tpublic RefStructImpl newStruct(String n) {");
		Iterator it =
			MofHelper
				.filterContentsByClass(p, StructureType.class, true)
				.iterator();
		while (it.hasNext()) {
			StructureType content = (StructureType) it.next();
			out.println(
				"\t\t if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return new " +ImplHelper.implPrefix
					+ JMIProvider.shortQualifierOf(content)
					+ "."
					+ content.getName()
					+ "Impl"
					+ "();");
		}
		out.println("\t\t throw new RuntimeException(\"invalide StructureType: '\" + n + \"'\");");		
		out.println("\t}");
	}

	void newEnumTemplate(Namespace p) {
		out.println("\tpublic Class newEnum(String n) {");
		Iterator it =
			MofHelper
				.filterContentsByClass(p, EnumerationType.class, true)
				.iterator();
		while (it.hasNext()) {
			EnumerationType content = (EnumerationType) it.next();
			out.println(
				"\t\t if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return "
					+ JMIProvider.jmiDataTypeQualifiedName(content)
					+ "Enum.class;");
		}
		out.println("\t\t throw new RuntimeException(\"invalide EnumType : '\" + n + \"'\");");		
		out.println("\t}");
	}

	// produce code to create argument list
	static class ArgList {
		String code = "\t\tjava.util.List list = new java.util.Vector(); \n";

		void add(TypedElement t) {
			code =
				code
					+ "\t\t list.add("
					+ ImplHelper.wrap(t, t.getName())
					+ ");\n";
		}

	}

}
