/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.impl.tayloredBased;

import org.objectweb.modfact.jmi.helper.*;
import org.objectweb.modfact.jmi.logging.ModFactLogger;

import javax.jmi.model.*;

/**
 * @author Xavier
 *
 */
public class InstanceImplementationGenerator extends CommonImplementationGenerator {
	MofClass[] input;

	ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofClass[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofClass) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * @see org.objectweb.modfact.generation.Generator#generate()
	 */
	public void generate() {

		MofClass c = input[0];

		out.println("package " +ImplHelper.implPrefix + JMIProvider.shortQualifierOf(c) + ";" );
		out.println("import " + JMIProvider.qualifierOf(c) + ".*;" );
		
		out.println("import org.objectweb.modfact.jmi.reflect.*;");

		out.print("public class " + JMIProvider.jmiClassName(c) +"Impl"
			+" extends RefObjectImpl "
			+"implements "
			+JMIProvider.jmiClassQualifiedName(c)
			+" {" );


		//for each instance level attribute of this class, generate the appropriate code
		//MofAttribute[] attributes_d = MofHelper.attributesOfClass(c , ScopeKind.instance_level , true );
		//for (int i = 0 ; i < attributes_d.length ; i++) {
		//	attributeTemplate(attributes_d[i]);
		//}

		Attribute[] attributes_nd =
			MofHelper.attributesOfClass(
				c,
				ScopeKindEnum.forName("instance_level"), true);
				
		for (int i = 0; i < attributes_nd.length; i++) {
			attributeTemplate(attributes_nd[i]);
		}

		//for each instance level reference of this class, generate the appropriate code if public
		Reference[] references = MofHelper.referencesOfClass( c, true );
		for (int i = 0; i < references.length; i++) {
			referenceTemplate(references[i]);
		}

		//for each operation, generate the appropriate code if public_vis
		Operation[] operations = MofHelper.operationsOfClass(c, ScopeKindEnum.INSTANCE_LEVEL, true );
		for (int i = 0; i < operations.length; i++) {
			if (operations[i].getVisibility().equals(VisibilityKindEnum.PUBLIC_VIS))
				operationTemplate(operations[i]);
		}

		//end
		out.println("}"); //end of interface
		out.flush();
	}
}
