package org.objectweb.modfact.jmi.ps;

import javax.jmi.reflect.*;
import java.util.*;
import org.objectweb.modfact.jmi.reflect.*;
import java.sql.SQLException;

/**
 * @author gregory
 *
 * The persistant service provide common operation
 * for loadind and unloading a model in the repository.
 * All metamodels are uniquely identified by their name.
 * When unloading a metamodel, it is written in the database
 */
public class PersistantService {
	
	
	private RefPackage repository;
	
	
	public PersistantService(){
		this.repository = RefPackageImpl.getM2Repository();	
	}


	/**
	 * Method load.
	 * @param name
	 * @return boolean
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 *
	 * load a metamodel in the M2 level repository
	 * */

	public  boolean load(String name)	throws ClassNotFoundException, SQLException {
		
		if(isLoaded(name))
			return true;
		else{
		
		Reader psReader = new Reader(name, repository);
		psReader.read();
		return  true;		
		}
		
		
	}


	/**
	 * Method isLoaded.
	 * @param metamodel
	 * @return boolean
	 */
	private boolean isLoaded(String metamodel){
		RefObject o = null ;
		Iterator it = repository.refAllPackages().iterator();
		while( it.hasNext()){
			if(((RefObject)it.next()).refGetValue("name").equals(metamodel))
				return true;
		}	
		return false;
		
	}
	

	/**
	 * Method store.
	 * @param name
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 * 
	 * Very dirty : we delete the content of the metamodel before writing it
	 * again ...
	 */
	public  void store(String name)
		throws ClassNotFoundException, SQLException {

		boolean modelExist;
		String modelId;
		String elementId;
		modelExist = PersistantHelper.ifModelExist(name);
		if (!modelExist)
			PersistantHelper.createMetamodel(name);
		else
			PersistantHelper.deleteMetamodelContent(name);

		modelId = PersistantHelper.getModelIdbyName(name);

		MOFPolymorphicIterator mofIt = new MOFPolymorphicIterator(repository);

		Writer psWriter = new Writer(repository);
		while (mofIt.hasNext()) {
			RefBaseObject ooo = (RefBaseObject) mofIt.next();
			if (ooo instanceof RefObject) {

				elementId =	psWriter.create((RefObject) ooo,modelId);
				psWriter.rename((RefObject) ooo, elementId);
			}
		}
		//null all reference of the repository where outermost package == metamodelname
		psWriter = null;
		mofIt = null;
		System.gc();
	}

	public  void deleteMetamodel(RefPackage metaModel) {
	
	}
	
	public  RefObject getRepository(String metamodel){
		
		
		return null;
	}

}
