package org.objectweb.modfact.jmi.ps;
import javax.jmi.reflect.*;
import java.util.*;
import java.sql.*;
/**
 * @author gregory
 * This class is a wrapper for accessing the database and read a metamodel from
 * in order to build the metamodel in the M2 repository given as a parameter.
 * The granulariry of reading is the metamodel
 */
public class Reader {

	Connection connection;
	MOFPolymorphicIterator mofIt;
	MOFPolymorphicIterator mofModelIt;
	RefPackage mofModel;
	String modelName;

	Hashtable object2ContainerMofId;
	Hashtable mofId2object;
	Hashtable referenceObject2referencedEndmofId;

	List allRefObjects;
	String mofId;
	RefObject containerObject;

	/**
	 * Method Reader.
	 * @param modelName
	 * @param repository
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 * 
	 * read the model uniquely identified by its name and store it
	 * into the repository
	 */
	public Reader(String modelName, RefPackage repository)
		throws ClassNotFoundException, SQLException {

		Class.forName("com.mysql.jdbc.Driver");
		String url = "jdbc:mysql://localhost:3306/test";
		connection = DriverManager.getConnection(url);
		this.modelName = modelName;
		this.mofModel = repository;
		this.mofId2object = new Hashtable();
		this.object2ContainerMofId = new Hashtable();
		this.referenceObject2referencedEndmofId = new Hashtable();
		this.allRefObjects = new Vector();
	}

	/**
	 * Method read.
	 * 
	 * the building of me metamodel in the repository is done in a two pass way.
	 * because of dependencies between the repository object.
	 * The necessary information are temporary stored in hashtables.
	 */
	public void read() {
		RefObject oo = null;
		RefObject o = null;
		Vector params = new Vector();

		String query =
			"SELECT * FROM RefPackage WHERE ModelContainerId='"
				+ PersistantHelper.getModelIdbyName(modelName)
				+ "';";

		try {
			Statement statement = connection.createStatement();
			ResultSet rs = statement.executeQuery(query);
			while (rs.next()) {

				if (rs.getString("type").equals("RefPackage")) {

					o = mofModel.refClass("Package").refCreateInstance(null);
					o.refSetValue("name", rs.getString("name"));
					o.refSetValue("annotation", rs.getString("annotation"));
					o.refSetValue("isRoot", rs.getString("isRoot"));
					o.refSetValue("isLeaf", rs.getString("isLeaf"));
					o.refSetValue("isAbstract", rs.getString("isAbstract"));
					o.refSetValue("visibility", rs.getString("visibility"));

					object2ContainerMofId.put(o, rs.getString("container"));
					mofId2object.put(rs.getString("RefPackageId"), o);
					allRefObjects.add(o);

				}

				if (rs.getString("type").equals("RefClass")) {

					params.addElement(rs.getString("name"));
					params.addElement(rs.getString("annotation"));
					params.addElement(rs.getString("isRoot"));
					params.addElement(rs.getString("isLeaf"));
					params.addElement(rs.getString("isAbstract"));
					params.addElement(rs.getString("visibility"));
					params.addElement(rs.getString("isSingleton"));

					o = mofModel.refClass("Class").refCreateInstance(params);

					params.removeAllElements();
					object2ContainerMofId.put(o, rs.getString("container"));
					mofId2object.put(rs.getString("RefPackageId"), o);
					allRefObjects.add(o);
				}

				if (rs.getString("type").equals("RefAssociation")) {

					params.addElement(rs.getString("name"));
					params.addElement(rs.getString("annotation"));
					params.addElement(rs.getString("isRoot"));
					params.addElement(rs.getString("isLeaf"));
					params.addElement(rs.getString("isAbstract"));
					params.addElement(rs.getString("visibility"));
					params.addElement(rs.getString("isDerived"));
					
					o =	mofModel.refClass("Association").refCreateInstance(
							params);
					params.removeAllElements();
					object2ContainerMofId.put(o, rs.getString("container"));
					mofId2object.put(rs.getString("RefPackageId"), o);
					allRefObjects.add(o);
				}
				if (rs.getString("type").equals("RefAssociationEnd")) {

					o =
						mofModel.refClass("AssociationEnd").refCreateInstance(
							null);
					o.refSetValue("name", rs.getString("name"));
					o.refSetValue("annotation", rs.getString("annotation"));
					o.refSetValue("isNavigable", rs.getString("isNavigable"));
					o.refSetValue("aggregation", rs.getString("aggregation"));
					o.refSetValue("multiplicity", rs.getString("multiplicity"));
					o.refSetValue("isChangeable", rs.getString("isChangeable"));

					object2ContainerMofId.put(o, rs.getString("container"));
					mofId2object.put(rs.getString("RefPackageId"), o);
					allRefObjects.add(o);
				}

				if (rs.getString("type").equals("RefAttribute")) {

					o = mofModel.refClass("Attribute").refCreateInstance(null);
					o.refSetValue("name", rs.getString("name"));
					o.refSetValue("annotation", rs.getString("annotation"));
					o.refSetValue("visibility", rs.getString("visibility"));
					o.refSetValue("scope", rs.getString("scope"));
					o.refSetValue("isChangeable", rs.getString("isChangeable"));
					o.refSetValue("multiplicity", rs.getString("multiplicity"));
					o.refSetValue("isDerived", rs.getString("isDerived"));

					object2ContainerMofId.put(o, rs.getString("container"));
					mofId2object.put(rs.getString("RefPackageId"), o);
					allRefObjects.add(o);
				}

				if (rs.getString("type").equals("RefReference")) {

					o = mofModel.refClass("Reference").refCreateInstance(null);
					o.refSetValue("name", rs.getString("name"));
					o.refSetValue("annotation", rs.getString("annotation"));
					o.refSetValue("visibility", rs.getString("visibility"));
					o.refSetValue("scope", rs.getString("scope"));
					o.refSetValue("isChangeable", rs.getString("isChangeable"));
					o.refSetValue("multiplicity", rs.getString("multiplicity"));

					object2ContainerMofId.put(o, rs.getString("container"));
					mofId2object.put(rs.getString("RefPackageId"), o);

					referenceObject2referencedEndmofId.put(
						o,
						rs.getString("referencedEnd"));

					allRefObjects.add(o);
				}

				if (rs.getString("type").equals("Tag")) {

					o = mofModel.refClass("Tag").refCreateInstance(null);
					o.refSetValue("name", rs.getString("name"));
					o.refSetValue("annotation", rs.getString("annotation"));
					o.refSetValue("tagId", rs.getString("tagId"));
					o.refSetValue("values", rs.getString("tagtagIdalues"));
					o.refSetValue("elements", rs.getString("taggedElements"));

					object2ContainerMofId.put(o, rs.getString("container"));
					mofId2object.put(rs.getString("RefPackageId"), o);
					allRefObjects.add(o);
				}

			} //end while

			rs.close();
			statement.close();

			//Get the reference of the container object and set this property
			Iterator it = allRefObjects.iterator();
			while (it.hasNext()) {

				oo = (RefObject) it.next();
				mofId = object2ContainerMofId.get(oo).toString();
				containerObject = (RefObject) mofId2object.get(mofId);
				oo.refSetValue("container", containerObject);
			}

			//Set the referenceEnd property for reference objects
			it = allRefObjects.iterator();
			while (it.hasNext()) {

				oo = (RefObject) it.next();

				if (oo
					.refMetaObject()
					.refGetValue("name")
					.equals("Reference")) {
					mofId =
						referenceObject2referencedEndmofId.get(oo).toString();
					oo.refSetValue("referencedEnd", mofId2object.get(mofId));
				}

			}

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

}
