package org.objectweb.modfact.jmi.reflect;

import javax.jmi.reflect.*;
import java.util.*;

// this link is bind to Association
// the update to the other end is transparent to users
public class AssoEndList implements java.util.List {
    
    RefAssociationImpl asso;
    RefObject key;
    boolean keyIsFirstEnd;
    
    public AssoEndList(RefAssociationImpl asso, RefObject key, boolean keyIsFirstEnd) {
        this.asso = asso;
        this.key = key;
        this.keyIsFirstEnd = keyIsFirstEnd;
    }
    
    public boolean add(Object obj) {
        if(forwardList().contains(obj)) return false;
        reverseList(obj).add(key);
        return forwardList().add(obj);
    }
    
    public void add(int index, Object obj) {
        int i = forwardList().indexOf(obj);
        if(i==index) return;
        if(i!=-1) throw new RuntimeException("obj already exists at " +i);  
        forwardList().add(index, obj);
        reverseList(obj).add(key);    
    }
    
    public boolean addAll(java.util.Collection collection) {
        Iterator it =  collection.iterator();
        boolean r = false;
        while(it.hasNext()) {
           Object o = it.next(); 
           if(!forwardList().contains(o)) {
              reverseList(o).add(key);
              forwardList().add(o);
              r = true;
           }
        }
        return r;
    }
    
    public boolean addAll(int index, java.util.Collection collection) {
        throw new RuntimeException("not supported");      
    }
    
    public void clear() {
       Iterator it = forwardList().iterator();
       while(it.hasNext()) {
          reverseList(it.next()).remove(key);
       }
       forwardList().clear();
    }
   
       
    public boolean remove(Object obj) {
       reverseList(obj).remove(key);
       return forwardList().remove(obj);
    }
    
    public Object remove(int index) {
       Object obj = forwardList().remove(index);
       reverseList(obj).remove(key);
       return obj;
    }
    
    public boolean removeAll(java.util.Collection collection) {
       Iterator it = collection.iterator();
       boolean r = false;
       while(it.hasNext()) {
           Object o = it.next();
           if(forwardList().contains(o)) {
             reverseList(o).remove(key);
             forwardList().remove(o);
             r = true;
          }
       }
       return r;
    }
    
    public boolean retainAll(java.util.Collection collection) {
       Iterator it = forwardList().iterator();
       while(it.hasNext()) {
           Object o = it.next();
           if(!collection.contains(o) && forwardList().contains(o)) {
             reverseList(o).remove(key);
          }
       }
       return forwardList().retainAll(collection);
    }
    
    public Object set(int index, Object obj) {
      int i = forwardList().indexOf(obj);
      if(i==index) return obj;
      if(i!=-1) throw new RuntimeException("obj already exists at " +i);      
      Object replaced = forwardList().set(index, obj);
      reverseList(obj).add(key);
      reverseList(replaced).remove(key);
      return replaced;
    }
    
    //------------------------------------
    // Helper functions
    
    List forwardList = null;
    
    List forwardList() {
        if(forwardList!=null) return forwardList;
        if(keyIsFirstEnd) {
           Object temp = asso.table_1_2.get(key);
           forwardList = (List) temp;           
           if(forwardList==null) {
             forwardList = new Vector();
             asso.table_1_2.put(key, forwardList);
           }
        } else {
           forwardList = (List) asso.table_2_1.get(key);
           if(forwardList==null) {
             forwardList = new Vector();
             asso.table_2_1.put(key, forwardList);        
           }
        }
        return forwardList; 
    }
    
    List reverseList(Object o) {
        if(!asso.container.canModify) {
            throw new RuntimeException("Not allowed to modify");
        }
        List r;
        if(keyIsFirstEnd) {
           r = (List) asso.table_2_1.get(o);
           if(r==null) {
             r = new Vector();
             asso.table_2_1.put(o, r);
           }
        } else {
           r = (List) asso.table_1_2.get(o);
           if(r==null) {
             r = new Vector();
             asso.table_1_2.put(o, r);        
           }
        }
        return r;
    }
    
    public String toString() {
       return forwardList().toString();
    }
    
    
    //--------------------------------------------------------------    
    //--------------------------------------------------------------    
    // Query operations
    
    public boolean contains(Object obj) {
       return forwardList().contains(obj); 
    }
    
    public boolean containsAll(java.util.Collection collection) {
       return forwardList().containsAll(collection); 
    }
    
    public Object get(int index) {
       return forwardList().get(index); 
    }
    
    public int indexOf(Object obj) {
       return forwardList().indexOf(obj); 
    }
    
    public boolean isEmpty() {
        return forwardList().isEmpty();
    }
    
    public int lastIndexOf(Object obj) {
        return forwardList().lastIndexOf(obj);
    }
    
    // *** users should not use iterator to modify the list ***
    public java.util.Iterator iterator() {
        return forwardList().iterator();
    }    
    public java.util.ListIterator listIterator() {
        return forwardList().listIterator();
    }
    public java.util.ListIterator listIterator(int index) {
        return forwardList().listIterator(index);
    }
    
    public int size() {
      return forwardList().size();  
    }
    
    public java.util.List subList(int i1, int i2) {
      return forwardList().subList(i1, i2);  
    }
    
    public Object[] toArray() {
      return forwardList().toArray();
    }
    
    public Object[] toArray(Object[] obj) {
      return forwardList().toArray(obj);
    }
    
    
}
