package org.objectweb.modfact.jmi.reflect;

import java.lang.reflect.*;
import javax.jmi.reflect.*;
import java.util.*;

public class RefClassImpl extends DataTypeContainer 
							implements javax.jmi.reflect.RefClass, java.io.Serializable {
    
    Collection allOfClass = new Vector();
    List directSubtypes = null;   //list of RefClass
    List directSupertypes = null; //list of RefClass
    
    //****** Properties of this class only
    // Class-level Attritbutes                                
    Object[] classAttValues = null;
    int[] classAttMultiplicities = null; //upper
    List classAttNames = new Vector();

    //****** Properties of this class AND its supertypes    
    // Instance-level Attributes
    int[] attMultiplicities = null;  //upper  
    List attNames = new Vector();
        
    // References
    private transient RefAssociationImpl[] assos = null; 
    List[] assoNames = null;
    String[] assoEndNames = null;
    int[] refMultiplicities = null; //upper
    List refNames = new Vector();     
        
    public RefClassImpl() {
    }
    
    
    public java.util.Collection refAllOfClass() {
        return new Vector(allOfClass);
    }
    
    public java.util.Collection refAllOfType() {
        Collection allOfType = new DistinctList(allOfClass);
        // go down to sub types and add instances
        Iterator it = directSubtypes().iterator();
        while(it.hasNext()) {
            RefClass subtype = (RefClass)it.next();
            allOfType.addAll(subtype.refAllOfType());
        }
        return allOfType;
    }
    
    // collection of class proxies
    List directSubtypes() {
      if(directSubtypes!=null) return directSubtypes();	
      RefObject meta = refMetaObject();
      Iterator it = meta.refOutermostPackage()
      	.refAssociation("Generalizes")
      	.refQuery("supertype",meta)
      	.iterator();
      List r = new Vector();
      while(it.hasNext()) {
        RefObjectImpl typeMeta = (RefObjectImpl) it.next();
		r.add( ReflectHelper.resolveQualifiedNameInExtent(
				   this, ReflectHelper.getQualifiedName(typeMeta)
			   )
		);
      }
      return r;
    }
    
	// collection of class proxies    
    List directSupertypes() {
		if(directSupertypes!=null) return directSupertypes();	
		RefObject meta = refMetaObject();
		Iterator it = meta.refOutermostPackage()
		  .refAssociation("Generalizes")
		  .refQuery("subtype",meta)
		  .iterator();
		List r = new Vector();
		while(it.hasNext()) {
		  RefObjectImpl typeMeta = (RefObjectImpl) it.next();
		  r.add( ReflectHelper.resolveQualifiedNameInExtent(
					 this, ReflectHelper.getQualifiedName(typeMeta)
				 )
		  );
		}
		return r;    	
    }
    
    RefAssociationImpl[] associations() {
      if(assos!=null) return assos;
      assos = new RefAssociationImpl[assoNames.length];
      for(int i=0; i<assoNames.length; i++) {
      	assos[i] = (RefAssociationImpl) ReflectHelper.resolveQualifiedNameInExtent(this, assoNames[i]);      	
      }
      return assos; 
    }
    
    
    public javax.jmi.reflect.RefObject refCreateInstance(java.util.List args) {
        if(!container.canModify) {
            throw new RuntimeException("Not allowed to modify");
        }
        if(args!=null) {
          if( args.size()!=attNames.size())
             throw new RuntimeException(this +": Wrong size argument "
               +args.size() +" expect " +attNames.size() +"."
             );             
          // check multiplicity conformance
          // todo: check type too
          for(int i=0; i<attMultiplicities.length; i++ ) {
            if((attMultiplicities[i]!=1) && (args.get(i) instanceof Collection)) 
             throw new RuntimeException(this +": Invalide args at " +i 
                +" " +args.get(i) 
                +" should be Collection type"
             );    
          }
        }
        return _refCreateInstance(args);
    }
  
    javax.jmi.reflect.RefObject _refCreateInstance(java.util.List list) {
        RefObjectImpl instance = newObject();
        
        instance.metaObject = metaObject;
        instance.container = container;
        instance.name = name;
        instance.classProxy = this;
        
        instance.setArgs(list);
        allOfClass.add(instance);
        return instance;
    }
     
    public javax.jmi.reflect.RefObject refMetaObject() {
        if(metaObject!=null) return metaObject;
        return findObjectByName( container.refClass("Class"), name);
    }    
    

    
    void readContentsFromMetaObject() {
      readContent_without_Inheritance();
      readContent_with_Inheritance();
      readDataTypeContents();
    }
    
    void readContent_without_Inheritance() {
      Iterator it;
      it = ((Collection)metaObject.refGetValue("contents")).iterator();
      while(it.hasNext()) {
        RefObject content = (RefObject)it.next();
        String contentName = (String) content.refGetValue("name");
        String contentType = (String) content.refMetaObject().refGetValue("name");
        if(contentType.equals("Attribute")) {
          if( content.refImmediatePackage()
          		.refGetEnum("ScopeKind" ,"classifier_level")
          		.equals(
             		content.refGetValue("scope")
            	)  
            && Boolean.FALSE.equals(content.refGetValue("isDerived"))
          ) { 
            classAttNames.add(contentName);
          }
        } 
      }
      // Classifier-level Attributes
      classAttMultiplicities = new int[classAttNames.size()];
      
      it = ((Collection)metaObject.refGetValue("contents")).iterator();
      while(it.hasNext()) {
        RefObject content = (RefObject)it.next();
        String contentName = (String) content.refGetValue("name");
        String contentType = (String) content.refMetaObject().refGetValue("name");
        
        if(contentType.equals("Attribute")) {
          if( content.refImmediatePackage().refGetEnum("ScopeKind" ,"classifier_level").equals(
             content.refGetValue("scope")
            ) && Boolean.FALSE.equals(content.refGetValue("isDerived"))
          ) { 
             int i = classAttNames.indexOf(contentName);
             classAttMultiplicities[i] = getMultiplicity(content);             
          } 
        } 
      }       
    }
    
    void readContent_with_Inheritance() {    
      Iterator it;
      it = getContentsInAllSupertypes(metaObject).iterator();
      while(it.hasNext()) {
        RefObject content = (RefObject)it.next();
        String contentName = (String) content.refGetValue("name");
        String contentType = (String) content.refMetaObject().refGetValue("name");
                
        if(contentType.equals("Attribute")) {

        //  if(!isM3()) System.out.println(this +"." +contentName +":" +contentType );            
          
          if( content.refImmediatePackage().refGetEnum("ScopeKind" ,"instance_level").equals(
             content.refGetValue("scope")
            ) && Boolean.FALSE.equals(content.refGetValue("isDerived"))
          ) { 
            attNames.add(contentName);
          } 
        } else if(contentType.equals("Reference")) {
          refNames.add(contentName);
        } 
      }
      // Instance-level Attributes
      attMultiplicities = new int[attNames.size()];
      
      // References
      assoNames = new List[refNames.size()]; 
      assoEndNames = new String[refNames.size()];
      refMultiplicities = new int[refNames.size()];      
      
      it = getContentsInAllSupertypes(metaObject).iterator();
      while(it.hasNext()) {
        RefObject content = (RefObject)it.next();
        String contentName = (String) content.refGetValue("name");
        String contentType = (String) content.refMetaObject().refGetValue("name");
        
        if(contentType.equals("Attribute")) {
          if( content.refImmediatePackage().refGetEnum("ScopeKind" ,"instance_level").equals(
             content.refGetValue("scope")
            )  && Boolean.FALSE.equals(content.refGetValue("isDerived"))
          ) { 
             int i = attNames.indexOf(contentName);
             attMultiplicities[i] = getMultiplicity(content);
          } 
          
        } else if(contentType.equals("Reference")) {
           int i = refNames.indexOf(contentName); 
           refMultiplicities[i] = getMultiplicity(content);
           RefObject assoEnd = (RefObject) content.refGetValue("referencedEnd"); 
           assoEndNames[i] = (String) assoEnd.refGetValue("name");
           RefObjectImpl asso = (RefObjectImpl) metaObject.refImmediatePackage().refAssociation("Contains").refQuery("containedElement", assoEnd).toArray()[0];
           assoNames[i] = ReflectHelper.getQualifiedName(asso);       
        } 
      }      
    }
    
    
    
    // implements RefFeature
    
    public void refSetValue(String str, Object obj) {
        int i = classAttNames.indexOf(str);
        if(i>=0) {
          if(classAttMultiplicities[i]!=1) {
            throw new RuntimeException("Cannot do refSetValue with multiple-value " +"'"+ str +"' in " +this );
          }             
          classAttValues[i] = obj;
          return;
        }
        Iterator it = directSupertypes().iterator();
        while(it.hasNext()) {
          try {
            RefClassImpl superClass = (RefClassImpl) it.next();
            superClass.refSetValue(str, obj);
            return;
          } catch(InvalidNameException e) {  
          }
        }
        throw new InvalidNameException("'"+ str +"' not found in " +this );
    }
    
    public Object refGetValue(String str) {
        int i = classAttNames.indexOf(str);
        if(i>=0) {
          return classAttValues[i];
        }
        Iterator it = directSupertypes().iterator();
        while(it.hasNext()) {
          try {
            RefClassImpl superClass = (RefClassImpl) it.next();
            return superClass.refGetValue(str);
          } catch(InvalidNameException e) { 
          	// then try another supertype 
          }
        }        
        throw new InvalidNameException("'"+ str +"' not found in " +this );
    }  
    
	public Object refGetValue(javax.jmi.reflect.RefObject refObject) {
		String name = (String) refObject.refGetValue("name");
		return refGetValue(name);
	}
    
	public void refSetValue(javax.jmi.reflect.RefObject refObject, Object obj) {
		String name = (String) refObject.refGetValue("name");
		refSetValue(name, obj);
	}    
    
	public Object refInvokeOperation(javax.jmi.reflect.RefObject refObject, java.util.List list) 
								throws javax.jmi.reflect.RefException {
		String name = (String) refObject.refGetValue("name");
		return refInvokeOperation(name, list);
	}    
    
	transient List opNames = null;
	transient Method[] ops = null;
    
	// the operation names must be distinct
	public Object refInvokeOperation(String str, java.util.List list) throws javax.jmi.reflect.RefException {
		if(opNames==null) loadOperations();
		Object r;
		Method m = ops[ opNames.indexOf(str) ];
		try {
			r = m.invoke(this, list.toArray());
			return r;
		} catch(Exception e) {
		  throw new RuntimeException(e);
		}
	}
    
	void loadOperations() {
		ops = getClass().getMethods();
		opNames = new Vector();
		for(int i=0; i<ops.length; i++) {
		  opNames.add(ops[i].getName());
		}
	}
             
    ///////// be be overwriten ///////////////////          
    public RefObjectImpl newObject() {
      return new RefObjectImpl();
    }
    
    
}
