package org.objectweb.modfact.jmi.reflect;

import java.lang.reflect.*;
import javax.jmi.reflect.*;
import java.util.*;

public class RefObjectImpl extends RefBaseObjectImpl
                implements javax.jmi.reflect.RefObject, java.io.Serializable {
    
    RefClassImpl classProxy;
    
    // Attritbute                                
    Object[] attValues = null;
 
    public RefObjectImpl() {
    }
    
    void setArgs(List args) {
		if(args!=null) {
		   attValues = args.toArray();
		} else {
		   attValues = new Object[classProxy.attNames.size()];
		   for(int i=0; i<attValues.length; i++ ) {
			 if(classProxy.attMultiplicities[i]!=1)  
				attValues[i] = new Vector(); 
		   }             
		}        
    }
    
    
    public javax.jmi.reflect.RefClass refClass() {
        return classProxy;
    }    

    public void refDelete() {
        throw new RuntimeException("No implementation: RefObject.refDelete");
    }      
    
    public boolean refIsInstanceOf(javax.jmi.reflect.RefObject meta, boolean considerSubtype) {
        RefObject instanceType = refMetaObject();
        if(considerSubtype == false) {
            return instanceType.equals(meta);
        }
        return isSubType(instanceType, meta);
    }
    
	public String toString() {
	  String type = super.toString();

	  int i = classProxy.attNames.indexOf("name");
	  if(i>=0) {
	  	return type + (String) attValues[i];
	  }
	  return type;  
	}    
    
    boolean isSubType(RefObject sub, RefObject sup) {
        if(sub.equals(sup)) return true;
        Iterator it = ((Collection)sub.refGetValue("supertypes")).iterator();
        while(it.hasNext()) {
           RefObject superType = (RefObject)it.next();
           if( isSubType(superType,sup))  return true;
        }
        return false;        
    }
    

    public javax.jmi.reflect.RefFeatured refImmediateComposite() {
       RefObject meta = refMetaObject();

	   // First, find aggregated association that associate this element with its composite
       //   To do this :
       //   Navivation in meta level: metaobject(type)-(typed)AssociationEnd(contained)-(container)Association(container)-(contained)AssociationEnd[aggregation=composite]
       // then, we get the association from association meta-object
       // then we use this association to navigate to the composite object in instance level
       Iterator it = queryInAllSupertypes("IsOfType", "type", meta).iterator();
       while(it.hasNext()) {
         RefObject typedElement = (RefObject)it.next();
         if(typedElement.refMetaObject().refGetValue("name").equals("AssociationEnd")) {
           RefObject thisEnd = typedElement;
           RefObject oppositeEnd = getOppositeAssociationEnd(thisEnd);
           if(oppositeEnd.refGetValue("aggregation").toString().equals("composite")) {
             RefObjectImpl assoMeta = (RefObjectImpl) thisEnd.refGetValue("container");  
             Object asso =  ReflectHelper.resolveQualifiedNameInExtent(
             	this, ReflectHelper.getQualifiedName(assoMeta)
             );
             // System.err.println(asso.getClass());
             Collection c = (Collection) ((RefAssociation)asso).refQuery(thisEnd, this);
             if(!c.isEmpty()) 
             	return (RefFeatured)c.toArray()[0];
           }
         }
       }
       return null;
    }   
    
    
    public javax.jmi.reflect.RefFeatured refOutermostComposite() {
        RefFeatured r = this;
        RefFeatured p = this.refImmediateComposite(); 
        while((p!=null)&&(p instanceof RefObject)) {
            r = p;
            p = ((RefObject) p).refImmediateComposite(); 
        }
        return (p==null)? r:p;        
    }
    
    public javax.jmi.reflect.RefObject refMetaObject() {
        if(metaObject!=null) return metaObject;
        return findObjectByName( container.refClass("Class"), name);
    }    
    
    // implements RefFeature
    
    public void refSetValue(String str, Object obj) {
        if(!container.canModify) {
            throw new RuntimeException("Not allowed to modify");
        }
        int i = classProxy.attNames.indexOf(str);
        if(i>=0) {
          if(classProxy.attMultiplicities[i]!=1) {
            throw new RuntimeException("Cannot do refSetValue with multivalued " +"'"+ str +"' in " +this );
          }
          // System.out.println(this +".refSetvalue " +str +"=" +obj );
          attValues[i] = obj;
          return;
        }
        i = classProxy.refNames.indexOf(str);
        if(i>=0) {
          if(classProxy.refMultiplicities[i]!=1) {
            throw new RuntimeException("Cannot do refSetValue with multiple-value " +"'"+ str +"' in " +this );
          }              
          RefAssociationImpl a = (RefAssociationImpl) classProxy.associations()[i];

          if(obj!=null) {
            if(a.firstEndName.equals(str)) {
              a.refAddLink( (RefObject)obj, this );
            } else {
              a.refAddLink(this, (RefObject)obj );
            }
          } else {
            //remove old associated object
            RefObject old = (RefObject) refGetValue(str);  
            if(old==null) return;
            if(a.firstEndName.equals(str)) {
              a.refRemoveLink( old, this );
            } else {
              a.refRemoveLink(this, old );
            }          
          }
          return;
        }
        System.err.println("'"+ str +"' not found in " +this );        
        throw new InvalidNameException("'"+ str +"' not found in " +this );
    }
    
    public Object refGetValue(String str) {

        int i = classProxy.attNames.indexOf(str);
        if(i>=0) {
          if( (!container.canModify) && (attValues[i] instanceof Collection) )  
              return new Vector((Collection)attValues[i]);
          else
              return attValues[i];
        }
        i = classProxy.refNames.indexOf(str);
        if(i>=0) {
          RefAssociationImpl a = classProxy.associations()[i];
          Collection c = a.refQuery( a.getOtherEndName(classProxy.assoEndNames[i]), (RefObject)this);
          if(classProxy.refMultiplicities[i]==1) {
            return c.isEmpty()? null : c.toArray()[0];
          } 
          return c;
        }
        
		if(str.equals("qualifiedName")) // this attribute has "isDerived"=true
		  return ReflectHelper.getQualifiedName(this);         
        //System.err.println("'"+ str +"' not found in " +this );
        throw new RuntimeException("'"+ str +"' not found in " +this );
    }  

       
	public Object refGetValue(javax.jmi.reflect.RefObject refObject) {
		String name = (String) refObject.refGetValue("name");
		return refGetValue(name);
	}
    
	public void refSetValue(javax.jmi.reflect.RefObject refObject, Object obj) {
		String name = (String) refObject.refGetValue("name");
		refSetValue(name, obj);
	}    
    
	public Object refInvokeOperation(javax.jmi.reflect.RefObject refObject, java.util.List list) 
								throws javax.jmi.reflect.RefException {
		String name = (String) refObject.refGetValue("name");
		return refInvokeOperation(name, list);
	}    
    
   
	// the operation names must be distinct
	public Object refInvokeOperation(String str, java.util.List list) throws javax.jmi.reflect.RefException {
		Object r;
		Method m = ReflectHelper.getMethod(this.getClass(), str);		
		try {
			if(m==null) throw new RuntimeException("operation not found:" +str);
			r = m.invoke(this, list.toArray());
			return r;
		} catch(Exception e) {
			// try M2 operation implementations in reflective repository
			try {
			  m = ReflectHelper.getMethod(ReflectHelper.class,name);
			  if(m==null){
			    if(e instanceof RuntimeException) throw (RuntimeException) e;
			    throw new RuntimeException(e);
			  } 
			    List l2 =	new Vector();
				l2.add(this);
				l2.addAll(list);		  			  
				r = m.invoke(this, l2.toArray());
				return r;				
			} catch(Exception e2) {
				if(e2 instanceof RuntimeException) throw (RuntimeException) e2;
				throw new RuntimeException(e2);	  
			}			
		}
	}
    
	
	// use for initialize M3
	void setAttValue(String name, Object value) {
	   if(name.equals("qualifiedName")) // this attribute has "isDerived"=true
		   return;  
	   int i = classProxy.attNames.indexOf(name);
	   attValues[i] = value; 
	}      
   
	
//	////////////////////////////////////////////////////////////
//	// implementation of some M3 operations
//	//
//	// ModelElement Namespace.resolveQualifiedName(List qualifiedName)
//	public RefObject _resolveQualifiedName(List qualifiedName) {
//	   return ReflectHelper.resolveQualifiedName(this, qualifiedName);
//	}   
//	// ModelElement Namespace.lookupElement(String name)
//	public RefObject _lookupElement(String name) {
//		return ReflectHelper.lookupElement(this, name);
//	}    
//	// ModelElement GeneralizableElement.lookupElementExtended(String name)
//	public RefObject _lookupElementExtended(String name) {
//		return ReflectHelper.lookupElementExtended(this, name);
//	}
//	// List GeneralizableElement.allSupertypes() 
//	public List _allSupertypes() {
//	   return ReflectHelper.allSupertypes(this); 
//	}    
//	// derived attribute in ModelElement
//	public List _getQualifiedName() {
//	  return ReflectHelper.getQualifiedName(this);
//	}

	
    
}
