package org.objectweb.modfact.jmi.reflect;

import java.util.List;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.jmi.reflect.RefAssociationLink;
import javax.jmi.reflect.RefBaseObject;
import javax.jmi.reflect.RefObject;
import javax.jmi.reflect.RefPackage;
import javax.jmi.xmi.MalformedXMIException;
import javax.jmi.xmi.XmiReader;
import javax.jmi.xmi.XmiWriter;

import org.objectweb.modfact.jmi.xmi.XmiReaderImpl;
import org.objectweb.modfact.jmi.xmi.XmiWriterImpl;

public class RefPackageFactory {

	public XmiReader xmiReader = new XmiReaderImpl();
	public XmiWriter xmiWriter = new XmiWriterImpl();

	public RefPackage createM2() {
		return RefPackageImpl.getM2Repository();
	}

	public RefPackage createM2(String m2url)
		throws MalformedXMIException, IOException {

		RefPackage p = RefPackageImpl.getM2Repository();
		xmiReader.read(m2url, p);		
		return p;
	}
	
	public RefPackage createM1(String m2url)
		throws MalformedXMIException, IOException {
		RefPackage m2Extent = createM2(m2url);
		Collection outermostMetaPackages = ReflectHelper.getOuterMostMetaPackages(m2Extent);		
		return createM1(outermostMetaPackages);
	}	

	public RefPackage createM1(Collection outermostMetaPackages) {
		
		Map extentMap = createExtentMap(outermostMetaPackages);
		Collection topIndependantExtents = setImports(outermostMetaPackages, extentMap);

		if (topIndependantExtents.size() == 1) {
			return (RefPackage) topIndependantExtents.iterator().next();		
		}
		
		System.err.println("[warning] found " +topIndependantExtents.size() +" extents: "+ topIndependantExtents );
		
		// return the package that contains at least one class
		Iterator it = topIndependantExtents.iterator();
		while(it.hasNext()) {
			RefPackage extent = (RefPackage) it.next();
			if(!extent.refAllClasses().isEmpty()) {
				return extent;
			}
		}
		return (RefPackage) topIndependantExtents.iterator().next();
	}
	

	public Map createM1s(String m2url)
		throws MalformedXMIException, IOException {
		RefPackage m2Extent = createM2(m2url);
		Collection outermostMetaPackages = ReflectHelper.getOuterMostMetaPackages(m2Extent);		
		return createM1s(outermostMetaPackages);	
	}	

	//return extent map
	public Map createM1s(Collection outermostMetaPackages)
		throws MalformedXMIException, IOException {
			
		if(outermostMetaPackages.isEmpty()) {  
			throw new RuntimeException("NoPackageDefined"); 
		} 
		Map extentMap = createExtentMap(outermostMetaPackages);
		setImports(outermostMetaPackages, extentMap);
		
		return extentMap;
	}
	
	private Map createExtentMap(Collection outermostMetaPackages) {
		// create extent for each outermost packages
		// map outermost metaPackage to extent
		Map extentMap = new Hashtable();	
		Iterator it = outermostMetaPackages.iterator();
		while (it.hasNext()) {
			RefObject metap = (RefObject) it.next();			
			RefPackage extent = RefPackageImpl.getM1Repository(metap);
			extentMap.put(metap.refGetValue("name"), extent);
		}
		return extentMap;		
	}
	
	
	// return top independant extents
	private Collection setImports(Collection outermostMetaPackages, Map extentMap) {
		Set topIndependantExtents = new HashSet(extentMap.values());

		// links imported extents
		RefPackage m2Extent = ((RefObject) outermostMetaPackages.iterator().next()).refOutermostPackage();			
		Iterator it = m2Extent.refAssociation("Aliases").refAllLinks().iterator();
		while (it.hasNext()) {
			RefAssociationLink l = (RefAssociationLink) it.next();
			RefObject importObject = l.refFirstEnd();
			RefObjectImpl importerPackage =
				(RefObjectImpl) importObject.refGetValue("container");
			//verify that the importer is a package
			if (!"Package"
				.equals(importerPackage.refMetaObject().refGetValue("name"))) {
				continue;
			}
			List importerPackageName = ReflectHelper.getQualifiedName(importerPackage);

			RefPackageImpl importerExtent =( RefPackageImpl) ReflectHelper.resolveQualifiedNameInExtents(extentMap, importerPackageName);
			if(importerExtent==null) {
				continue;
			}
			
			RefObjectImpl importedMetaObject = (RefObjectImpl) l.refSecondEnd();
			String importedObjectType = ReflectHelper.getType(importedMetaObject);
			
			List importedObjectName = ReflectHelper.getQualifiedName(importedMetaObject);

			RefBaseObject importedNs = ReflectHelper.resolveQualifiedNameInExtents(extentMap, importedObjectName);
			if(importedNs==null) {
				continue;
			}
			
			String unqname =
				(String) importedObjectName.get(importedObjectName.size() - 1);
			if ("Package".equals(importedObjectType)) {
				importerExtent.setImportedExtent(
					unqname,
					(RefPackageImpl) importedNs);
			} else if ("Class".equals(importedObjectType)) {
				importerExtent.setImportedClassProxy(
					unqname,
					(RefClassImpl) importedNs);
			}
			topIndependantExtents.remove(importedNs);
		}
		return topIndependantExtents;		
		
	}

	public RefPackage createM1(String m2url, String m1url)
		throws MalformedXMIException, IOException {

		RefPackage p = createM1(m2url);
		xmiReader.read(m1url, p);
		return p;
	}

}
