package org.objectweb.modfact.jmi.reflect;

import javax.jmi.reflect.*;

import java.lang.reflect.Method;
import java.util.*;

public class ReflectHelper {
	
	public static String getType(RefObject o) {
		return (String) o.refMetaObject().refGetValue("name");
	}

	public static RefObject findObjectByName(Collection col, String name) {
		Iterator it = col.iterator();
		while(it.hasNext()) {
			RefObject content = (RefObject) it.next();
			if(content.refGetValue("name").equals(name)) {
				return content;
			}
		}
		throw new RuntimeException("ElementNotFound name='" +name);  
	}

	public static RefObject findObjectByName(RefClass c, String name) {
		return findObjectByName(c.refAllOfClass(), name);		  
	}	
	
    
	public static List filterContentsByType(RefObject container
				, String type
				, boolean considerSupertypes) {
	  Collection c = (considerSupertypes)? 
							  getContentsInAllSupertypes(container)
							: (Collection) container.refGetValue("contents");
	  Iterator it = (c).iterator();
	  List r = new Vector();
	  while(it.hasNext()) {
		RefObject content = (RefObject) it.next();
		if(content.refMetaObject().refGetValue("name").equals(type)) {
			r.add(content);
		}
	  }
	  return r;    
	}
     
	public static List getContentsInAllSupertypes(RefObject container) {
	   List r = new DistinctList();
	   Iterator it = ((List) container.refGetValue("supertypes")).iterator();
	   while(it.hasNext()) {
		 RefObject sup = (RefObject) it.next();
		 r.addAll((List)getContentsInAllSupertypes(sup));
	   }
	   r.addAll((List)container.refGetValue("contents"));       
	   return r;
	}
	
	
	// find the othermost Package objects
	public static Collection getOuterMostMetaPackages(RefPackage metamodel) {
	  Iterator it =  metamodel.refClass("Package").refAllOfClass().iterator();
	  Set outermosts = new HashSet();	  
	  // filter outermost packages
	  while(it.hasNext()) {
	  	RefObject p = (RefObject) it.next();
	  	outermosts.add(p.refOutermostComposite());
	  }
	  return outermosts;
	}
		
	// find all elements associated to this one and to any of its supertypes    
	static Collection queryInAllSupertypes(String assoName, String endName, RefObject end ) {
	  return queryInAllSupertypes(
			end.refImmediatePackage().refAssociation(assoName)
			, endName ,end );
	}
    
	// find all elements associated to this one and to any of its supertypes
	static Collection queryInAllSupertypes(RefAssociation a, String endName, RefObject end ) {
	   Collection r = new DistinctList();
	   Iterator it = ((Collection) end.refGetValue("supertypes")).iterator();
	   while(it.hasNext()) {
		 RefObject sup = (RefObject) it.next();
		 r.addAll( queryInAllSupertypes(a, endName, sup) );
	   }
	   r.addAll(a.refQuery(endName,end));       
	   return r;
	}    
    
    
	public static RefObject getOppositeAssociationEnd(RefObject end) {
	  RefObject asso = (RefObject) end.refGetValue("container");
	  Iterator it = ((Collection)asso.refGetValue("contents")).iterator();
	  while(it.hasNext()) {
		RefObject content = (RefObject) it.next();
		if( !content.equals(end)
		  && content.refMetaObject().refGetValue("name").equals("AssociationEnd")) 
			return content;
	  }
	  return null;
	}
  
    
	public static int getMultiplicity(RefObject meta) {
	  return ((Integer)((RefStruct)meta.refGetValue("multiplicity")).refGetValue("upper")).intValue();
	}    
    
	public static List arrayToList(Object[] array) {
	  List l = new Vector();
	  for(int i=0; i<array.length; i++) {
		l.add(array[i]);
	  }
	  return l;
	}
    
    
    public static RefBaseObject resolveQualifiedNameInExtents(Map extentMap,  List qName) {
		String extentName = (String) qName.get(0);
		RefPackage extent = (RefPackage) extentMap.get(extentName);
		if(extent==null) {
			return null; 			
		}
		return resolveQualifiedNameInExtent((RefPackageImpl)extent ,qName ,1);
    }
	
	
	//lookup for contained RefClass, RefPackage, RefAssociation	
	public static RefBaseObject resolveQualifiedNameInExtent(RefBaseObject extent, List qName) {
	  if(qName.isEmpty()) throw new RuntimeException("invalide argument: empty qName");
	  RefPackageImpl p = (RefPackageImpl) extent.refOutermostPackage();
	  if(!p.name.equals(qName.get(0))) { 
	  	throw new RuntimeException("invalide root package '"+ qName.get(0) +"'"); 
	  } 
	  return resolveQualifiedNameInExtent(p ,qName ,1);
	}
	
	private static RefBaseObject resolveQualifiedNameInExtent(RefPackageImpl p, List qName, int index) { 
	  if(index >= qName.size()) return p;	  
	  String currentNameItem = (String)qName.get(index);
	  if(p.classNames.contains(currentNameItem)) {
		return p.refClass(currentNameItem);	    
	  }
	  if(p.associationNames.contains(currentNameItem)) {
		return p.refAssociation(currentNameItem);	    
	  }	  
	  return resolveQualifiedNameInExtent(
		  (RefPackageImpl)p.refPackage(currentNameItem) ,qName ,index+1);	
	} 	
    
	////////////////////////////////////////////////////////////
	// implementation of some M3 operations
	/////////////////////////////////////////////////////////////
    
	// ModelElement Namespace.resolveQualifiedName(List qualifiedName)
	public static RefObject resolveQualifiedName(RefObject context, List qualifiedName) {
	   RefObject lookupElement = lookupElement(context, (String)qualifiedName.get(0));
	   if(qualifiedName.size()==1) {
		 return lookupElement;
	   }
	   return ReflectHelper.resolveQualifiedName(
	   		lookupElement, qualifiedName.subList(1,qualifiedName.size()) 
	   );
	}
    
	// ModelElement Namespace.lookupElement(String name)
	public static RefObject lookupElement(RefObject context, String name) {
		Iterator it = ((Collection) context.refGetValue("contents")).iterator();
		while(it.hasNext()) {
		  RefObject content = (RefObject)it.next();
		  if(content.refGetValue("name").equals(name)) {
			 return content;
		  }
		}
		throw new RuntimeException(context +": Name not found: " +name);
	}
    
	// ModelElement GeneralizableElement.lookupElementExtended(String name)
	public static RefObject lookupElementExtended(RefObject context, String name) {
		Iterator it = getContentsInAllSupertypes(context).iterator();
		while(it.hasNext()) {
		  RefObject content = (RefObject)it.next();
		  if(content.refGetValue("name").equals(name)) {
			 return content;
		  }
		}
		// for debugging
		/*
		System.out.println("debug info...");
		it = getContentsInAllSupertypes(this).iterator();
		while(it.hasNext()) {
		  RefObject content = (RefObject)it.next();
		  System.out.println(content.refGetValue("name"));
		}
		*/		
		throw new RuntimeException("Name not found '" +name +"' in namespace " +getQualifiedName(context)  );
	}

	// List GeneralizableElement.allSupertypes() 
	public static List allSupertypes(RefObject context) {
	   List r = new DistinctList();
	   Iterator it = ((List)context.refGetValue("supertypes")).iterator();
	   while(it.hasNext()) {
		 RefObjectImpl sup = (RefObjectImpl) it.next();
		 r.addAll( ReflectHelper.allSupertypes(sup) );   
		 r.add(sup);
	   }    
	   return r; 
	}
    
	// derived attribute in ModelElement
	public static List getQualifiedName(RefObject context) {
	  // *** NOT to be confised with RefPackage RefBaseObject_impl.container  
	  RefObjectImpl containerObject = (RefObjectImpl) context.refGetValue("container");
	  List r = null;
	  if(containerObject!=null) {
		r = ReflectHelper.getQualifiedName(containerObject);
	  } else {
		r = new Vector();
	  }
	  r.add(context.refGetValue("name"));
	  return r;
	}    

	public static Method getMethod(Class clazz, String name) {
		Method[] ms = clazz.getMethods();
		for(int i=0; i<ms.length; i++) {
			if(ms[i].getName().equals(name)) {
				return ms[i];
			}
		}
		return null;
	}
	
	public static Collection findTagsFor(RefObject modelElement) {
	  RefAssociation asso = (RefAssociation) modelElement.refOutermostPackage().refAssociation("AttachesTo");   
	  return asso.refQuery("modelElement",modelElement);
	}
    
	public static String findTagValue(RefObject modelElement, String tagId) {
	 try {
	   Iterator it = findTagsFor(modelElement).iterator();	
	   while(it.hasNext()) {
		 RefObject t = (RefObject) it.next();
		 if(tagId.equalsIgnoreCase((String)t.refGetValue("tagId"))) {
		   // System.out.println(t.refGetValue("values"));	
		   return (String) ((Collection)t.refGetValue("values")).toArray()[0];
		 } 
	   }
	 } catch(Exception e) {
	   e.printStackTrace();
	 }
	 return null;
	}	

	public static String dotQName(List l) {
		Iterator it = l.iterator();
		String r = (String) it.next();
		while(it.hasNext()) {
			r = r +"." +it.next();
		}
		return r;
	}	

	public static String unqualifiedName(String dotList) {
		dotList = dotList.substring(
			dotList.lastIndexOf(":")+1 ,dotList.length()
		);
		return dotList.substring(
			dotList.lastIndexOf(".")+1 ,dotList.length()
		);
	}
	
	public static String getName(RefObject o) {
		return (String) o.refGetValue("name");
	}
	    
	public Map getExtentMaps(Collection extents) {
		Iterator it = extents.iterator();
		Map result = new Hashtable();
		while(it.hasNext()) {
			RefPackage extent = (RefPackage)it.next();
			String name = getName(extent.refMetaObject());
			result.put(name, extent);
		}
		return result;    
	}		

}
