/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.service;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import javax.jmi.model.ModelPackage;
import javax.jmi.model.MofPackage;
import javax.jmi.xmi.XmiReader;

// Imports for the logging
import org.objectweb.modfact.jmi.logging.ModFactLogger;

// Imports for the generators
import org.objectweb.modfact.jmi.impl.JMI1_0ImplementationGenerator;
import org.objectweb.modfact.jmi.api.JMI1_0Generator;
import org.objectweb.modfact.jmi.xmiio.exporter.JMI1_0ExportGenerator;
import org.objectweb.modfact.jmi.xmiio.importer.JMI1_0ImportGenerator;

// Imports for the MOF Repository
import org.objectweb.modfact.jmi.xmi.XmiReaderImpl;
import org.objectweb.modfact.jmi.reflect.RefPackageImpl;



public class Generation {;

    /** The all MOF Packages. */
    protected MofPackage[] allPackages;
        
    /**
     * Generates all the selected items.
     * @param metaModelFile The metamodel file.
     * @param apiFile The API Zip file.
     * @param implFile The Implementation Zip file.
     * @param importFile The XMI Importer Zip file.
     * @param exportFile The XMI Exporter Zip file.
     */
    public void execute(InputStream metaModelFile, OutputStream apiFile, OutputStream implFile, OutputStream importFile, OutputStream exportFile)
        throws java.io.IOException, javax.jmi.xmi.MalformedXMIException {
        
        ModelPackage modelPackage;
        modelPackage = (ModelPackage) RefPackageImpl.getM2Repository();
        // input XML model
        XmiReader reader = new XmiReaderImpl();
        reader.read(metaModelFile, "", modelPackage);
        setMetaModel(modelPackage);
        
        if(apiFile!=null) {
            generateAPIs(apiFile);
        }
        if(implFile!=null) {
            generateImpl(implFile);   
        }
        if(importFile!=null) {
            generateImport(importFile);   
        }
        if(exportFile!=null) {
            generateExport(exportFile);   
        }
    }
    
	/**
	 * Generates all the selected items.
	 * @param metaModelFile The metamodel file.
	 * @param apiFile The API Zip file.
	 * @param implFile The Implementation Zip file.
	 * @param importFile The XMI Importer Zip file.
	 * @param exportFile The XMI Exporter Zip file.
	 */
    public void execute(String metaModelFile, String apiFile, String implFile, String importFile, String exportFile)
        throws java.io.IOException, javax.jmi.xmi.MalformedXMIException {
        
       ModelPackage modelPackage;
       modelPackage = (ModelPackage) RefPackageImpl.getM2Repository();
       // input XML model
       XmiReader reader = new XmiReaderImpl();
       reader.read(new FileInputStream(metaModelFile), "", modelPackage);
       setMetaModel(modelPackage);
        
       if(apiFile!=null) {
           generateAPIs(new FileOutputStream(apiFile));
       }
       if(implFile!=null) {
           generateImpl(new FileOutputStream(implFile));   
       }
       if(importFile!=null) {
           generateImport(new FileOutputStream(importFile));   
       }
       if(exportFile!=null) {
           generateExport(new FileOutputStream(exportFile));   
       }
    }
    
    public void setMetaModel(ModelPackage modelPackage) {
        allPackages = (MofPackage[]) modelPackage.getMofPackage().refAllOfType().toArray(new MofPackage[0]);        
    }
    
    public void generateAPIs(OutputStream out)
        throws java.io.IOException {
        ModFactLogger logger =
            ModFactLogger.getLogger("fr.lip6.modfact.generator.jmi.JMI1_0Generator");

        JMI1_0Generator jmiGenerator = new JMI1_0Generator();
        jmiGenerator.setLogger(logger);
        jmiGenerator.setInput(allPackages);
        jmiGenerator.setOutput(out);
        jmiGenerator.generate();
        //System.out.println("JMI APIs generated");
    }

    public void generateImpl(OutputStream out)
        throws java.io.IOException {
        ModFactLogger logger =
            ModFactLogger.getLogger(
                "fr.lip6.modfact.generator.jmi_impl.JMI1_0Generator_impl");
        JMI1_0ImplementationGenerator jmiGenerator = new JMI1_0ImplementationGenerator();
        jmiGenerator.setLogger(logger);
        jmiGenerator.setInput(allPackages);
        jmiGenerator.setOutput(out);
        jmiGenerator.generate();
        //System.out.println("JMI APIs implemented");
    }

    public void generateImport(OutputStream out)
        throws java.io.IOException {

        ModFactLogger logger =
            ModFactLogger.getLogger(
                "fr.lip6.modfact.generator.jmi_impl.JMI1_0ImportGenerator_impl");

        JMI1_0ImportGenerator jmiImportGenerator = new JMI1_0ImportGenerator();
        jmiImportGenerator.setLogger(logger);
        jmiImportGenerator.setInput(allPackages);
        jmiImportGenerator.setOutput(out);
        jmiImportGenerator.generate();
        //System.out.println("JMI Importers generated");
    }

    public void generateExport(OutputStream out)
        throws java.io.IOException {

        ModFactLogger logger =
            ModFactLogger.getLogger(
                "fr.lip6.modfact.generator.jmi_impl.JMI1_0ExportGenerator_impl");

        JMI1_0ExportGenerator jmiExportGenerator = new JMI1_0ExportGenerator();
        jmiExportGenerator.setLogger(logger);
        jmiExportGenerator.setInput(allPackages);
        jmiExportGenerator.setOutput(out);
        jmiExportGenerator.generate();
        //System.out.println("JMI Exporters generated");
    }
    
}
