/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.service.cmdline.client;

import org.objectweb.modfact.jmi.service.GenerationService;
import org.objectweb.util.cmdline.lib.DefaultOptionBase;

/**
 * Class representing the option to set the XMI file to read.
 */
public class OptionImport extends DefaultOptionBase {

    /**
     * Reference to the Import Application. 
     */
    protected GenerationService application_;

    /**
     * The constructor with the owner Import Application.
     * @param application Reference to the XMI Import Application.
     */
    public OptionImport(GenerationService application) {
        // Calls the DefaultOptionBase constructor.
        super();
        setLabels(new String[] { "-import" });
        setArguments(new String[] { "file.zip" });
        setDescription(new String[] { "Set the Zip file which will contain the XMI importer to write" });
        setMandatory(false);

        // Inits internal state.
        application_ = application;
    }

    // ==================================================================
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     * @param iterator The command line argument iterator.
     */
    public void consume(org.objectweb.util.cmdline.api.Iterator iterator) {
        checkAlreadySet(iterator);
        String file = consumeArgument(iterator);
        application_.setImportFile(file);
    }

}
