package org.objectweb.modfact.jmi.xmi;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jmi.reflect.*;

import org.objectweb.modfact.jmi.reflect.ReflectHelper;


/**
 *  Create RefObject, RefStruct, RefEnum from 
 * 		   dot qualified name
 * 		or unqualified name 
 * 		or qualified name (List)
 */
public class ObjectFactoryManager {
	
	Map classProxies = new Hashtable();
	Map structFactories = new Hashtable();
	Map enumManagers = new Hashtable();
	Map assoProxies = new Hashtable();	

	public ObjectFactoryManager(RefPackage outermostExtent) {
		cacheAllFactories(outermostExtent);
	}

	private void cacheAllFactories(RefPackage p) {
		Iterator it = ((Collection) p.refMetaObject().refGetValue("contents")).iterator();
		while(it.hasNext()) {
		  RefObject content = (RefObject) it.next();
		  String name = (String) content.refGetValue("name");
		  String qname = ReflectHelper.dotQName(ReflectHelper.getQualifiedName(content));
		  String type = (String) content.refMetaObject().refGetValue("name");
		  if("Class".equals(type)) {
			classProxies.put(name, p.refClass(name));	      
			classProxies.put(qname, p.refClass(name));	
						
			cacheAllFactories(p.refClass(name));	  
		  
		  } else if("StructureType".equals(type)) {
			structFactories.put(name, p);
			structFactories.put(qname, p);			
			
		  } else if("EnumerationType".equals(type)) {
			enumManagers.put(name, p);
			enumManagers.put(qname, p);
			
		  } else if("Association".equals(type)) {
			assoProxies.put(name, p.refAssociation(name));	
			assoProxies.put(qname, p.refAssociation(name));	
						
		  } else if("Package".equals(type)) {
			cacheAllFactories(p.refPackage(name));	
				  	
		  } else if("Import".equals(type)) {
			try {
			  RefObject ns = (RefObject) content.refGetValue("importedNamespace");
			  String nsName = (String) ns.refGetValue("name");
			  String nsQName = ReflectHelper.dotQName(ReflectHelper.getQualifiedName(ns));
			  String nsType = (String) ns.refMetaObject().refGetValue("name");
			  if("Package".equals(nsType)) {
				  cacheAllFactories(p.refPackage(nsName));	
				  				
			  } else if("Class".equals(nsType)){
				  RefClass c = p.refClass(nsName);
				  classProxies.put(nsName, c);	 
				  classProxies.put(nsQName, c);					  
				       
				  cacheAllFactories(c);				
			  }
			  
			} catch(Exception e) {
			  // e.printStackTrace();
			}	      	
		  }
		}
	  }
    
	  private void cacheAllFactories(RefClass c) {
		  Iterator it = ((Collection) c.refMetaObject().refGetValue("contents")).iterator();
		  while(it.hasNext()) {
			RefObject content = (RefObject) it.next();
			String name = (String) content.refGetValue("name");
			String qname = ReflectHelper.dotQName(ReflectHelper.getQualifiedName(content));
			String type = (String) content.refMetaObject().refGetValue("name");
			if("StructureType".equals(type)) {
			  structFactories.put(name, c);
			  structFactories.put(qname, c);
			  			  
			} else if("EnumerationType".equals(type)) {
			  enumManagers.put(name, c);
			  enumManagers.put(qname, c);
			  			
			}	      	
		  }    	    	
	  }
	
	public RefClass getRefClass(String name) {
		name = 	name.substring(name.indexOf(":")+1);		
		return (RefClass) classProxies.get(name);
	}
	  
	public boolean canCreateObject(String name) {
		name = 	name.substring(name.indexOf(":")+1);
		return classProxies.containsKey(name);
	}

	public RefObject createObject(String name) {
		name = 	name.substring(name.indexOf(":")+1);	
		RefClass c = (RefClass)classProxies.get(name);
		if(c==null) {
			System.err.println("warning: class '" +name +"' undefined");			
			return null;
		}
		RefObject o = c.refCreateInstance(null);		
		return o;		
	}
		
	
	public RefEnum getEnum(String name, String value) {
		name = 	name.substring(name.indexOf(":")+1);		
		Object m = enumManagers.get(name);
		if(m==null) {
		  System.err.println("warning: enumtype '" +name +"' undefined");
		  return null;
		}
		String unqName = ReflectHelper.unqualifiedName(name);              
		if(m instanceof RefPackage) {
		  return ((RefPackage)m).refGetEnum(unqName, value);
		}        
		try {
			return ((RefClass)m).refGetEnum(unqName, value);		
		} catch(Exception e) {
			System.err.println("warning: enum " +name +"." +value  +" undefined");			
		}
		return null;	
	}
	
	

	

		
	public RefStruct createStruct(String name, List args) {
		name = 	name.substring(name.indexOf(":")+1);
		Object f = structFactories.get(name);
		if(f==null) { 		  
			System.err.println("warning: struct '" +name +"' undefined");
			return null;
		}
		String unqName = ReflectHelper.unqualifiedName(name);
		if(f instanceof RefPackage) {
		  return ((RefPackage) f).refCreateStruct(unqName, args);
		}
		return ((RefClass) f).refCreateStruct(unqName, args);  		
	}
	
	
	

}



