package org.objectweb.modfact.jmi.xmi;

import java.util.*;

import javax.jmi.reflect.*;

import org.objectweb.modfact.jmi.reflect.ReflectHelper;


public class PropertyTypeManager {

	Hashtable typeCach = new Hashtable();
	
	public PropertyTypeManager() {
	}
	
	

	public TypeInfo findTypeFor(RefObject metaObjectOwner, String unqPropName )  {
		RefObject propMeta = (RefObject) ReflectHelper.lookupElementExtended(metaObjectOwner, unqPropName);            
		return findTypeFor(propMeta);
	}
    

	// propMetaObject describes the property
	// propMetaObject.name = property name
	// propMetaObject.type = property type meta object
	// propMetaObject.type.name = type name
	private TypeInfo findTypeFor(RefObject propMetaObject) {
	   return findTypeFor(propMetaObject, false); 
	}
    

	private TypeInfo findTypeFor(RefObject propMetaObject, boolean isStructField) {
		// find in Cache
		TypeInfo typeInfo = (TypeInfo)typeCach.get(propMetaObject);
		if(typeInfo!=null)  return typeInfo;
		// find type info in M2 (to load M1 XMI data) 
		//             or in M3 (to load M2 XMI data) 
		typeInfo = new TypeInfo();
		if(!isStructField) {
		  typeInfo.isMultiValued = (
			((Integer)((RefStruct)propMetaObject.refGetValue("multiplicity")).refGetValue("upper")).intValue()!=1
		  );
		}
		RefObject type= (RefObject)propMetaObject.refGetValue("type");
		typeInfo.name = (String) type.refGetValue("name");
		typeInfo.qname = (String) ReflectHelper.dotQName(ReflectHelper.getQualifiedName(type));
		String typeKind = (String) type.refMetaObject().refGetValue("name");
		if(typeKind.equals("PrimitiveType")) {
		  typeInfo.isPrimitive = true;
		} else if(typeKind.equals("EnumerationType")) {
		  typeInfo.isEnum = true;
		  typeInfo.enumUnprefix = ReflectHelper.findTagValue(type,"org.omg.xmi.enumerationUnprefix");
          
		} else if(typeKind.equals("StructureType")) {
		  typeInfo.isStruct = true;
		  List fieldTypes = new Vector();
		  List fieldNames = new Vector();
		  Iterator it = ((List)type.refGetValue("contents")).iterator();
		  while(it.hasNext()) {
			RefObject content = (RefObject)it.next();
			if(content.refMetaObject().refGetValue("name").equals("StructureField")) {
			  fieldTypes.add(findTypeFor(content, true));
			  fieldNames.add(content.refGetValue("name"));
			}
		  }
		  typeInfo.structFieldTypes = (TypeInfo[]) fieldTypes.toArray(new TypeInfo[0]);
		  typeInfo.structFieldNames = (String[])   fieldNames.toArray(new String[0]);
		} 
		// put in cache
		typeCach.put(propMetaObject, typeInfo);
		return typeInfo;
	}
       


   

	public static class TypeInfo {
		public String name;
		public String qname;
		public boolean isMultiValued = false;
		public boolean isEnum = false;
		public String enumUnprefix = null; //correspond to tag "xmi.enumerationUnprefix"
		public boolean isStruct = false;
		public boolean isPrimitive = false;
		public TypeInfo[] structFieldTypes = null;
		public String[] structFieldNames = null;
	}
}
