package org.objectweb.modfact.jmi.xmi;

import javax.xml.parsers.*;

import org.objectweb.modfact.jmi.reflect.ReflectHelper;
import org.w3c.dom.*;

import org.objectweb.modfact.jmi.xmi.PropertyTypeManager.TypeInfo;
import javax.jmi.reflect.*;
import java.util.*;
import java.io.*;



public class XmiParser {

    static DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
    static DocumentBuilder builder;
    static {
       try {           
         builder = documentBuilderFactory.newDocumentBuilder();
       } catch(Exception e) {
         System.err.println("Couldnot initialize DOM Parser");
         throw new RuntimeException(e);
       }
    }
    
 
    Document document;
    Node rootNode;
               
    // xmi.id to RefObject
    Hashtable id_object = new Hashtable();
    List unresolvedProps = new Vector();
    
    RefPackage extent;
    ObjectFactoryManager factManager;
        
    XmiReaderImpl caller;
    
	static Object lock = "";    
    XmiParser(XmiReaderImpl owner, RefPackage p, InputStream in) {
      this.caller = owner;	
      this.extent = p;        
      try {  
      	synchronized(lock) {                    
          document = builder.parse(in);
          rootNode = document.getDocumentElement().getElementsByTagName("XMI.content").item(0);
        }
      }   catch(Exception e) {
          throw new RuntimeException(e);
      }
    }
    
    private Set createdObjects = new HashSet() {
      public boolean add(Object o) {
      	if(o==null) {
      		throw new RuntimeException("cannot add null");
      	}
      	return super.add(o);      	
      }
    };
    
    void run() {
    	extent = extent.refOutermostPackage();
		factManager = new ObjectFactoryManager(extent);
    	
        Node xmi_content = document.getElementsByTagName("XMI.content").item(0);
        NodeList children = xmi_content.getChildNodes();
        for (int i=0; i < children.getLength(); i++) {
            Node node = children.item(i);
            if(node instanceof Element) {
              process1((Element) node);
            }
        } 
                       
        Iterator it = elements.iterator();
        while(it.hasNext()) {
          process2((Element)it.next());
        }
    }
    
    Collection getOuterMostObjects() {
      Collection r = new HashSet(); 
      Iterator it = createdObjects.iterator();
      while(it.hasNext()) {	
    	RefObject o = (RefObject) it.next();
    	RefBaseObject outer = o.refOutermostComposite();

    	if(outer instanceof RefObject) r.add(outer);	
      }
      return r;    	
    }
    
    
    

     
   
    // list of empty element to assign properties' values to
    List elements = new Vector();    
    
    // create corresponding object
    // and map xmi.id to it
    void process1(Element elem) {

		String xmi_id = elem.getAttribute("xmi.id");		
		String elemName = ReflectHelper.unqualifiedName(elem.getTagName());
		RefObject o;
		        
        if(!"".equals(xmi_id)) {
          // support for XMI 1.0, 1.1
		  o = doCorbaType(xmi_id, elemName, elem);
          if(o==null) {     
		    if(caller.verbose) System.err.println("creating object '" +elem.getTagName() +"' xmi.id=" +xmi_id );        		  
		    o = createObject(elem);
            if(o!=null) {
			  id_object.put(xmi_id, o);
			  elements.add(elem); // to be processed later;             	
            }
          }         
        }

        NodeList children = elem.getChildNodes();
        for (int i=0; i < children.getLength(); i++) {
            Node child = children.item(i);
            if(child instanceof Element) {
              try {
                  process1((Element) child);
              } catch(RuntimeException e) {
                 if(!caller.ignoreError) throw e;        
              }
            }
        } 
    }
    
    
    RefObject createObject(Element elem) {
		String xmi_id = elem.getAttribute("xmi.id");		
		
		RefObject o = factManager.createObject(elem.getTagName());
		if(o == null) {
			System.out.println("warning: cannot create " +elem.getTagName() );
		} else {
			createdObjects.add(o);
		}
			
		return o;    
    }
    
    // Set values of all properties
    void process2(Element elem) {

		String xmi_id = elem.getAttribute("xmi.id");
		RefObject o = (RefObject) id_object.get(xmi_id);     
		
		if(caller.verbose) 
			System.err.println("setting properties for object '" 
				+elem.getNodeName() +"' xmi.id=" +xmi_id 
			);                
		try {
		  setProperties(elem ,o);
		} catch(RuntimeException e) {          
		  RuntimeException e2 = new RuntimeException("at xmi.id="+xmi_id, e);
		  throw e2;
		}    		  
		
    }
    
    void setProperties(Element elem, RefObject o) {

		  NamedNodeMap attrs = elem.getAttributes();
           
          NodeList children = elem.getChildNodes();
          for (int i=0; i < children.getLength(); i++) {
            Node child = children.item(i);
            if(child instanceof Element)
              setValue(o, child );
          }
        
          for (int i=0; i < attrs.getLength(); i++) {
            Attr attr = (Attr) attrs.item(i);
            if(!attr.getName().equals("xmi.id"))
              setValue(o, attr );
          }
            
    }
    
    
    PropertyTypeManager typeManager = new PropertyTypeManager(); 

    
    void setValue(RefObject owner, Node propNode) {
        RefObject metaObject = owner.refMetaObject();
        String propName = propNode.getNodeName();
        
        // ignore xmi extention tag
        if( propName.toLowerCase().indexOf("xmi.")>=0) {
          if(caller.verbose)
              System.err.println("\t\t\t*** ignore node '" +propName +"'");
          return;
        }
        
        String unqPropName = ReflectHelper.unqualifiedName(propNode.getNodeName());
        TypeInfo type;
        try {
          type = typeManager.findTypeFor(metaObject, unqPropName);
        } catch(Exception e) {

			String err = "property '" +propName +"' undefined in '" +metaObject.refGetValue("qualifiedName") +"'";
			if(caller.ignoreError) {
				if(caller.verbose)  System.err.println("***" +err);
				return;
			} else  { 
                throw new RuntimeException( err ,e );
			}
        }
        try {
          if(type.isMultiValued) {
            Collection values = (propNode instanceof Attr)?
                  createValuesFrmAttr(type, (Attr) propNode)
                : createValuesFrmElement(type, (Element)propNode);  
            if(caller.verbose) System.err.println("    set property '" 
                +propName +"' =" +values
            );    
            ((Collection)owner.refGetValue(unqPropName)).addAll(values);        
          } else {
            Object value = (propNode instanceof Attr)?
                  createValueFrmAttr(type, (Attr)propNode)
                : createValueFrmElement(type, (Element)propNode); 
            if(caller.verbose) 
            	System.err.println("    set property '" 
                  +propName +"' =" +value
            	);
            if(value!=null) {            
              owner.refSetValue(unqPropName, value );
            }
          } 
        } catch(RuntimeException e) {
            String err = "Could not set property '" +propName +"' from Node "+ propNode;
            if(caller.ignoreError) {
              if(caller.verbose)  System.err.println("***" +err);
            } else  { 
              throw new RuntimeException(err, e);
            }			  
        }
    }
    

    
   Object createValueFrmAttr(TypeInfo type, Attr propNode) {
       return createValue(type, propNode.getNodeValue());
   }
   
   Object createValueFrmElement(TypeInfo type, Element propNode) {
        if(type.isStruct) {            
          return createStruct(type, propNode);
        }
        
        if(propNode.hasAttribute("xmi.value")) {
          return createValue(type, propNode.getAttribute("xmi.value"));        
        }
        
        String xmi_idref = null;
        NodeList children = ((Element)propNode).getChildNodes();
        for(int i=0; i<children.getLength(); i++) {
           Node child = children.item(i);
           if(child instanceof Element) {
              Element childElem = (Element)child;
              if(childElem.hasAttribute("xmi.id")) {
                return createValue(type, childElem.getAttribute("xmi.id"));

              } else if(childElem.hasAttribute("xmi.idref")) {
                return createValue(type, childElem.getAttribute("xmi.idref"));

			  } else if(childElem.getTagName().equalsIgnoreCase("XMI.any")) {
				return createValue(type, childElem.getFirstChild().getNodeValue());

              } else if( !type.isEnum && !type.isPrimitive 
                         && factManager.canCreateObject( childElem.getTagName() )
                     ) {  // if type is object but the XML element has no xmi.id 
                     	  // then create an object now
              	RefObject o = factManager.createObject(childElem.getTagName());
              	setProperties(childElem, o);
                return o;
              } 
              // otherwise ignore this element
              System.err.println("\t\t\t*** ignore element '" +childElem +"'");
           } else if(child instanceof Text) {
              // avoid empty text nodes mixed with child elements 
              if(child.getNodeValue().trim().length() > 0)                
                return createValue( type, child.getNodeValue() );
           
           } else if(child instanceof Comment) {
           
           } else {
             // otherwise ignore 
             System.err.println("\t\t\t*** ignore node '" +child +"'");
           }
        }
        return createValue(type ,"");
    }
    
    Collection createValuesFrmAttr(TypeInfo type, Attr propNode) {
          List r = new Vector();
          // parse space-seperated string
          StringTokenizer t = new StringTokenizer(propNode.getNodeValue());
          while(t.hasMoreTokens()) {
            String vs = t.nextToken();
            Object vo = createValue(type,vs);
            if(vo!=null) r.add(vo);
          }
          return r;
    }
    
    Collection createValuesFrmElement(TypeInfo type, Element propNode) {    
        List r = new Vector();
        if(type.isStruct) {
          Object v = createStruct(type, propNode);
          if(v!=null) r.add(v);
          return r;
        }
        if(propNode.hasAttribute("xmi.value")) {
          Object v = createValue(type, propNode.getAttribute("xmi.value"));	
		  if(v!=null) r.add(v);
          return r;
        }        
        NodeList list = ((Element)propNode).getChildNodes();
        for(int i=0; i<list.getLength(); i++) {
          Node child = list.item(i);
          if(child instanceof Element) {
             Element childElem = (Element) child;
             if(childElem.hasAttribute("xmi.id")) {
               Object v = createValue(type, childElem.getAttribute("xmi.id"));
               if(v!=null) r.add(v);
               
             } else if(childElem.hasAttribute("xmi.idref")) {
			   Object v = createValue(type, childElem.getAttribute("xmi.idref"));
			   if(v!=null) r.add(v);
               
             } else if(childElem.getTagName().equalsIgnoreCase("XMI.any")) {
			   Object v = createValue(type, childElem.getFirstChild().getNodeValue());
			   if(v!=null) r.add(v);
            

			 } else if( !type.isEnum && !type.isPrimitive 
						&& factManager.canCreateObject( childElem.getTagName() )
					) {  // if type is object but the XML element has no xmi.id 
						 // then create an object now             
			   RefObject v = factManager.createObject(childElem.getTagName());
			   setProperties(childElem, v);
			   if(v!=null) r.add(v);
             
             } else {
               // otherwise ignore this element
               System.err.println("\t\t\t*** ignore element '" +childElem +"'");
             }
          
          } else if(child instanceof Text) {
             // avoid empty text nodes mixed with child elements 
             if(child.getNodeValue().trim().length() > 0) {            
                 Object v = createValue(type, child.getNodeValue());
				 if(v!=null) r.add(v);
             }
          } else if(child instanceof Comment) {
          
          } else {
            // otherwise ignore 
            System.err.println("\t\t\t*** ignore node '" +child +"'");
          }
        }
        return r;
    }    

    Object createValue(TypeInfo type, String value) {
        if(type.isPrimitive) {
          return parsePrimitiveValue(type.name, value);
        }
        if(type.isEnum) {           
          return getEnum(type, value);
        }
        if(type.isStruct) {            
          throw new RuntimeException("Struct value should not be encoded as attribute");
        }
        // type is RefObject
        String xmi_idref = value; //the same action even if value comes from xmi.id
        Object r = id_object.get(xmi_idref);
        if(r==null) {
           throw new RuntimeException("unresolved xmi.idref '" +xmi_idref +"'");
        }
        return r;
    }    
    
    Object parsePrimitiveValue(String type, String value) {
      if(type.equalsIgnoreCase("string")) {
        return value;
      } else if(type.equalsIgnoreCase("long")
        || type.equalsIgnoreCase("integer") ) {
        return new Integer(Integer.parseInt(value));
      } else if(type.equalsIgnoreCase("double")
        || type.equalsIgnoreCase("float")  ) {
        return new Double(Double.parseDouble(value));  
      }  else if(type.equalsIgnoreCase("boolean")) {
        return Boolean.valueOf(value);
      }
      throw new RuntimeException("unknown primitive type '" + type +"'"); 
    }
    
    
    RefEnum getEnum(TypeInfo type, String label) {       
       String prefixLabel = label; 
       if(type.enumUnprefix!=null) { 
       	 if(!label.startsWith(type.enumUnprefix)) {
		    prefixLabel = type.enumUnprefix + label;
       	 }
       } 
	   return factManager.getEnum(type.qname, prefixLabel);           
    }

   /*
    format 1:
                 <Model.StructuralFeature.multiplicity>
                    <XMI.field>1</XMI.field>
                    <XMI.field>1</XMI.field>
                    <XMI.field>false</XMI.field>
                    <XMI.field>false</XMI.field>
                 </Model.StructuralFeature.multiplicity>
    format 2:
       <Model:StructuralFeature.multiplicity>
            <Model:MultiplicityType lower="1" upper="1" is_ordered="false" is_unique="false" /> 
       </Model:StructuralFeature.multiplicity>  
      
    todo: Test with nested Structures (Structure of Structures)
    */
    RefStruct createStruct(TypeInfo type, Element elem) {
      if(isStructFormat1(elem)) {
        return createStruct_Format1(type, elem);
      } 
      return createStruct_Format2(type, elem);
    }
    
    boolean isStructFormat1(Element elem) {
      return getChildFromLocalName(elem, "XMI.field")!=null;
    }
    
    Element getChildFromLocalName(Element elem, String name) {
      NodeList list = elem.getChildNodes();
      for(int i=0; i<list.getLength() ; i++) {
        Node child = list.item(i);
        if((child instanceof Element) && child.getNodeName().endsWith(name) )   
            return (Element)child;
      }
      return null;
    }
        
    RefStruct createStruct_Format1(TypeInfo type, Element elem) {
        List args = new Vector();
        NodeList list = elem.getChildNodes();
        int j = 0;
        for(int i=0; i<list.getLength(); i++) {
          Node child = list.item(i);
          String nodeName = child.getNodeName();
          if((child instanceof Element) && nodeName.endsWith("XMI.field")) {
            args.add( createValueFrmElement(type.structFieldTypes[j], (Element)child) );
            j++;
          }
        }
        return factManager.createStruct(type.qname, args);
    }
    
    RefStruct createStruct_Format2(TypeInfo type, Element elem) {
        Element inner = getChildFromLocalName(elem, type.name);
        List args = new Vector();
        for(int i=0; i<type.structFieldTypes.length; i++) {
          try {  
            String value = inner.getAttribute(type.structFieldNames[i]);
            args.add(
                createValue(type.structFieldTypes[i], value )
            );
          } catch(RuntimeException e) {
            System.err.println("CreateStruct: Error at att[" +i +"] "
            	+type.structFieldNames[i]            	
            );
            throw e;
          }
    	}
        return factManager.createStruct(type.qname, args);
    }    
    
    
    
    Map sharedPrimitiveTypes = new Hashtable();
    
    // support for XMI 1.0 / 1.1
    // translate corba typecode to PrimitiveType
    // return false if this element does not have corba typecode
    RefObject doCorbaType(String xmi_id, String elemName, Element elem) {
        
        if(!elemName.equals("DataType")) { 
            return null;
        }        
		Element tc = findTypeCodeInDataType(elem); 
		if(tc==null) return null;
		       

		//resolve alias type
		while(tc.getNodeName().equalsIgnoreCase("XMI.CorbaTcAlias")) {
		  tc = findTypeCodeInDataType(tc);
		}
		
		String name = null;
		RefObject type; // object to be create		
             
        if(tc.getNodeName().equalsIgnoreCase("XMI.CorbaTcEnum")) {
          if(caller.verbose) System.err.println(
            "creating EnumerationType '" +name +"' xmi.id=" +xmi_id
          );                  
          type = extent.refClass("EnumerationType").refCreateInstance(null);
		  name = tc.getAttribute("xmi.tcName");
		  if(name.length()==0) {
			throw new RuntimeException("Expecting xmi.tcName in " +tc.getNodeName());
		  }
		  type.refSetValue("name", name);		  
		  NodeList labels = tc.getElementsByTagName("XMI.CorbaTcEnumLabel");
          List type_labels = (List)type.refGetValue("labels");
          for(int i=0; i<labels.getLength() ; i++) {
            Element label = (Element) labels.item(i);
            type_labels.add(label.getAttribute("xmi.tcName"));
          }
		  type.refSetValue("name", name);

        } else {          
		  //handle primitive type
		  // normalize the first letter of type name to upper case
		  // ex, 'integer' to 'Integer'
		  name = getTcName(tc);		  
		  char firstChar = name.charAt(0);
		  name = Character.toUpperCase(firstChar) + name.substring(1);		  
		  type = (RefObject) sharedPrimitiveTypes.get(name);
		  if(type==null) {
		    if(caller.verbose) System.err.println(
			  "creating PrimitiveType '" +name +"' xmi.id=" +xmi_id
		    );         
            type = extent.refClass("PrimitiveType").refCreateInstance(null);
			type.refSetValue("name", name); 
		  }   
        }
        id_object.put(xmi_id, type);
		createdObjects.add(type);        
        return type;
    }
    
    
    // find child element with tag XMI.CorbaTc...
    // example : XMI.CorbaTcString XMI.CorbaTcBoolean
    Element findTypeCodeInDataType(Element dataType) {
		NodeList typeCode = dataType.getElementsByTagName("XMI.CorbaTypeCode");		  
		if(typeCode.getLength()==0) {
		  return null;
		}          
		Element tcElem = (Element) typeCode.item(0);     
		NodeList tcContentList = tcElem.getChildNodes();
		for(int i = 0; i<tcContentList.getLength(); i++) {
		  Node n = tcContentList.item(i); 
		  if(n instanceof Element) {
			Element tcContent = (Element)n;
			String s = tcContent.getNodeName().toLowerCase();
			if(s.indexOf("xmi.corbatc")>=0) {
			  return tcContent;
			}
		  }
		}		          
		throw new RuntimeException("Cannot handle CorbaTypeCode " + tcElem);
    }
    
    String getTcName(Element tc) {
      String n = tc.getNodeName().toLowerCase();
      return n.replaceAll("xmi.corbatc","");      
    }
    
        





    
}
