/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.exporter;

import java.util.List;
import java.util.Vector;

import javax.jmi.model.Attribute;
import javax.jmi.model.Classifier;
import javax.jmi.model.DataType;
import javax.jmi.model.EnumerationType;
import javax.jmi.model.Import;
import javax.jmi.model.ModelElement;
import javax.jmi.model.MofClass;
import javax.jmi.model.MofPackage;
import javax.jmi.model.Namespace;
import javax.jmi.model.PrimitiveType;
import javax.jmi.model.Reference;
import javax.jmi.model.ScopeKindEnum;
import javax.jmi.model.VisibilityKindEnum;

import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.helper.XMIIOHelper;
import org.objectweb.modfact.jmi.logging.Level;
import org.objectweb.modfact.jmi.logging.ModFactLogger;
import org.objectweb.modfact.jmi.xmiio.common.AbstractXMIIOGenerator;

/**
 * The XMI Export generator.
 * This class allows the generation of the XMI exporter (into XMI).
 */
public class GenerationXMIExport extends AbstractXMIIOGenerator {

	/** The logger. */
	protected ModFactLogger _logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiio.exporter.GenerationXMIExport");

	/**
	 * Default Constructor.
	 */
	public GenerationXMIExport() {
		super();
		ModFactLogger logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiio.exporter.GenerationXMIExportApplication");
	}

	/**
	 * XMIExport generation.
	 * @param package_ The root package.
	 */
	public void generateXMIExport(MofPackage package_, String className) throws java.io.IOException {
		_logger.log(Level.FINE, "XMIExport beginning of the generation");
		_logger.log(Level.FINE, className + " generation ...");
		packageModuleTemplate(package_, className);
		flushFile();
	}

	java.util.Vector classes = new java.util.Vector();
	java.util.Vector dataTypes = new java.util.Vector();

	private void processContents(MofPackage package_) {
		java.util.List contentsList = package_.getContents();
		java.util.Iterator it = contentsList.iterator();
		while (it.hasNext()) {
			javax.jmi.model.ModelElement contents = (javax.jmi.model.ModelElement) it.next();
			if (contents instanceof javax.jmi.model.MofPackage)
				processContents((javax.jmi.model.MofPackage) contents);
			else if (contents instanceof javax.jmi.model.Import)
				processContents((javax.jmi.model.MofPackage) ((javax.jmi.model.Import) contents).getImportedNamespace());
			else if (contents instanceof javax.jmi.model.MofClass)
				classes.addElement((javax.jmi.model.MofClass) contents);
			else if (contents instanceof javax.jmi.model.DataType)
				dataTypes.addElement((javax.jmi.model.DataType) contents);
		}
	}

	/**
	 * Find all the Class Attributes of the package.
	 * @param package_ The package to parse.
	 * @return The found Class Attributes.
	 */
	protected java.util.Vector findAllClassAttributes(MofPackage package_) {
		MofClass[] contents = XMIIOHelper.allClassesOfPackageOrderedByInheritance(package_);
		java.util.Vector attributes = new java.util.Vector();
		for (int i = 0; i < contents.length; i++) {
			if (contents[i] instanceof MofClass) {
				MofClass class_ = (MofClass) contents[i];
				Reference references[] = MofHelper.referencesOfClass(class_, true);
				for (int l = 0; l < references.length; l++) {
					Classifier classifier = references[l].getType();
					if (!attributes.contains(classifier))
						attributes.add(classifier);
				}
				Attribute attr[] = MofHelper.attributesOfClass(class_, ScopeKindEnum.forName("instance_level"), true);
				for (int l = 0; l < attr.length; l++) {
					Classifier classifier = attr[l].getType();
					if (classifier instanceof MofClass && !attributes.contains(classifier))
						attributes.add(classifier);
				}
			} else if (contents[i] instanceof Import) {
				Import imp = (Import) contents[i];
				java.util.Vector call = findAllClassAttributes((MofPackage) imp.getImportedNamespace());
				java.util.Iterator iterator = call.iterator();
				while (iterator.hasNext()) {
					boolean found = false;
					Object next = iterator.next();
					for (int k = 0; k < attributes.size(); k++) {
						if (attributes.elementAt(k).equals(next))
							found = true;
					}
					if (!found)
						attributes.add(next);
				}
			}
		}
		return attributes;
	}

	/**
	 * Package processing.
	 * @param package_ The package to process.
	 */
	public void packageModuleTemplate(MofPackage package_, String className) {
		_logger.log(Level.FINE, "XMIExport packageModuleTemplate");
		if (package_.getVisibility() == VisibilityKindEnum.forName("public_vis")) {
			MofClass classes_ordered[] = XMIIOHelper.allClassesOfPackageOrderedByInheritance(package_);

			outputln("package " + JMIProvider.qualifierOf(package_) + ".xmi;");
			outputln();
			annotationTemplate(package_.getAnnotation());
			outputln("public class " + className + " extends org.objectweb.modfact.jmi.xmiio.exporter.XMIExport {");

			processContents(package_);
			constructor(package_.getName(), className);
			classAttributes(findAllClassAttributes(package_));
			classDataTypeAttributes(dataTypes);
			classTemplate(package_, classes_ordered);
			rootPackage(package_, classes_ordered);
			outputln("} // end of class " + className);
			outputln();
		}
	}

	/**
	 * XMIExport generation for constructor.
	 * @param package_name The package name.
	 * @param class_name The class name.
	 */
	public void constructor(String package_name, String class_name) {
		_logger.log(Level.FINE, "XMIExport constructor(" + package_name + ", " + class_name + ")");
		outputln("/**");
		outputln(" * Default " + class_name + " Constructor");
		outputln(" * @param dtd The DTD File.");
		outputln(" */");
		outputln("public " + class_name + "(java.lang.String dtd) {");
		outputln("this(dtd, org.objectweb.modfact.jmi.logging.ModFactLogger.getLogger(\"" + package_name + "XMIExport\"));");
		outputln("}");
		outputln();
		outputln("/**");
		outputln(" * " + class_name + " Constructor");
		outputln(" * @param dtd The DTD File.");
		outputln(" * @param logger The logger.");
		outputln(" */");
		outputln("public " + class_name + "(java.lang.String dtd, org.objectweb.modfact.jmi.logging.ModFactLogger logger) {");
		outputln("super(dtd, \"" + JMIProvider.jmiFormat1(package_name) + "\", \"Model\");");
		outputln("_logger = logger;");
		outputln("}");
		outputln();
	}

	protected String refMethod(MofClass cls, String package_name) {
		String method = "get" + JMIProvider.jmiFormat1(JMIProvider.jmiClassName(cls)) + "()";
		ModelElement clsTmp = cls;
		while (!clsTmp.getContainer().getName().equals(package_name)) {
			Namespace ns = clsTmp.getContainer();
			if (ns instanceof Classifier) {
				Classifier container = (Classifier) ns;
				method = "get" + JMIProvider.jmiPackageExtentName((MofPackage) container) + "()." + method;
			} else {
				method = "get" + JMIProvider.jmiPackageExtentName((MofPackage) clsTmp.getContainer()) + "()." + method;
			}
			clsTmp = clsTmp.getContainer();
		}
		return method;
	}

	/**
	 * Generate the methods for the writing of complex type.
	 * @param attributes List of DataType attributes.
	 */
	public void classAttributes(Vector attributes) {
		_logger.log(Level.FINE, "XMIExport classAttributeTemplate()");
		java.util.Iterator iterator = attributes.iterator();
		outputln("// ==================================================================");
		outputln("//  Method to write some Classes ");
		outputln("// ==================================================================");
		while (iterator.hasNext()) {
			Classifier classifier2 = (Classifier) iterator.next();
			MofClass classifier = (MofClass) classifier2;
			annotationTemplate(classifier.getAnnotation());
			String classifierQualifiedName;
			classifierQualifiedName = JMIProvider.jmiClassifierQualifiedName(classifier);
			outputln("public void write" + JMIProvider.jmiFormat1(classifier.getName()) + "(" + classifierQualifiedName + " _toWrite) {");
			outputln("if (_toWrite == null)");
			outputln("    return;");
			outputln("outputln(\"<" + JMIProvider.jmiFormat1(classifier.getName()) + " xmi.id=\\\"\" + getXMIId(_toWrite) + \"\\\">\");");
			Attribute attr[] = MofHelper.attributesOfClass(classifier, ScopeKindEnum.forName("instance_level"), true);
			for (int j = 0; j < attr.length; j++) {
                if (!attr[j].isDerived())
				    classAttributeTemplate(attr[j], "_toWrite");
			}
			Reference ref[] = MofHelper.referencesOfClass(classifier, true);
			for (int j = 0; j < ref.length; j++) {
				classReferenceTemplate(ref[j], "_toWrite");
			}
			outputln("outputln(\"</" + JMIProvider.jmiFormat1(classifier.getName()) + ">\");");
			outputln("}");
			outputln();
		}
	}

	/**
	 * Generate the methods for the writing of complex type.
	 * @param attributes List of DataType attributes.
	 */
	public void classDataTypeAttributes(Vector attributes) {
		_logger.log(Level.FINE, "XMIExport classAttributeTemplate()");
		java.util.Iterator iterator = attributes.iterator();
		outputln("// ==================================================================");
		outputln("//  Method to write the DataTypes ");
		outputln("// ==================================================================");
		while (iterator.hasNext()) {
			DataType dataType = (DataType) iterator.next();
			if (dataType instanceof EnumerationType) {
				EnumerationType enumType = (EnumerationType) dataType;
				outputln(
					"public java.lang.String enumeration"
						+ JMIProvider.jmiFormat1(enumType.getName())
						+ "ToString("
						+ JMIProvider.jmiClassifierQualifiedName(enumType)
						+ " value) {");

				List labels = enumType.getLabels();
				for (int i = 0; i < labels.size(); i++) {
					outputln(
						"if (value.equals(" + JMIProvider.jmiClassifierQualifiedName(enumType) + "Enum." + labels.get(i).toString().toUpperCase() + "))");
					outputln("    return \"" + JMIProvider.jmiFormat2(labels.get(i).toString()) + "\";");
				}
				outputln("return new java.lang.String();");
				outputln("}");
				outputln();
			}
		}
	}

	/**
	 * XMIExport generation for the class template.
	 * @param package_ The package.
	 * @param classes The classes of the package.
	 */
	public void classTemplate(MofPackage package_, MofClass classes[]) {
		_logger.log(Level.FINE, "XMIExport classTemplate()");
		String package_name = package_.getName();
		outputln("// ==================================================================");
		outputln("//  XXXTemplate with XXX an entity ");
		outputln("// ==================================================================");
		// Classes
		for (int i = 0; i < classes.length; i++) {
			MofClass cls = classes[i];
			annotationTemplate(cls.getAnnotation());
			String model_package_type;
			model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
			outputln("public void " + XMIIOHelper.format1FirstMin(cls.getName()) + "Template(" + model_package_type + " _model_package) {");
			String clsClass = XMIIOHelper.format1FirstMin(cls.getName()) + "Class";
			String clsTab = XMIIOHelper.testJavaConflict(XMIIOHelper.format1FirstMin(cls.getName()) + "s");
			String clsName;
			String xmiQualifiedName = JMIProvider.jmiFormat1(cls.getName());
			try {
				clsName = JMIProvider.jmiClassName(cls);
			} catch (NullPointerException notSet) {
				clsName = cls.getName();
			}
			String javaQualifiedName = JMIProvider.jmiClassQualifiedName(cls);
			outputln(
				"_logger.log(org.objectweb.modfact.jmi.logging.Level.FINE, \""
					+ XMIIOHelper.format1FirstMin(cls.getName())
					+ "Template(\" + _model_package + \")\");");
			outputln(javaQualifiedName + "Class " + clsClass + " = _model_package." + refMethod(cls, package_name) + ";");
			try {
				outputln("java.util.Collection " + clsTab + " = " + clsClass + ".refAllOfType" + "();");
			} catch (NullPointerException notSet) {
				outputln("java.util.Collection " + clsTab + " = " + clsClass + ".refAllOfType" + "();");
			}
			//outputln("for (int i=0 ; i<" + clsTab + ".length ; i++) {");
            outputln("java.util.Iterator iterator = " + clsTab + ".iterator();");
            outputln("while (iterator.hasNext()) {");
            outputln(javaQualifiedName + " current = (" + javaQualifiedName + ")iterator.next();");
			outputln("outputln(\"<" + xmiQualifiedName + " xmi.id=\\\"\" + getXMIId(current) + \"\\\">\");");
			Attribute attributes[] = MofHelper.attributesOfClass(cls, ScopeKindEnum.forName("instance_level"), true);
			for (int j = 0; j < attributes.length; j++) {
                if (!attributes[j].isDerived())
				    classAttributeTemplate(attributes[j], "current");
			}
			Reference references[] = MofHelper.referencesOfClass(cls, true);
			for (int j = 0; j < references.length; j++) {
				Reference ref = references[j];
				String variable = JMIProvider.jmiAccessorName(ref);

				if (ref.getMultiplicity().getUpper() == -1) {
					outputln("for (java.util.Iterator it = current." + variable + "().iterator() ; it.hasNext() ;) {");
					outputln(
						"write"
							+ JMIProvider.jmiFormat1(ref.getType().getName())
							+ "(("
							+ JMIProvider.jmiClassifierQualifiedName(ref.getType())
							+ ")it.next());");
					outputln("}");
				} else {
					String content = "write" + JMIProvider.jmiFormat1(ref.getType().getName()) + "(current." + variable + "());";
					if (ref.getMultiplicity().getLower() == 0) {
						outputln("try {");
						outputln(content);
						outputln("} catch (java.lang.NullPointerException notSet) { /* Nothing to do */ }");
					} else {
						outputln(content);
					}
				}

			}
			outputln("outputln(\"</" + xmiQualifiedName + ">\");");
			outputln("}");
			outputln("}");
			outputln();
		}
	}

	/**
	 * XMIExport generation for the class attribute template.
	 * @param attribute The class attribute.
	 * @param variable_name The variable name.
	 */
	public void classAttributeTemplate(Attribute attribute, String variable_name) {
		_logger.log(Level.FINE, "XMIExport classAttributeTemplate(" + attribute.getName() + ")");
		try {
			String attrName = JMIProvider.jmiFormat1(attribute.getContainer().getName()) + "." + XMIIOHelper.format1FirstMin(attribute.getName());
			String tag1 = "<" + attrName + ">";
			String tag2 = "</" + attrName + ">";
			String readAttribute = variable_name + "." + JMIProvider.jmiAccessorName(attribute) + "()";
            boolean voidType = false;
			String content = "outputln(\"" + tag1 + "\" + " + readAttribute + " + \"" + tag2 + "\");";
			Classifier type = attribute.getType();
			if (type instanceof EnumerationType) {
				EnumerationType enumType = (EnumerationType) type;
                String condition = "if (" + readAttribute + "!= null)";
				readAttribute = "enumeration" + JMIProvider.jmiFormat1(enumType.getName()) + "ToString(" + readAttribute + ")";
                voidType = true;
				content = condition + "\n" + TABULATION + "outputln(\"" + tag1 + "\" + " + readAttribute + " + \"" + tag2 + "\");";
			} else if (type instanceof MofClass) {
					MofClass class_ = (MofClass) type;
					content = "if (" + readAttribute + " != null) {\n";
					readAttribute = "write" + JMIProvider.jmiFormat1(class_.getName()) + "(" + readAttribute + ")";
                    voidType = true;
					content = content + "outputln(\"" + tag1 + "\");\n" + readAttribute + ";\n" + "outputln(\"" + tag2 + "\");" + "\n}";
			}

			if (attribute.getMultiplicity().getUpper() == -1) {
                outputln("for (java.util.Iterator it = " + readAttribute + ".iterator() ; it.hasNext() ;) {");
				outputln("output(\"" + tag1 + "\" + it.next());");
                outputln("if (it.hasNext()) output(\" \");");
                outputln("outputln(\"" + tag2 + "\");");
				outputln("}");
			} else {
                if (!voidType && !(attribute.getType() instanceof PrimitiveType))
                    outputln("if (" + readAttribute + " != null)");
                else if (attribute.getType() instanceof PrimitiveType)
                    outputln("if (" + variable_name + ".refGetValue(\"" + XMIIOHelper.format1FirstMin(attribute.getName()) + "\") != null)");
				outputln(content);
			}
		} catch (NullPointerException notSet) {
			// Nothing to do
		}
	}

	/**
	 * XMIExport generation for the class reference template.
	 * @param reference The class reference.
	 * @param variable_name The variable name.
	 */
	public void classReferenceTemplate(Reference reference, String variable_name) {
		_logger.log(Level.FINE, "XMIExport classReferenceTemplate(" + reference.getName() + ")");
		try {
			String refName = JMIProvider.jmiFormat1(reference.getContainer().getName()) + "." + JMIProvider.jmiFormat1(reference.getName());
			String refList = XMIIOHelper.format2Idl2JavaConflict(reference.getName());
			String refType = JMIProvider.jmiClassifierQualifiedName(reference.getType());
            String refTypeXmi = JMIProvider.jmiFormat1(reference.getType().getName());
			if (reference.getMultiplicity().getUpper() == -1) {
				outputln("java.util.Collection " + refList + " = " + variable_name + "." + JMIProvider.jmiAccessorName(reference) + "();");
				outputln("if (" + refList + ".size() > 0) {");
				outputln("outputln(\"<" + refName + ">\");");
				outputln("java.util.Iterator it = " + refList + ".iterator();");
				outputln("while (it.hasNext())");
				outputln("    outputln(\"<" + refTypeXmi + " xmi.idref=\\\"\" + getXMIId(it.next()) + \"\\\" />\");");
				outputln("outputln(\"</" + refName + ">\");");
				outputln("}");
			} else {
				if (reference.getMultiplicity().getLower() == 0)
					outputln("try {");
				outputln(refType + " " + refList + " = " + variable_name + "." + JMIProvider.jmiAccessorName(reference) + "();");
				outputln("if (" + refList + " != null) {");
				outputln("outputln(\"<" + refName + ">\");");
				outputln("outputln(\"<" + refTypeXmi + " xmi.idref=\\\"\" + getXMIId(" + refList + ") + \"\\\" />\");");
				outputln("outputln(\"</" + refName + ">\");");
				outputln("}");
				if (reference.getMultiplicity().getLower() == 0)
					outputln("} catch (java.lang.NullPointerException notSet) { /* Nothing to do */ }");
			}
		} catch (NullPointerException notSet) {
			// Nothing to do
		}
	}

	/**
	 * XMIExport generation for the root package.
	 * @param package_ The package.
	 * @param classes The classes of the package.
	 */
	public void rootPackage(MofPackage package_, MofClass classes[]) {
		_logger.log(Level.FINE, "XMIExport rootPackage()");
		outputln("public void rootPackageTemplate(Object _root_package) {");
		outputln("_logger.log(org.objectweb.modfact.jmi.logging.Level.FINE, \"rootPackageTemplate(\" + _root_package + \")\");");
		String packageName = JMIProvider.jmiFormat1(package_.getName());
		String model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		outputln(model_package_type + " _model_package = (" + model_package_type + ")_root_package;");
		for (int i = 0; i < classes.length; i++) {
			MofClass cls = (MofClass) classes[i];
			if (!cls.isAbstract())
				outputln(XMIIOHelper.format1FirstMin(cls.getName()) + "Template(_model_package);");
		}
		outputln("}");
		outputln();
	}

}
