/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.importer;

import javax.jmi.model.MofPackage;

import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.XMIIOHelper;
import org.objectweb.modfact.jmi.xmiio.common.AbstractXMIIOGenerator;


/**
 * The XMI Import Application generator.
 * This class allows the generation of XMI importer launcher.
 */
public class GenerationXMIImportApplication extends AbstractXMIIOGenerator {


    /** The IOR output file name. */
    private String _ior_filename = new String("");

    /**
     * XMIImport generation.
     * @param package_ The root package.
     * @param class_name_ The class name of the generated class.
     * @param class_name_application The application class name.
     */
    public void generateXMIImportApplication(MofPackage package_, String class_name_, String class_name_application_) {
    	String package_name_ = JMIProvider.jmiPackageExtentName(package_);
        head(package_, class_name_application_);
        constructors(package_name_, class_name_application_);
        startMethod(package_name_, package_, class_name_);
        mainMethod(class_name_application_);
        outputln("} // end class " + class_name_);
        out.flush();
    }

    /**
     * XMIImport generation for the head.
     * @param package_name_ The package name.
     * @param class_name_application_ The application class name.
     */
    public void head(MofPackage package_, String class_name_application_) {
		String packageQualifiedName = JMIProvider.qualifierOf(package_);
		outputln("package " + packageQualifiedName + ".xmi;");
        outputln();
        outputln("public class " + class_name_application_);
        outputln("     extends org.objectweb.modfact.jmi.service.cmdline.xmiio.XMIImportApplication {");
        outputln();
    }

    /**
     * XMIImport generation for the constructors.
     * @param package_name_ The package name.
     * @param class_name_application_ The application class name.
     */
    public void constructors(String package_name_, String class_name_application_) {
        outputln("public " + class_name_application_ + "() {");
        outputln("super(\"" + package_name_ + "\", \"" + class_name_application_ + "\", \"" + JMIProvider.jmiFormat1(package_name_) + "Package.ior\");");
        outputln("}");
        outputln();
    }

    /**
     * XMIImport generation for the main method.
     * @param class_name_application_ The application class name.
     */
    public void mainMethod(String class_name_application_) {
        outputln("/**");
        outputln(" * The main method");
        outputln(" */");
        outputln("public static void main (java.lang.String [] args) {");
        outputln(class_name_application_ + " application = new " + class_name_application_ + "();");
        outputln("application.runMain(args);");
        outputln("}");
        outputln();
    }

    /**
     * XMIImport generation for the start method.
     * @param package_name_ The package name.
     * @param class_name_ The class name.
     */
    public void startMethod(String package_name_, MofPackage package_, String class_name_) {
        outputln("/**");
        outputln(" * Starts the main function");
        outputln(" */");
        outputln("public int start(java.lang.String[] args) {");
        outputln("try {");
        outputln();
        outputln("// Creation of Model ");
        
        String model_package_type;
		model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		
		outputln(model_package_type + " pck = null;");
		outputln("try {");
		outputln("java.io.File metaModel = new java.io.File(metaModelFile);");
        String shortQualif = JMIProvider.shortQualifierOf(package_);
		outputln("pck = org.objectweb.modfact.jmi.repository." + shortQualif + "." + JMIProvider.jmiFormat1(package_name_) + "PackageImpl.create(metaModel.toURL().toString());");
		outputln("} catch (Exception e) {");
		outputln("System.err.println(\"exception : \" + e.getMessage());");
		outputln("e.printStackTrace();");
		outputln("}");
		
        outputln(class_name_ + " xmiImport = new " + class_name_ + "();");
        outputln("xmiImport.initiate(pck);");
        outputln(model_package_type + " _package = " + "(" + model_package_type + ") " + "xmiImport.parse(modelFile_);");
        outputln("System.out.println(\"Package is in the repository\");");
        output("} ");
        outputCatchException("org.xml.sax.SAXException");
        outputCatchException("javax.xml.parsers.ParserConfigurationException");
        outputCatchException("java.io.FileNotFoundException");
        outputCatchException("java.io.IOException");
        outputln();
        outputln("return 0;");
        outputln("}");
        outputln();
    }

    /**
     * XMIImport generation for a 'catch'.
     * @param exception_ The exception to catch.
     */
    public void outputCatchException(String exception_) {
        String exception_name;
        if (exception_.indexOf(".") != -1)
            exception_name = exception_.substring(exception_.lastIndexOf(".") + 1);
        else
            exception_name = exception_;
        outputln("catch (" + exception_ + " " + XMIIOHelper.format1FirstMin(exception_name) + ") {");
        outputln("System.err.println(\"" + JMIProvider.jmiFormat1(exception_name) + " : \" + " + XMIIOHelper.format1FirstMin(exception_name) + ".getMessage());");
        output("} ");
    }
    

}
