/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.importer;

import java.util.Iterator;
import java.util.Vector;

import javax.jmi.model.MofPackage;

import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.helper.XMIIOHelper;
import org.objectweb.modfact.jmi.xmiio.common.AbstractXMIIOGenerator;

/**
 * The XMI Import generator.
 * This class allows the generation of the XMI importer (into Java).
 */
public class GenerationXMIImportAssociations extends AbstractXMIIOGenerator {

	/**
	 * Default Constructor.
	 */
	public GenerationXMIImportAssociations() {
		super();
	}

	/**
	 * XMIImport generation.
	 * @param package_ The root package.
	 * @param class_name_ The class name of the generated class.
	 */
	public void generateXMIImportAssociations(javax.jmi.model.MofPackage package_, String class_name_)
		throws java.io.FileNotFoundException {
		packageModuleTemplate(package_, class_name_);
		out.flush();
	}

	/**
	 * XMIImport generation for constructor.
	 * @param class_name The class name.
	 * @param package_name The package name.
	 */
	public void constructor(String class_name, String package_name) {
		outputln("/** The XMIImportClasses class. */");
		outputln(package_name + "XMIImportClasses _xmi_import_classes;");
		outputln();
		outputln("/** Hashtables for types (Key / Model) */");
		outputln("private java.util.Hashtable _types;");
		outputln();
		outputln("/**");
		outputln(" * Default " + class_name + " Constructor");
		outputln(" * @param types The defined types.");
		outputln(" */");
		outputln("public " + class_name + "(java.util.Hashtable types) {");
		outputln("_types = types;");
		outputln("}");
		outputln();
		outputln("public void setXMIImportClasses (" + package_name + "XMIImportClasses xmi_import_classes) {");
		outputln("_xmi_import_classes = xmi_import_classes;");
		outputln("}");
		outputln();
	}

	java.util.Vector classes = new java.util.Vector();
	java.util.Vector associations = new java.util.Vector();

	private void processContents(MofPackage package_) {
		java.util.List contentsList = package_.getContents();
		Iterator it = contentsList.iterator();
		while (it.hasNext()) {
			javax.jmi.model.ModelElement contents = (javax.jmi.model.ModelElement) it.next();
			if (contents instanceof javax.jmi.model.MofPackage)
				processContents((javax.jmi.model.MofPackage) contents);
			else if (contents instanceof javax.jmi.model.Import)
				processContents((javax.jmi.model.MofPackage) ((javax.jmi.model.Import) contents).getImportedNamespace());
			else if (contents instanceof javax.jmi.model.MofClass)
				classes.addElement((javax.jmi.model.MofClass) contents);
			else if (contents instanceof javax.jmi.model.Association)
				associations.addElement((javax.jmi.model.Association) contents);
		}
	}

	/**
	 * Package processing.
	 * @param package_ The package to process.
	 * @param class_name_ The class name of the generated class.
	 */
	public void packageModuleTemplate(javax.jmi.model.MofPackage package_, String class_name_) {
		if (package_.getVisibility() == javax.jmi.model.VisibilityKindEnum.forName("public_vis")) {
			processContents(package_);

			java.util.Vector classes_ordered = new java.util.Vector();
			while (classes.size() != 0) {
				javax.jmi.model.MofClass courant = null;
				for (int i = 0; i < classes.size(); i++) {
					courant = (javax.jmi.model.MofClass) classes.elementAt(i);
					java.util.List supertypes = courant.getSupertypes();
					boolean is_top = true;
					Iterator itSup = supertypes.iterator();
					while (itSup.hasNext()) {
						if (classes.contains((javax.jmi.model.GeneralizableElement) itSup.next()))
							is_top = false;
					}
					if (is_top) {
						classes_ordered.addElement(courant);
						classes.removeElement(courant);
					}
				}
			}

			// // Get class contents to reduce CORBA calls during classes contents processing
			// for (int i = 0; i < classes_ordered.size(); i++)
			//     _helper.allContentsOfClasses((org.omg.mof.Model.Class) classes_ordered.elementAt(i));

			String packageName = JMIProvider.jmiPackageExtentName(package_);
			String packageQualifiedName = JMIProvider.qualifierOf(package_);
			outputln("package " + packageQualifiedName + ".xmi;");
			outputln();
			annotationTemplate(package_.getAnnotation());
			outputln("public class " + class_name_ + " extends org.objectweb.modfact.jmi.xmiio.importer.XMIImport {");
			classAttributes(packageName, package_, associations);
			constructor(class_name_, packageName);
			initiateMethod(packageName, package_, associations);
			associationTemplate(associations);
			outputln("} // end of class " + class_name_);
			outputln();
		}
	}

	/**
	* XMIImport generation for the class attributes.
	* @param package_name_ The package name.
	* @param associations_ The associations of the package.
	*/
	public void classAttributes(String package_name_, MofPackage package_, Vector associations_) {
		outputln("// Factories Declaration");
		String model_package_type;
		model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		outputln("private " + model_package_type + " _" + JMIProvider.jmiFormat2(package_name_) + "_package = null;");
		outputln();
		// Associations
		for (int i = 0; i < associations_.size(); i++) {
			javax.jmi.model.Association asso = (javax.jmi.model.Association) associations_.elementAt(i);
			String javaName = JMIProvider.jmiAssociationQualifiedName(asso);
			outputln("private " + javaName + " _" + JMIProvider.jmiFormat2(javaName) + "_class = null;");
		}
	}

	/**
	 * XMIImport generation for the initiate method.
	 * @param package_name_ The package name.
	 * @param associations_ The associations of the package.
	 */
	public void initiateMethod(String package_name_, MofPackage package_, Vector associations_) {
		String model_package_type;
		model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		outputln("public void initiate(" + model_package_type + " _model_package) {");
		outputln("// Factories creation");
		outputln("this._" + JMIProvider.jmiFormat2(package_name_) + "_package = _model_package;");
		// Associations
		initiateMethodAssociations(package_name_, package_, associations_);
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the "association attributes" of the initiate method.
	 * @param package_name_ The package name.
	 * @param classes_ The classes of the package.
	 */
	public void initiateMethodAssociations(String package_name_, MofPackage package_, Vector associations_) {
		for (int i = 0; i < associations_.size(); i++) {
			javax.jmi.model.Association asso = (javax.jmi.model.Association) associations_.elementAt(i);
			String javaName = JMIProvider.jmiFormat2(JMIProvider.jmiAssociationQualifiedName(asso));
			String method = "get" + JMIProvider.jmiAssociationName(asso) + "()";
			try {
				javax.jmi.model.ModelElement assoTmp = asso;
				while (!assoTmp.getContainer().equals(package_)) {
					method = "get" + JMIProvider.jmiPackageExtentName((MofPackage) assoTmp.getContainer()) + "()." + method;
					assoTmp = assoTmp.getContainer();
				}

			} catch (NullPointerException notSet) {
				// Nothing to do
			}
			outputln("_" + javaName + "_class = _model_package." + method + ";");
		}
	}

	/**
	 * XMIImport generation for the association template.
	 * @param associations_ The list of association of the package.
	 */
	public void associationTemplate(Vector associations_) {
		for (int i = 0; i < associations_.size(); i++) {
			javax.jmi.model.Association asso = (javax.jmi.model.Association) associations_.elementAt(i);
			String assoName = JMIProvider.jmiAssociationName(asso);
			String methodName = XMIIOHelper.format1FirstMin(assoName) + "Template";
			String param = JMIProvider.jmiFormat2(assoName) + "_element";
			javax.jmi.model.AssociationEnd[] ends = MofHelper.associationEndsOfAssociation(asso);
			boolean endsChangeable = true;
			for (int j = 0; j < ends.length; j++) {
				endsChangeable = (endsChangeable && ends[j].isChangeable());
			}
			if (endsChangeable) {
				outputln("public void " + methodName + "(org.w3c.dom.Element " + param + ") {");
				for (int j = 0; j < ends.length; j++) {
					outputln(
						JMIProvider.jmiClassifierQualifiedName(ends[j].getType())
							+ " "
							+ XMIIOHelper.testJavaConflict(XMIIOHelper.format1FirstMin(ends[j].getName()))
							+ " = null;");
				}
				outputln("org.w3c.dom.NodeList _child_nodes = " + param + ".getChildNodes();");
				outputln("for (int i=0 ; i<_child_nodes.getLength() ; i++) {");
				outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child_nodes.item(i);");
				for (int j = 0; j < ends.length; j++) {
					String ends_variable = XMIIOHelper.testJavaConflict(XMIIOHelper.format1FirstMin(ends[j].getName()));
					String type_variable = JMIProvider.jmiClassifierQualifiedName(ends[j].getType());
					String end_type_name = JMIProvider.findSubstituteName(ends[j].getType());
					if (end_type_name == null)
						end_type_name = JMIProvider.classNameFormat(ends[j].getType().getName());
					String methodTemplate = "_xmi_import_classes." + XMIIOHelper.format1FirstMin(end_type_name) + "Template(_current)";
					output("if (_current.getNodeName().endsWith(\"" + JMIProvider.jmiFormat1(ends[j].getType().getName()) + "\")");
					try {
						javax.jmi.model.MofClass[] subclasses = XMIIOHelper.subClasses(ends[j].getType());
						for (int k = 0; k < subclasses.length; k++) {
							output(" || _current.getNodeName().endsWith(\"" + JMIProvider.jmiFormat1(subclasses[k].getName()) + "\")");
							if (k % 2 == 0 && k < subclasses.length - 1) {
								outputln();
								output(TABULATION);
							}
						}
					} catch (NullPointerException notSet) {
						// Nothing to do.
					}
					outputln(") {");

					if (ends[j].getType().isAbstract()) {
						outputln(
							"org.objectweb.modfact.jmi.xmiio.mofobject.MOFObject "
								+ ends_variable
								+ "Obj = "
								+ methodTemplate
								+ ".getFirstObject();");
					} else {
						outputln("org.objectweb.modfact.jmi.xmiio.mofobject.MOFObject " + ends_variable + "Obj = " + methodTemplate + ";");
					}
					outputln("if (" + ends_variable + "Obj.isAnObject())");
					outputln(TABULATION + ends_variable + " = (" + type_variable + ")" + ends_variable + "Obj.get();");
					outputln("else");
					outputln(TABULATION + ends_variable + " = (" + type_variable + ") _types.get(" + ends_variable + "Obj.get());");
					outputln("}");
				}
                // every other time (1/2)
                outputln("if (i%2 == 1)");
				output(TABULATION + "_" + JMIProvider.jmiFormat2(JMIProvider.jmiAssociationQualifiedName(asso)) + "_class.add(");
				for (int j = 0; j < ends.length; j++) {
					output(XMIIOHelper.testJavaConflict(XMIIOHelper.format1FirstMin(ends[j].getName())));
					if (j < ends.length - 1)
						output(", ");
				}
				outputln(");");
                outputln("}");
				outputln("}");
				outputln();
			}
		}
		outputln();
	}

}
