/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.importer;

import java.util.Iterator;
import java.util.Vector;

import javax.jmi.model.MofClass;
import javax.jmi.model.MofPackage;

import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.XMIIOHelper;
import org.objectweb.modfact.jmi.xmiio.common.AbstractXMIIOGenerator;

/**
 * The XMI Import generator.
 * This class allows the generation of the XMI importer (into Java).
 */
public class GenerationXMIImportAttributes extends AbstractXMIIOGenerator {

	/** The generated association attributes methods. */
	private java.util.List _associationAttributes;

	/** The classes of the package. */
	private java.util.Vector classes = new java.util.Vector();

	/** The associations of the package. */
	private java.util.Vector associations = new java.util.Vector();

	/**
	 * Default Constructor.
	 */
	public GenerationXMIImportAttributes() {
		super();
		_associationAttributes = new java.util.ArrayList();
	}

	/**
	 * XMIImport generation.
	 * @param package_ The root package.
	 * @param class_name_ The class name of the generated class.
	 */
	public void generateXMIImportAttributes(javax.jmi.model.MofPackage package_, String class_name_) throws java.io.FileNotFoundException {
		packageModuleTemplate(package_, class_name_);
		out.flush();
	}

	/**
	 * XMIImport generation for constructor.
	 * @param class_name The class name.
	 * @param package_name The package name.
	 */
	public void constructor(String class_name, String package_name) {
		outputln("/** The XMIImportClasses class. */");
		outputln(package_name + "XMIImportClasses _xmi_import_classes;");
		outputln();
		outputln("/** Hashtables for types (Key / Model) */");
		outputln("private java.util.Hashtable _types;");
		outputln();
		outputln("/**");
		outputln(" * Default " + class_name + " Constructor");
		outputln(" */");
		outputln("public " + class_name + "(java.util.Hashtable types) {");
		outputln("_types = types;");
		outputln("}");
		outputln();
		outputln("public void setXMIImportClasses (" + package_name + "XMIImportClasses xmi_import_classes) {");
		outputln("_xmi_import_classes = xmi_import_classes;");
		outputln("}");
		outputln();
	}

	/**
	 * Parse the content of the package.
	 * @param package_ The root package.
	 */
	private void processContents(javax.jmi.model.MofPackage package_) {
		java.util.List contentsList = package_.getContents();
		Iterator it = contentsList.iterator();
		while (it.hasNext()) {
			javax.jmi.model.ModelElement contents = (javax.jmi.model.ModelElement) it.next();
			if (contents instanceof javax.jmi.model.MofPackage)
				processContents((javax.jmi.model.MofPackage) contents);
			else if (contents instanceof javax.jmi.model.Import)
				processContents((javax.jmi.model.MofPackage) ((javax.jmi.model.Import) contents).getImportedNamespace());
			else if (contents instanceof javax.jmi.model.MofClass)
				classes.addElement((javax.jmi.model.MofClass) contents);
			else if (contents instanceof javax.jmi.model.Association)
				associations.addElement((javax.jmi.model.Association) contents);
		}
	}

	/**
	 * Package processing.
	 * @param package_ The package to process.
	 * @param class_name_ The class name of the generated class.
	 */
	public void packageModuleTemplate(javax.jmi.model.MofPackage package_, String class_name_) {
		if (package_.getVisibility() == javax.jmi.model.VisibilityKindEnum.forName("public_vis")) {
			processContents(package_);

			java.util.Vector classes_ordered = new java.util.Vector();
			while (classes.size() != 0) {
				javax.jmi.model.MofClass courant = null;
				for (int i = 0; i < classes.size(); i++) {
					courant = (javax.jmi.model.MofClass) classes.elementAt(i);
					java.util.List supertypes = courant.getSupertypes();
					boolean is_top = true;
					Iterator itSup = supertypes.iterator();
					while (itSup.hasNext()) {
						if (classes.contains((javax.jmi.model.GeneralizableElement) itSup.next()))
							is_top = false;
					}
					if (is_top) {
						classes_ordered.addElement(courant);
						classes.removeElement(courant);
					}
				}
			}

			String packageName = JMIProvider.jmiPackageExtentName(package_);
			String packageQualifiedName = JMIProvider.qualifierOf(package_);
			outputln("package " + packageQualifiedName + ".xmi;");
			outputln();
			annotationTemplate(package_.getAnnotation());
			outputln("public class " + class_name_ + " extends org.objectweb.modfact.jmi.xmiio.importer.XMIImport {");
			constructor(class_name_, packageName);
			classAttributes(packageName, package_);
			associationAttributes(packageName, associations);
			initiateMethod(packageName, package_);
			readingAttributes(packageName, classes_ordered);
			putAssociationsValues(packageName, associations);
			outputln("} // end of class " + class_name_);
			outputln();
		}
	}

	/**
	 * XMIImport generation for the class attributes.
	 * @param package_name_ The package name.
	 * @param package_ The package.
	 */
	public void classAttributes(String package_name_, MofPackage package_) {
		outputln("// Factories Declaration");
		String model_package_type;
		model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		outputln("private " + model_package_type + " _" + JMIProvider.jmiFormat2(package_name_) + "_package = null;");
		outputln();
	}

	/**
	 * XMIImport generation for the association attributes.
	 * @param package_name_ The package name.
	 * @param associations_ The associations of the package.
	 */
	public void associationAttributes(String package_name_, Vector associations_) {
		outputln("// Hashtables for associations (Model / Key Vector)");
		for (int i = 0; i < associations_.size(); i++) {
			javax.jmi.model.Association asso = (javax.jmi.model.Association) associations_.elementAt(i);
			try {
				javax.jmi.model.Reference[] refs = XMIIOHelper.getReferences(asso);
				for (int j = 0; j < refs.length; j++) {
					outputln(
						"private java.util.Hashtable _"
							+ JMIProvider.jmiFormat2(JMIProvider.jmiAssociationName(asso))
							+ "_"
							+ JMIProvider.jmiFormat2(refs[j].getName())
							+ " = new java.util.Hashtable();");
				}
			} catch (NullPointerException notSet) {
				// Nothing to do
			}
		}
		outputln();
	}

	/**
	 * XMIImport generation for the initiate method.
	 * @param package_name_ The package name.
	 * @param package_ The package.
	 */
	public void initiateMethod(String package_name_, MofPackage package_) {
		String model_package_type;
		model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		outputln("public void initiate(" + model_package_type + " _model_package) {");
		outputln("// Factories creation");
		outputln("this._" + JMIProvider.jmiFormat2(package_name_) + "_package = _model_package;");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the attributes reading methods.
	 * @param package_name_ The package name.
	 * @param classes_ The classes of the package.
	 */
	public void readingAttributes(String package_name_, Vector classes_) {
		outputln("// ==================================================================");
		outputln("//  readXXXYYY with XXX an entity and YYY its property");
		outputln("// ==================================================================");
		outputln();
		// Classes
		for (int i = 0; i < classes_.size(); i++) {
			javax.jmi.model.MofClass cls = (javax.jmi.model.MofClass) classes_.elementAt(i);
			String clsName = JMIProvider.jmiClassName(cls);
			java.util.List contents = cls.getContents();
			Iterator it = contents.iterator();
			while (it.hasNext()) {
				javax.jmi.model.ModelElement content = (javax.jmi.model.ModelElement) it.next();
				if (content instanceof javax.jmi.model.Attribute) {
					javax.jmi.model.Attribute attr = (javax.jmi.model.Attribute) content;
					javax.jmi.model.Classifier type = attr.getType();
					String attrName = attr.getName();
					if (attr.isChangeable()) {
						if (type instanceof javax.jmi.model.DataType) {
							if (type instanceof javax.jmi.model.EnumerationType) {
								java.util.List associatedTags = new java.util.ArrayList();
								try {
									java.util.List clsContents = type.getContainer().getContents();
									Iterator itContents = clsContents.iterator();
									while (itContents.hasNext()) {
										javax.jmi.model.ModelElement clsContent = (javax.jmi.model.ModelElement) itContents.next();
										if (clsContent instanceof javax.jmi.model.Tag) {
											javax.jmi.model.Tag tag = (javax.jmi.model.Tag) clsContent;
											java.util.Collection elements = tag.getElements();
											Iterator itElements = elements.iterator();
											while (itElements.hasNext()) {
												javax.jmi.model.ModelElement element = (javax.jmi.model.ModelElement) itElements.next();
												if (element.equals(type))
													associatedTags.add(tag);
											}
										}
									}
								} catch (NullPointerException notSet) {
									// Nothing to do
								}
								readClassEnumerationTypeAttributeMethod(
									package_name_,
									cls,
									attr,
									(javax.jmi.model.EnumerationType) type,
									associatedTags);
							} else if (type instanceof javax.jmi.model.StructureType) {
								readClassStructureTypeAttributeMethod(package_name_, cls, attr);
							} else if (type instanceof javax.jmi.model.AliasType) {
								//readClassAliasTypeMethod(package_name_, cls, attr, type_code.content_type());
							} else {
								readClassAttributeMethod(package_name_, cls, (javax.jmi.model.Attribute) content, type);
							}
						} else {
							readClassAttributeMethod(package_name_, cls, (javax.jmi.model.Attribute) content, type);
						}
					}
				} else if (content instanceof javax.jmi.model.Reference) {
					try {
						javax.jmi.model.Reference ref = (javax.jmi.model.Reference) content;
						javax.jmi.model.AssociationEnd assoEnd = ref.getReferencedEnd();
						javax.jmi.model.AssociationEnd otherEnd = XMIIOHelper.otherAssociationEnd(assoEnd);
						boolean none = (assoEnd.getAggregation() == javax.jmi.model.AggregationKindEnum.forName("none"));
						boolean composite = (assoEnd.getAggregation() == javax.jmi.model.AggregationKindEnum.forName("composite"));

						if (none) {
							if (composite) {
								javax.jmi.model.MofClass[] subClasses = XMIIOHelper.subClasses(cls);
								readCompositeAssociationAttributeMethod(package_name_, cls, subClasses, assoEnd.getName());
							}
							javax.jmi.model.Association asso = (javax.jmi.model.Association) assoEnd.getContainer();
							javax.jmi.model.Reference[] refs = XMIIOHelper.getReferences(asso);
							for (int r = 0; r < refs.length; r++) {
								String hashtable =
									"_"
										+ JMIProvider.jmiFormat2(JMIProvider.jmiAssociationName(asso))
										+ "_"
										+ JMIProvider.jmiFormat2(refs[r].getName());
								try {
									readAssociationAttributeMethod(package_name_, refs[r].getContainer(), refs[r], hashtable);
								} catch (NullPointerException notSet) {
									notSet.printStackTrace();
								}
							}
						}
					} catch (NullPointerException notSet) {
						// Nothing to do
					}
				}
			}
		}
	}

	/**
	 * XMIImport generation for the class attribute reading method.
	 * @param package_name_ The package name.
	 * @param classes_ The class.
	 * @param attribute_ The attribute.
	 * @param type_ The Type of the attribute.
	 */
	public void readClassAttributeMethod(
		String package_name_,
		javax.jmi.model.MofClass class_,
		javax.jmi.model.Attribute attribute_,
		javax.jmi.model.Classifier type_) {
		String type_name_ = type_.getName();
		String type = JMIProvider.jmiFormat1(type_name_);
		String attrName = JMIProvider.findSubstituteName(attribute_);
		if (attrName == null)
			attrName = attribute_.getName();
		String attrName4 = XMIIOHelper.format1FirstMin(attrName);
		String className = JMIProvider.jmiClassName(class_);
		outputln("/**");
		outputln(" * Read the '" + JMIProvider.jmiFormat1(attrName) + "' attribute of the " + JMIProvider.jmiFormat1(className) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + JMIProvider.jmiFormat1(className) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
        output(
            "public void read"
                + JMIProvider.classNameFormat(className)
                + JMIProvider.jmiFormat1(attrName)
                + "(");
		//output(
		//	"public void read"
		//		+ JMIProvider.classNameFormat(className)
		//		+ JMIProvider.jmiFormat1(JMIProvider.attributNameFormat(attrName))
		//		+ "(");
		outputln("org.w3c.dom.Element parent, " + JMIProvider.jmiClassQualifiedName(class_) + " model, org.w3c.dom.Element element) {");
		if (attribute_.getMultiplicity().getUpper() == 1) {
			if (type_ instanceof javax.jmi.model.PrimitiveType) {
				String method = primitiveTypeToReadMethod((javax.jmi.model.PrimitiveType) type_);
				outputln("if (parent.hasAttribute(\"" + attrName4 + "\"))");
				outputln(
					"    model."
						+ JMIProvider.jmiMutatorName(attribute_)
						+ "("
						+ method
						+ "(parent.getAttribute(\""
						+ attrName4
						+ "\")));");
				outputln("else if (element != null && nodeNameCorrespondsTo(element, \"" + JMIProvider.jmiFormat1(attrName) + "\")) {");
				outputln("if (element.getFirstChild() != null)");
				outputln(
					"    model."
						+ JMIProvider.jmiMutatorName(attribute_)
						+ "("
						+ method
						+ "(element.getFirstChild().getNodeValue().trim()));");
				outputln("else");
				outputln(
					"    model."
						+ JMIProvider.jmiMutatorName(attribute_)
						+ "("
						+ method
						+ "(element.getAttribute(\"xmi.value\").trim()));");
				outputln("}");
			} else {
				outputln(
					"if (element != null && element.getFirstChild() != null && nodeNameCorrespondsTo(element, \""
						+ JMIProvider.jmiFormat1(attrName)
						+ "\"))");
				String paramType = JMIProvider.jmiClassifierQualifiedName(attribute_.getType());
				String getMethod = ".get()";
				if (attribute_.getType().isAbstract())
					getMethod = ".getFirstObject()";
				outputln(
					"    model."
						+ JMIProvider.jmiMutatorName(attribute_)
						+ "(("
						+ paramType
						+ ")_xmi_import_classes."
						+ XMIIOHelper.format1FirstMin(JMIProvider.classNameFormat(type_name_))
						+ "Template((org.w3c.dom.Element)element.getFirstChild())"
						+ getMethod
						+ ");");
			}
		} else {
			if (type_ instanceof javax.jmi.model.PrimitiveType) {
				String method = primitiveTypeToReadMethod((javax.jmi.model.PrimitiveType) type_);
				outputln("if (parent.hasAttribute(\"" + attrName4 + "\"))");
				outputln(
					"    model."
						+ JMIProvider.jmiAccessorName(attribute_)
						+ "().add("
						+ method
						+ "(parent.getAttribute(\""
						+ attrName4
						+ "\")));");
				outputln("else if (element != null && nodeNameCorrespondsTo(element, \"" + JMIProvider.jmiFormat1(attrName) + "\")) {");
				outputln("if (element.getFirstChild() != null)");
				outputln(
					"    model."
						+ JMIProvider.jmiAccessorName(attribute_)
						+ "().add("
						+ method
						+ "(element.getFirstChild().getNodeValue().trim()));");
				outputln("else");
				outputln(
					"    model."
						+ JMIProvider.jmiAccessorName(attribute_)
						+ "().add("
						+ method
						+ "(element.getAttribute(\"xmi.value\").trim()));");
				outputln("}");
			} else {
				outputln(
					"if (element != null && element.getFirstChild() != null && nodeNameCorrespondsTo(element, \""
						+ JMIProvider.jmiFormat1(attrName)
						+ "\"))");
				String paramType = JMIProvider.jmiClassifierQualifiedName(attribute_.getType());
				if (attribute_.getType().isAbstract()) {
                    outputln("org.objectweb.modfact.jmi.xmiio.mofobject.MOFObjectList mofList = _xmi_import_classes." + XMIIOHelper.format1FirstMin(type) + "Template((org.w3c.dom.Element)element.getFirstChild());");
                    outputln("java.util.Iterator it = mofList.getObjects();");
                    outputln("while (it.hasNext()) {");
                    outputln(TABULATION + "model."+ JMIProvider.jmiAccessorName(attribute_) + "().add((" + paramType + ")it.next());");
                    outputln("}");
				} else {
					outputln(
						"    model."
							+ JMIProvider.jmiAccessorName(attribute_)
							+ "().add(("
							+ paramType
							+ ")_xmi_import_classes."
							+ XMIIOHelper.format1FirstMin(type)
							+ "Template((org.w3c.dom.Element)element.getFirstChild()).get());");
				}
			}
		}
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the class attribute reading method (for a TypeCode attribute).
	 * @param package_name_ The package name.
	 * @param classes_ The class.
	 * @param attribute_name_ The attribute name.
	 */
	public void readClassTypeCodeAttributeMethod(String package_name_, javax.jmi.model.MofClass class_, String attribute_name_) {
		String class_name = JMIProvider.jmiClassName(class_);
		outputln("/**");
		outputln(
			" * Read the '" + JMIProvider.jmiFormat1(attribute_name_) + "' attribute of the " + JMIProvider.jmiFormat1(class_name) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + JMIProvider.jmiFormat1(class_name) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output("public void read" + JMIProvider.jmiFormat1(class_name) + JMIProvider.jmiFormat1(attribute_name_) + "(");
		outputln("org.w3c.dom.Element parent, " + JMIProvider.jmiClassQualifiedName(class_) + " model, org.w3c.dom.Element element) {");
		outputln("String repositoryID = \"IDL:\" + JMIProvider.jmiFormat1(model.getName());");
		outputln("if (element != null && nodeNameCorrespondsTo(element, \"" + JMIProvider.jmiFormat1(attribute_name_) + "\")) {");
		outputln("org.w3c.dom.Element _corba_typecode = (org.w3c.dom.Element) element.getElementsByTagName(\"XMI.CorbaTypeCode\").item(0);");
		outputln("org.w3c.dom.Element _element = (org.w3c.dom.Element) _corba_typecode.getChildNodes().item(0);");
		outputln("model.set_" + JMIProvider.jmiFormat2(attribute_name_) + "(xml2TypeCode(_element, repositoryID));");
		outputln("}");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the class attribute reading method (for an EnumarationType attribute).
	 * @param package_name_ The package name.
	 * @param classes_ The class.
	 * @param attribute_ The attribute.
	 * @param type_ The EnumerationType.
	 * @param associated_tags_ The tags associated to the Data Type.
	 */
	public void readClassEnumerationTypeAttributeMethod(
		String package_name_,
		javax.jmi.model.MofClass class_,
		javax.jmi.model.Attribute attribute_,
		javax.jmi.model.EnumerationType type,
		java.util.List associated_tags_) {
		String attrName = attribute_.getName();
		String attrName4 = XMIIOHelper.format1FirstMin(attrName);
		String className = JMIProvider.jmiClassName(class_);
		outputln("/**");
		outputln(" * Read the '" + JMIProvider.jmiFormat1(attrName) + "' attribute of the " + JMIProvider.jmiFormat1(className) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + JMIProvider.jmiFormat1(className) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output(
			"public void read"
				+ JMIProvider.jmiFormat1(className)
				+ JMIProvider.jmiFormat1(attrName)
				+ "(");
        //output(
        //    "public void read"
        //        + JMIProvider.jmiFormat1(className)
        //        + JMIProvider.jmiFormat1(JMIProvider.attributNameFormat(attrName))
        //        + "(");
		outputln("org.w3c.dom.Element parent, " + JMIProvider.jmiClassQualifiedName(class_) + " model, org.w3c.dom.Element element) {");
		outputln("java.lang.String " + attrName4 + " = new java.lang.String();");
		outputln("if (parent.hasAttribute(\"" + attrName4 + "\"))");
		outputln(TABULATION + attrName4 + " = parent.getAttribute(\"" + attrName4 + "\");");
		outputln("else if (element != null && nodeNameCorrespondsTo(element, \"" + JMIProvider.jmiFormat1(attrName) + "\")) {");
		outputln("if (element.getFirstChild() != null)");
		outputln(TABULATION + attrName4 + " = element.getFirstChild().getNodeValue().trim();");
		outputln("else");
		outputln(TABULATION + attrName4 + " = element.getAttribute(\"xmi.value\").trim();");
		outputln("}");
		outputln("if (" + attrName4 + " != null && " + attrName4 + ".length() > 0) {");

		// Get the prefix
		String prefix = new String();
		java.util.Iterator it = associated_tags_.iterator();
		while (it.hasNext()) {
			javax.jmi.model.Tag tag = (javax.jmi.model.Tag) it.next();
			if (tag.getTagId().equals("org.omg.xmi.enumerationUnprefix")) {
				java.util.List values = tag.getValues();
				Iterator itValues = values.iterator();
				while (itValues.hasNext()) {
					Object v = itValues.next();
					if (v instanceof String)
						prefix = v.toString();
				}
			}
		}
		Iterator itLabels = type.getLabels().iterator();
		while (itLabels.hasNext()) {
			String label = (String) itLabels.next();
			String member_name = XMIIOHelper.replaceFirst(label, prefix, "");
			outputln("if (" + attrName4 + ".endsWith(\"" + member_name + "\"))");
			outputln(
				TABULATION
					+ "model.set"
					+ JMIProvider.jmiFormat1(attrName)
					+ "("
					+ JMIProvider.jmiClassifierQualifiedName(attribute_.getType())
					+ "Enum.forName(\""
					+ label
					+ "\"));");
		}
		outputln("}");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the class attribute reading method (for a StructureType attribute).
	 * @param package_name_ The package name.
	 * @param classes_ The class.
	 * @param attribute_ The attribute.
	 */
	public void readClassStructureTypeAttributeMethod(
		String package_name_,
		javax.jmi.model.MofClass class_,
		javax.jmi.model.Attribute attribute_) {
		String attrName = attribute_.getName();
		String className = JMIProvider.jmiClassName(class_);
		outputln("/**");
		outputln(" * Read the '" + JMIProvider.jmiFormat1(attrName) + "' attribute of the " + JMIProvider.jmiFormat1(className) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + JMIProvider.jmiFormat1(className) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output("public void read" + JMIProvider.jmiFormat1(className) + JMIProvider.jmiFormat1(attrName) + "(");
		outputln("org.w3c.dom.Element parent, " + JMIProvider.jmiClassQualifiedName(class_) + " model, org.w3c.dom.Element element) {");
		outputln("if (element != null && nodeNameCorrespondsTo(element, \"" + JMIProvider.jmiFormat1(attrName) + "\")) {");

		outputln("org.w3c.dom.NodeList _" + JMIProvider.jmiFormat2(attrName) + " = element.getChildNodes();");
		outputln("for (int j = 0; j < _" + JMIProvider.jmiFormat2(attrName) + ".getLength(); j++) {");
		outputln("org.w3c.dom.Element _item = (org.w3c.dom.Element) _" + JMIProvider.jmiFormat2(attrName) + ".item(j);");
		outputln("if (_item.getFirstChild() != null) {");
		outputln("java.lang.String value = _item.getFirstChild().getNodeValue().trim();");

		outputln("}");
		outputln("}");
		output(
			"model.set" + JMIProvider.jmiFormat1(attrName) + "(new " + JMIProvider.jmiClassifierQualifiedName(attribute_.getType()) + "(");
		outputln("));");
		outputln("}");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the class attribute reading method (for an AliasType attribute).
	 * @param package_name_ The package name.
	 * @param cls The Model class.
	 * @param attribute_ The AliasType attribute.
	 * @param content_type_ The Content Type of the AliasType.
	 */
	public void readClassAliasTypeMethod(
		String package_name_,
		javax.jmi.model.MofClass cls,
		javax.jmi.model.Attribute attribute_,
		org.omg.CORBA.TypeCode content_type_) {
		String attrName = attribute_.getName();
		String attrName4 = XMIIOHelper.format1FirstMin(attrName);
		String attrName1 = JMIProvider.jmiFormat1(attrName);
		String attrName2 = JMIProvider.jmiFormat2(attrName);
		String clsName = JMIProvider.jmiClassName(cls);
		outputln("/**");
		outputln(" * Read the '" + JMIProvider.jmiFormat1(attrName) + "' attribute of the " + JMIProvider.jmiFormat1(clsName) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + JMIProvider.jmiFormat1(clsName) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output("public void read" + JMIProvider.jmiFormat1(clsName) + JMIProvider.jmiFormat1(attrName) + "(");
		outputln("org.w3c.dom.Element parent, " + JMIProvider.jmiClassQualifiedName(cls) + " model, org.w3c.dom.Element element) {");
		String contentType = typeCode2ReadingMethod(content_type_);
		outputln("if (parent.hasAttribute(\"" + attrName4 + "\"))");
		if (attribute_.getMultiplicity().getUpper() == 1) {
			outputln("    model.set" + attrName1 + "(" + contentType + "(parent.getAttribute(\"" + attrName4 + "\")));");
			outputln(
				"else if (element != null && element.getFirstChild() != null && nodeNameCorrespondsTo(element, \""
					+ JMIProvider.jmiFormat1(attrName)
					+ "\"))");
			outputln("    model.set" + attrName1 + "(" + contentType + "(element.getFirstChild().getNodeValue().trim()));");
		} else {
			outputln("    model.get" + attrName1 + "().add(" + contentType + "(parent.getAttribute(\"" + attrName4 + "\")));");
			outputln(
				"else if (element != null && element.getFirstChild() != null && nodeNameCorrespondsTo(element, \""
					+ JMIProvider.jmiFormat1(attrName)
					+ "\"))");
			outputln("    model.get" + attrName1 + "().add(" + contentType + "(element.getFirstChild().getNodeValue().trim()));");
		}
		outputln("}");
		outputln();
	}

	/**
	 * Pass from the TypeCode to the name of the associated reading method.
	 * @param type_code The TypeCode.
	 * @return The name of the reading method.
	 */
	public String typeCode2ReadingMethod(org.omg.CORBA.TypeCode type_code) {
		switch (type_code.kind().value()) {
			case org.omg.CORBA.TCKind._tk_boolean :
				return "readBooleanType";
			case org.omg.CORBA.TCKind._tk_short :
			case org.omg.CORBA.TCKind._tk_ushort :
			case org.omg.CORBA.TCKind._tk_long :
			case org.omg.CORBA.TCKind._tk_ulong :
				return "readIntegerType";
			case org.omg.CORBA.TCKind._tk_longlong :
			case org.omg.CORBA.TCKind._tk_ulonglong :
				return "readIntegerType";
			case org.omg.CORBA.TCKind._tk_double :
				return "readDoubleType";
			case org.omg.CORBA.TCKind._tk_float :
				return "readFloatType";
			case org.omg.CORBA.TCKind._tk_string :
			case org.omg.CORBA.TCKind._tk_wstring :
				return "readStringType";
			default :
				return "readStringType";
		}
	}

	/**
	 * Pass from the PrimitiveType to the name of the associated reading method.
	 * @param type The PrimitiveType.
	 * @return The name of the reading method.
	 */
	public String primitiveTypeToReadMethod(javax.jmi.model.PrimitiveType type_) {
		String typeName = type_.getName().toLowerCase();
		if (typeName.equals("string")) {
			return "java.lang.String.valueOf";
		} else if (typeName.equals("boolean")) {
			return "readBooleanType";
		} else if (typeName.equals("integer") || typeName.equals("long")) {
			return "java.lang.Integer.parseInt";
		} else if (typeName.equals("float")) {
			return "java.lang.Float.parseFloat";
		} else if (typeName.equals("double")) {
			return "java.lang.Double.parseDouble";
		} else {
			throw new RuntimeException("unknown primitive type: " + type_.getName());
		}
	}

	/**
	 * XMIImport generation for the association attribute reading method.
	 * @param package_name_ The package name.
	 * @param class_ The class name.
	 * @param ref_ The reference attribute.
	 * @param hashtable_ The name of the hashtable.
	 */
	public void readAssociationAttributeMethod(
		String package_name_,
		javax.jmi.model.Namespace class_,
		javax.jmi.model.Reference ref_,
		String hashtable_) {
		String className = JMIProvider.findSubstituteName(class_);
		if (className == null)
			className = class_.getName();
		className = JMIProvider.classNameFormat(className);
		String refName = JMIProvider.findSubstituteName(ref_);
		if (refName == null)
			refName = ref_.getName();
		refName = JMIProvider.classNameFormat(refName);
		String typeName = class_.getName();
		String signature = "public void read" + JMIProvider.jmiFormat1(className) + JMIProvider.jmiFormat1(refName) + "(";
		if (!_associationAttributes.contains(signature)) {
			outputln("/**");
			outputln(" * Read the '" + JMIProvider.jmiFormat1(refName) + "' attribute of the " + JMIProvider.jmiFormat1(className) + ".");
			outputln(" * @param parent The parent of the current element in the XMI file.");
			outputln(" * @param model The " + JMIProvider.jmiFormat1(className) + " to set.");
			outputln(" * @param element The current element in the XMI file.");
			outputln(" */");
			_associationAttributes.add(signature);
			output(signature);
			outputln(
				"org.w3c.dom.Element parent, "
					+ JMIProvider.jmiClassQualifiedName((MofClass) class_)
					+ " model, org.w3c.dom.Element element) {");

			//// If generation problem with the '_' character : use '_helper.format#(refName)' instead of 'refName'.
			outputln(
				"if (element != null && element.getNodeName().trim().endsWith(\""
					+ JMIProvider.jmiFormat1(typeName)
					+ "."
					+ XMIIOHelper.format1FirstMin(ref_.getName())
					+ "\")) {");
			outputln("org.w3c.dom.NodeList _child = element.getChildNodes();");
			if (ref_.getType().isAbstract()) {
				outputln(
					"org.objectweb.modfact.jmi.xmiio.mofobject.MOFObjectList list = _xmi_import_classes."
						+ XMIIOHelper.format1FirstMin(JMIProvider.classNameFormat(ref_.getType().getName()))
						+ "Template(element);");
				outputln("java.util.Iterator it = list.getObjects();");
				outputln("while (it.hasNext())");
				if (ref_.getMultiplicity().getUpper() == 1) {
					outputln(
						TABULATION
							+ "model.set"
							+ JMIProvider.jmiFormat1(ref_.getName())
							+ "(("
							+ JMIProvider.jmiClassifierQualifiedName(ref_.getType())
							+ ")it.next());");
				} else {
					outputln(
						TABULATION
							+ "model.get"
							+ JMIProvider.jmiFormat1(ref_.getName())
							+ "().add(("
							+ JMIProvider.jmiClassifierQualifiedName(ref_.getType())
							+ ")it.next());");
				}
				outputln("it = list.getXmiIdRef();");
				outputln("while (it.hasNext())");
				outputln(
					"org.objectweb.modfact.jmi.helper.XMIIOHelper.addHashtableValue("
						+ hashtable_
						+ ", model, (java.lang.String)it.next());");
			} else {
				outputln("int i = 0;");
				outputln("while (i<_child.getLength() || (_child.getLength()==0 && i==0)) {");
				outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child.item(i);");
				outputln("if (_current != null) {");
				String variableType = JMIProvider.jmiClassifierQualifiedName(ref_.getType());
				String ref_type_name = JMIProvider.findSubstituteName(ref_.getType());
				if (ref_type_name == null)
					ref_type_name = JMIProvider.classNameFormat(ref_.getType().getName());
				outputln(
					"org.objectweb.modfact.jmi.xmiio.mofobject.MOFObject mof_obj = _xmi_import_classes."
						+ XMIIOHelper.format1FirstMin(ref_type_name)
						+ "Template(_current);");
				outputln("if (mof_obj.isAnObject())");
				if (ref_.getMultiplicity().getUpper() == 1) {
					outputln(TABULATION + "model.set" + JMIProvider.jmiFormat1(ref_.getName()) + "((" + variableType + ")mof_obj.get());");
				} else {
					outputln(
						TABULATION + "model.get" + JMIProvider.jmiFormat1(ref_.getName()) + "().add((" + variableType + ")mof_obj.get());");
				}
				outputln("else if (mof_obj.isValid())");
				outputln(
					TABULATION
						+ "org.objectweb.modfact.jmi.helper.XMIIOHelper.addHashtableValue("
						+ hashtable_
						+ ", model, mof_obj.get());");
				outputln("}");
				outputln("i++;");
				outputln("}");
			}
			outputln("}");
			outputln("}");
			outputln();
		}
	}

	/**
	 * XMIImport generation for the composite association attribute method.
	 * @param package_name_ The package name.
	 * @param class_ The class.
	 * @param subclass_ The subclass.
	 * @param attribute_name_ The attribute name.
	 */
	public void readCompositeAssociationAttributeMethod(
		String package_name_,
		javax.jmi.model.MofClass class_,
		javax.jmi.model.MofClass[] subclass_,
		String attribute_name_) {
		String className = JMIProvider.jmiClassName(class_);
		outputln("/**");
		outputln(
			" * Read the '" + JMIProvider.jmiFormat1(attribute_name_) + "' attribute of the " + JMIProvider.jmiFormat1(className) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + JMIProvider.jmiFormat1(attribute_name_) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output(
			"public void read"
				+ JMIProvider.classNameFormat(className)
				+ JMIProvider.jmiFormat1(JMIProvider.attributNameFormat(attribute_name_))
				+ "(");
		outputln("org.w3c.dom.Element parent, org.w3c.dom.Element element) {");
		outputln(
			"if (element != null && element.getNodeName().trim().endsWith(\""
				+ JMIProvider.jmiClassQualifiedName(class_)
				+ "."
				+ JMIProvider.jmiFormat1(attribute_name_)
				+ "\")) {");
		outputln("org.w3c.dom.NodeList _child = element.getChildNodes();");
		outputln("int i = 0;");
		outputln("while (i<_child.getLength() || (_child.getLength()==0 && i==0)) {");
		outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child.item(i);");
		outputln("java.lang.String _current_name = _current.getNodeName().trim();");
		for (int i = 0; i < subclass_.length; i++) {
			if (i > 0)
				output("else ");
			String subClassName = JMIProvider.jmiClassName(subclass_[i]);
			outputln("if (_current_name.endsWith(\"" + JMIProvider.jmiFormat1(subClassName) + "\"))");
			outputln("   _xmi_import_classes." + XMIIOHelper.format1FirstMin(subClassName) + "Template(_current);");
		}
		outputln("i++;");
		outputln("}");
		outputln("}");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the "put" methods.
	 * @param package_name_ The package name.
	 * @param associations_ The associations of the package.
	 */
	public void putAssociationsValues(String package_name_, Vector associations_) {
		for (int i = 0; i < associations_.size(); i++) {
			javax.jmi.model.Association asso = (javax.jmi.model.Association) associations_.elementAt(i);
			try {
				javax.jmi.model.Reference[] refs = XMIIOHelper.getReferences(asso);
				for (int j = 0; j < refs.length; j++) {
					javax.jmi.model.AssociationEnd assoEnd = refs[j].getReferencedEnd();
					if (assoEnd.getAggregation() == javax.jmi.model.AggregationKindEnum.forName("none") && assoEnd.isChangeable()) {
                    //if (assoEnd.isChangeable()) {
						String hashtable =
							"_"
								+ JMIProvider.jmiFormat2(JMIProvider.jmiAssociationName(asso))
								+ "_"
								+ JMIProvider.jmiFormat2(refs[j].getName());
						String containerName = JMIProvider.findSubstituteName(refs[j].getContainer());
						if (containerName == null)
							containerName = JMIProvider.jmiFormat1(refs[j].getContainer().getName());
						String type = JMIProvider.jmiClassQualifiedName((MofClass) refs[j].getContainer());
						String typeName = refs[j].getContainer().getName();
						String typeClass = type;
						String ref = refs[j].getName();
						String typeRef = JMIProvider.jmiClassifierQualifiedName(refs[j].getReferencedEnd().getType());
						String vector = JMIProvider.jmiFormat2(ref) + "_vector";
						String type2 = JMIProvider.jmiFormat2(type);
						String ref1 = JMIProvider.jmiFormat1(ref);
						String ref2 = JMIProvider.jmiFormat2(ref);
						annotationTemplate(asso.getAnnotation());
						outputln("public void put" + JMIProvider.jmiFormat1(typeName) + JMIProvider.jmiFormat1(ref) + "() {");
						outputln("java.util.Enumeration _keys = " + hashtable + ".keys();");
						outputln("while (_keys.hasMoreElements()) {");
						outputln(typeClass + " _" + type2 + " = (" + typeClass + ") _keys.nextElement();");
						outputln("java.util.Vector _" + vector + " = (java.util.Vector) " + hashtable + ".get(_" + type2 + ");");
						outputln("for (int i=0 ; i<_" + vector + ".size() ; i++) {");
						outputln("java.lang.String _key = ((java.lang.String) _" + vector + ".elementAt(i)).trim();");
						outputln(typeRef + " _value = (" + typeRef + ") _types.get(_key);");
						if (refs[j].getMultiplicity().getUpper() == 1) {
							if (refs[j].getMultiplicity().getLower() == 0) {
								outputln("try {");
								outputln("if (_value != null && !_value.equals(_" + type2 + ".get" + ref1 + "()))");
								outputln("    _" + type2 + ".set" + ref1 + "(_value);");
								outputln("} catch (NullPointerException notSet) {");
								outputln("_" + type2 + ".set" + ref1 + "(_value);");
								outputln("}");
							} else {
								outputln(
									"if (_"
										+ type2
										+ ".get"
										+ ref1
										+ "() == null || (_value != null && !_value.equals(_"
										+ type2
										+ ".get"
										+ ref1
										+ "()))) {");
								outputln("_" + type2 + ".set" + ref1 + "(_value);");
								outputln("}");
							}
						} else {
							outputln("if (_value != null && !(_" + type2 + ".get" + ref1 + "().contains(_value))) {");
							outputln("_" + type2 + ".get" + ref1 + "().add(_value);");
							outputln("}");
						}
						if (refs.length > 1
							&& refs[(j + 1) % 2].getReferencedEnd().getAggregation() == javax.jmi.model.AggregationKindEnum.forName("none")) {
							// There is j=0 or j=1, so, to switch between the values, we can use: (j+1)%2
							if (refs[(j + 1) % 2].getMultiplicity().getUpper() == 1)
								outputln(
									"_value.set"
										+ JMIProvider.jmiFormat1(refs[(j + 1) % 2].getName())
										+ "(_"
										+ JMIProvider.jmiFormat2(type)
										+ ");");
							else
								outputln(
									"_value.get"
										+ JMIProvider.jmiFormat1(refs[(j + 1) % 2].getName())
										+ "().add(_"
										+ JMIProvider.jmiFormat2(type)
										+ ");");
						}
						outputln("}");
						outputln("}");
						outputln("}");
						outputln();
					}
				}
			} catch (NullPointerException notSet) {
				// Nothing to do
			}
		}
	}

}
