/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.mofobject;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * This class represents a list of objects and a list of xmi.idref corresponding to another objects.
 * For example, it will be use for the return type of XXXTemplate methods to get the parsed objects
 * and the xmi.idref of the referenced objects. 
 */
public class MOFObjectList {

	/** List of objects. */
	private List objectList;

	/** List of XMI Id Ref. */
	private List xmiIdRefList;

	/**
	 * Default Constructor of a MOFObjectList.
	 */
	public MOFObjectList() {
		objectList = new ArrayList();
		xmiIdRefList = new ArrayList();
	}

	/**
	 * Add an object to the corresponding list.
	 * @param obj The object to add.
	 */
	public void addObject(Object obj) {
		objectList.add(obj);
	}

	/**
	 * Get an interator from the list of objects.
	 * @return The iterator.
	 */
	public Iterator getObjects() {
		return objectList.iterator();
	}

	/**
	 * Add an XMI Id Ref to the corresponding list.
	 * @param xmiid The XMI Id to add.
	 */
	public void addXmiIdRef(String xmiid) {
		xmiIdRefList.add(xmiid);
	}

	/**
	 * Get an interator from the list of XMI Id Ref.
	 * @return The iterator.
	 */
	public Iterator getXmiIdRef() {
		return xmiIdRefList.iterator();
	}

	/**
	 * Add a MOFObjectList to this list.
	 * @param list The MOFObjectList to concatenate.
	 */
	public void addMOFObjectList(MOFObjectList list) {
		objectList.addAll(list.objectList);
		xmiIdRefList.addAll(list.xmiIdRefList);
	}

	/**
	 * Get the first object of the list. This object is a BuiltObject, a XMIIdRef or a NullObject.
	 * @return The first object.
	 */
	public MOFObject getFirstObject() {
		if (objectList.size() > 0)
			return new BuiltObject(objectList.get(0));
		if (xmiIdRefList.size() > 0)
			return new XMIIdRef((String) xmiIdRefList.get(0));
		return new NullObject();
	}

}
