/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.qvt.engine;

import javax.jmi.reflect.*;

import java.util.*;

public class EntryPointApplyer {
    
  RefPackage src, target, rulesPkg;
  Hashtable bindingTable = new Hashtable();
  RefObject rulesUnit;

  public EntryPointApplyer(RefPackage rulesPkg, RefPackage src, RefPackage target) throws Exception {    
    this.src = src;
    this.target = target;
	this.rulesPkg = rulesPkg;
    init();
  }
  
  public void applyAllUnits() throws Exception {
	Iterator it = rulesPkg.refClass("RulesUnit").refAllOfClass().iterator();
	while(it.hasNext()) {   
		RefObject rulesUnit = (RefObject) it.next();
		applyEntryPoints(rulesUnit);
	}   
  }
  
  public void applyEntryPoints(RefObject rulesUnit) throws Exception {
    
    RefObject[] rules;    
    rules = (RefObject[]) ( (Collection) rulesUnit.refGetValue("rules")
    ).toArray(new RefObject[0]);

    for(int i=0; i<rules.length; i++) {
      if( Reflection.isOfType(rules[i],"EntryPointRule") ) {
          applyEntryPoint( rules[i]);
      }
    }
  }

  void applyEntryPoint(RefObject entryPointRule) throws Exception {
      // resolve entry elements
      Object[] contexts = Reflection.allOfType(
        src ,(String)entryPointRule.refGetValue("context")
      ).toArray();
      // apply rule to resolved elements
      for(int i=0; i<contexts.length ; i++) {
        Hashtable args = new Hashtable();
        args.put("context", contexts[i]);
        (new RuleBinding( bindingTable 
            ,entryPointRule, src, target, args, 0)).execute();
      }
  }
  
  void init() {
    String verbose = System.getProperty("fr.lip6.modfact.qvt.engine.verbose");
    if("true".equals(verbose)) {
      RuleBinding.verbose = true;
    }  
  }
}

