package org.objectweb.modfact.qvt.engine;

import javax.jmi.reflect.*;

import java.util.*;

public class Reflection {

  static void assignProperty(RefObject owner, String propName, Object val) throws Exception {
    if(val==null) return;
    try {
      Object currentValue = owner.refGetValue(propName);  
      if( currentValue instanceof Collection ) {
        Collection prop = (Collection) currentValue;
        prop.clear();
        if(val instanceof Collection) {
          prop.addAll((Collection) val);
        } else {
          prop.add(val);
        }    
      } else {
        owner.refSetValue(propName, val);
      }
    } catch(Exception e)  {
      String msg = "assign prop: "
        +getTypeName(owner) +"." +propName +" = " + printValue(val);
      throw new ReflectException(msg ,e);
    }
  }

  static Object getPropertyValue(RefObject owner, String propName) throws Exception {
    try {
      return owner.refGetValue(propName);
    } catch(Exception e) {
      String msg = "call prop: "
        +getTypeName(owner) +"." +propName;
      throw new ReflectException(msg, e);
    }
  }

  static Object invokeOperation(
                String opName
              , Object context
              , List args) throws Exception {
    try {
        java.lang.reflect.Method[] ops = context.getClass().getMethods();
        java.lang.reflect.Method m = null;
        for(int i=0; i<ops.length; i++) {
          if(ops[i].getName().equals(opName)) {
            m = ops[i];  
            break;
          }
        }        
        Object r = m.invoke(context, args.toArray());
        return r;
    } catch(Exception e) {
      String msg = "invoke " +opName +" with "
        +getTypeName(context)
        +" " +printValue(args);
      throw new ReflectException(msg ,e);
    }
  }

  static Object createObject(RefPackage pkg, String conceptName) throws Exception {
    try {
      return pkg.refClass(conceptName).refCreateInstance(null);
    } catch(Exception e) {
      String msg = "create " +conceptName +" in " + pkg;
      throw new ReflectException(msg ,e);
    }
  }

  static Collection allOfType(RefPackage pkg, String conceptName)
      throws Exception {
    try {
      return pkg.refClass(conceptName).refAllOfType();
    } catch(Exception e) {
      String msg = "allOfType " +conceptName +" in " +pkg;
      throw new ReflectException(msg ,e);
    }
  }

  static Object getEnumValue(RefPackage pkg, String type, String val)
    throws Exception {
    try {
      return pkg.refGetEnum(type ,val);
    } catch(Exception e) {
      String msg = "create enum: " +type +" = " +val +" in " +getTypeName(pkg);
      throw new ReflectException(msg ,e);
    }
  }

  static boolean isOfType(RefObject o, String type) throws Exception {
    if(!(o instanceof RefObject)) return false;
    RefObject refo = ((RefObject) o);
    try {
        RefObject c = o.refImmediatePackage().refClass(type).refMetaObject();
        return o.refIsInstanceOf(c ,true);
    } catch(Exception e) {
      String msg =  getTypeName(o) +" isOfType " +type;
      throw new ReflectException(msg ,e);
    }
  }

  static String getTypeName(Object o) {
    if(!(o instanceof RefBaseObject)) return o.getClass().getName();
    return (String) ((RefBaseObject)o).refMetaObject().refGetValue("name");
  }

  static String printValue(Object val) {
    if(val==null) return "null";
    if(val instanceof Collection) {
      Object[] array = ((Collection)val).toArray();
	  String r ="{";
      for(int i=0; i<array.length; i++) {
        r = r + " " +printValue(array[i]);
      }
      return r + "}";
    }
    if(val instanceof RefBaseObject) {
      return getTypeName((RefBaseObject)val);
    }
    return val.toString();
  }
}




