package org.objectweb.modfact.qvt.engine;

import javax.jmi.reflect.*;
import org.objectweb.modfact.qvt.util.RulePrinter;
import java.util.*;

class RuleBinding {
    
  static boolean verbose = false;

  Hashtable bindingTable;  
  
  RefObject rule;
  Hashtable args;
  RefPackage src;
  RefPackage target;

  int indent;

  Object returnVal = null;
  ExpEvaluator evaluator;
  String key;
    
	RuleBinding(
            Hashtable bindingTable 
            ,RefObject rule, RefPackage src, RefPackage target
            ,Hashtable args, int indent) {
          
          this.bindingTable = bindingTable;
          this.rule = rule;
          this.args = args;
          this.src = src;
          this.target = target;
          this.indent = indent;
	}
        
        // remember applyed context ---> idempotent semantic
        boolean  isApplied() {
          key = key + "rule=" + rule.refGetValue("name") 
                +",context=" +getId(args.get("context"));
          if(rule.refMetaObject().refGetValue("name").equals("CalledRule")) {
             Iterator it = ((List)rule.refGetValue("params")).iterator();
             while(it.hasNext()) {
               String argName = (String)((RefObject)it.next()).refGetValue("name");
               Object arg = args.get(argName);
               key = key +"," + argName +"="+ getId(arg);
             }
          }
          if(!bindingTable.containsKey(key)) {
			bindingTable.put(key,this); 
            return false;
          }
          return true;
        }
        
        String getId(Object o) {
          if(o==null) return "null";
          if(o instanceof RefObject) 
            return ((RefObject)o).refMofId();
          return "" +o;
        }

	Object execute()  throws Exception {
            if(verbose) System.err.println( RulePrinter.printer.doIndent(indent) +"rule binding: " + rule.refGetValue("name"));
            
            if(isApplied()) {
              if(verbose) System.err.println( RulePrinter.printer.doIndent(indent+1) +"already applied to this context ->skip");
              RuleBinding b = (RuleBinding)bindingTable.get(key);
              return b.returnVal;
            }
            
            Object result = createResult(rule);
	    
            RefObject[] statements = (RefObject[]) ((Collection)rule.refGetValue("statements")).toArray(new RefObject[0]);
	    
            evaluator = new ExpEvaluator(this);
            
            for(int i = 0; i<statements.length; i++) {
               if(verbose) System.err.println( RulePrinter.printer.doIndent(indent+1)
                  // +"[" +rule.refGetValue("name") +"] "
                  +(i+1) + ": " +RulePrinter.printer.printShort(statements[i]) );
              try {
				evaluator.eval(statements[i]);
              } catch(Exception e) {
                 if(verbose) System.err.println( RulePrinter.printer.doIndent(indent)
                   +"ERROR at " +rule.refGetValue("name") +" "  +(i+1)
                   +": " + RulePrinter.printer.printShort(statements[i]) );
                throw e;
              }
	    }
            RefObject returnValExp = (Reflection.isOfType(rule,"CalledRule"))? (RefObject)rule.refGetValue("returnVal") : null;

            if(returnValExp!=null) {
              try {
                  if(verbose) System.err.println( RulePrinter.printer.doIndent(indent+1)
                    //+"[" +rule.refGetValue("name") +"] "
                    +"return: " + RulePrinter.printer.printShort(returnValExp) );
                  returnVal = evaluator.eval(returnValExp);
              } catch(Exception e) {
                if(verbose) System.err.println( RulePrinter.printer.doIndent(indent)
                   +"ERROR at " +rule.refGetValue("name") 
                   +": could not evaluate returned value" );
                throw e;
              }
            }
            
            if(verbose) System.err.println( RulePrinter.printer.doIndent(indent)
              +"end rule: " +rule.refGetValue("name")
            );            
            return returnVal;
	}


	// return result;
	Object createResult(RefObject rule) throws Exception {
	  String resultString = (String)rule.refGetValue("resultType");
	  if(resultString==null || resultString.equals("")) {
		  return null;
	  }
	  Object result = Reflection.createObject(
                    target
		  , resultString
	  );
      args.put("result",result);           
	  return result;
	}

}
