/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.qvt.syntaxe;

/**
 * 
 * @author Reda Bendraou
 */

import java.io.*;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jmi.reflect.RefObject;
import javax.jmi.xmi.XmiWriter;

import org.objectweb.modfact.jmi.repository.simpletrl.SimpleTRLPackageImpl;
import org.objectweb.modfact.jmi.xmi.XmiWriterImpl;
import org.objectweb.modfact.qvt.util.RulePrinter;

import simpletrl.SimpleTRLPackage;

class TranslationService {
	public static void main(String[] args) {

		FileInputStream fileInput = null;

		try {
			fileInput = new FileInputStream(args[0]);
			DataInputStream input = new DataInputStream(fileInput);
			STRLLexer lexer = new STRLLexer(input);
			STRLParser parser = new STRLParser(lexer);
			
			SimpleTRLPackage p =
				SimpleTRLPackageImpl.create(
					ClassLoader
						.getSystemResource("simpletrl/metamodel_SimpleTRL.xml")
						.toString());
			
			
			//Set a Logger
			Logger log = Logger.getLogger("org.objectweb.modfact.qvt.syntaxe.TranslationService");
			parser.setLogger(log);
			
			ConsoleHandler chJmi = new ConsoleHandler();
			chJmi.setLevel(Level.INFO);
			log.addHandler(chJmi);

			parser.setRuleModel(p);
			parser.file();

			/*
			 * CommonAST parseTree = (CommonAST)parser.getAST(); ASTFrame frame =
			 * new ASTFrame("The Tree", parseTree);
			 */

			XmiWriter w = new XmiWriterImpl();
			FileOutputStream out = new FileOutputStream(args[1]);
			w.write(out, p, "1.2");
			out.close();

			out = new FileOutputStream(args[2]);
			RulePrinter.printer.printRules(
				(RefObject) p.getRulesUnit().refAllOfClass().toArray()[0],
				new PrintWriter(out));
			out.close();

		} catch (Exception e) {
			System.err.println("exception REDA:  " + e);
			e.printStackTrace();
		}
	}
}
