package org.objectweb.modfact.qvt.util;

import java.io.*;
import java.util.*;

import javax.jmi.reflect.*;
import javax.jmi.xmi.XmiReader;

import org.objectweb.modfact.jmi.reflect.RefPackageImpl;
import org.objectweb.modfact.jmi.xmi.XmiReaderImpl;

public class RulePrinter {
	
  public static void main(String[] args) throws Exception {
    String xmiURL = args[0];
    String outFile = args[1];
    
    RefPackage p = RefPackageImpl.getM1Repository(
	  RefPackageImpl.createMetaObject(
	  	ClassLoader.getSystemResource(
			"org/objectweb/modfact/qvt/metamodel_SimpleTRL.xml"		 
		 ).toString()
	   )
    );
    XmiReader r = new XmiReaderImpl();
    r.read(xmiURL, p);
    printer.printRules(p, outFile);  
  }	
	
	

  public static final RulePrinter printer = new RulePrinter(0);
  
  int indent;

  RulePrinter(int i) {
    indent = i;
  }
  
  public void printRules(RefPackage pkg, String outputFileName) throws Exception { 
	FileWriter writer = new FileWriter(outputFileName);
	Iterator it = pkg.refClass("RulesUnit").refAllOfClass().iterator(); 
	 while(it.hasNext()) {
		printRules((RefObject)it.next(), writer);
	 } 	
  }

  public void printRules(RefObject unit, Writer writer) throws Exception {
    writer.write("Unit " +unit.refGetValue("name") +"\n" );
	writer.write("source " +unit.refGetValue("srcMm") +"\n" );
	writer.write("target " +unit.refGetValue("targetMm") +"\n" );
    RefObject[] rules = (RefObject[]) 
            ( (Collection)unit.refGetValue("rules") ).toArray(new RefObject[0]);
    for(int i=0; i<rules.length; i++) {
      writer.write("\n------------------------\n");
      writer.write(printRule(rules[i]));
    }
    writer.close();
  }

  public String printRule(RefObject rule) {
    try {
      RulePrinter p = new RulePrinter(indent+1);
      String end = "{"+ p.printStatements( (Collection)rule.refGetValue("statements") )
                  +"\n" +"}";
      if( isOfType(rule, "EntryPointRule") ) {
          return entryPointRule(rule) +end;

      } else if( isOfType(rule, "CalledRule") ) {
          return calledRule(rule) +end;
      }
      return null;
    } catch(RuntimeException e) {
      System.err.println("RulePrinter: fail at rule " + rule.refGetValue("name"));
      throw e;
    }
  }

  public String entryPointRule(RefObject rule) {
    return  "EntryPointRule " + rule.refGetValue("name")
            +( (rule.refGetValue("resultType").equals(""))? ""
                      : ("\ncreates " +rule.refGetValue("resultType") )
            );
  }

  public String calledRule(RefObject rule) {
    String resultType = (String) rule.refGetValue("resultType");
    return "Rule " + rule.refGetValue("name") + printArgs((Collection)rule.refGetValue("params"))
            +( (resultType!=null && resultType.length()>0 )? 
            	"\ncreates " +rule.refGetValue("resultType") : ""
            )
            +( (rule.refGetValue("returnVal")==null)? ""
                      : ("\nreturn " +print((RefObject)rule.refGetValue("returnVal")) )
            );

  }

  public String printShort(RefObject exp) {
    if( isOfType(exp ,"IfExp") ) {
      return "(IfExp) " +print((RefObject)exp.refGetValue("cond") );
    }
    return print(exp);
  }

  public String print(RefObject exp) {
    if(exp==null) return "";
    try {
      if( isOfType(exp ,"ParameterExp") ) {
        return parameterExp( exp );

      } else if( isOfType(exp ,"IfExp") ) {
        return ifExp( exp );

      } else if( isOfType(exp ,"ModelReferenceExp") ) {
        return modelReferenceExp( exp );

      } else if( isOfType(exp ,"LiteralExp") ) {
        return literalExp( exp );

      } else if(isOfType(exp ,"ContextExp") ) {
        return contextExp( exp);

      }
      throw new RuntimeException("unknown exp: " +getTypeName(exp) );
    } catch(RuntimeException e) {
      System.err.println("Cannot print " + exp.refMetaObject().refGetValue("name") );
      throw e;
    }
  }

  public String contextExp(RefObject exp) {
    String begin = print((RefObject)exp.refGetValue("context")) + ".";

    if( isOfType(exp ,"AbstractOperationCallExp") ) {
      return begin +abstractOperationCallExp(exp);

    } else if( isOfType(exp ,"PropertyCallExp") ) {
      return begin +propertyCallExp(exp);

    } else if( isOfType(exp ,"PropertyAssignExp") ) {
      return begin +propertyAssignExp(exp);

    } else if( isOfType(exp ,"CollectionExp") ) {
      return begin +collectionExp(exp);

    } else if( isOfType(exp ,"IsOfTypeExp") ) {
        return begin +isOfTypeExp(exp);

    } else if( isOfType(exp ,"CompareExp") ) {
        return begin +compareExp(exp);
    }
    return null;
  }

  public String abstractOperationCallExp(RefObject exp) {
    String end = printArgs((Collection)exp.refGetValue("args"));
    if( isOfType(exp ,"RuleCallExp") ) {
      return ruleCallExp(exp) +end;

    } else {
      return operationCallExp(exp) +end;
    }
  }

  String printArgs(Collection c) {
    RefObject[] args = (RefObject[]) c.toArray(new RefObject[0]);
    String r = "(";
    for(int i=0; i<args.length; i++) {
      r = r + ((i==0)? "" : " ,") + print(args[i]);
    }
    return r + ")";
  }

  public String ruleCallExp(RefObject exp) {
    return (String)((RefObject)exp.refGetValue("rule")).refGetValue("name");
  }

  public String operationCallExp(RefObject exp) {
    return (String) exp.refGetValue("opName");
  }

  public String parameterExp(RefObject exp) {
    return "#" + exp.refGetValue("name");
  }

  public String propertyAssignExp(RefObject exp) {
    return exp.refGetValue("propName") +"=" + print((RefObject)exp.refGetValue("value"));
  }

  public String collectionExp(RefObject exp) {
    String op = ( isOfType(exp ,"AddCollectionExp") )? "add" : "remove";
    return op + "(" + print((RefObject)exp.refGetValue("member")) + ")";
  }

  public String ifExp(RefObject exp) {
    if( isOfType(exp ,"IfStatementExp") ) {
      return ifStatementExp(exp);

    } else if( isOfType(exp ,"IfValueExp") ) {
      return ifValueExp(exp);
    }
    return null;
  }

  public String ifStatementExp(RefObject exp) {
    RulePrinter p = new RulePrinter(indent+1);
    return "if(" +print((RefObject)exp.refGetValue("cond")) +") {"
                +p.printStatements((Collection)exp.refGetValue("thenStatements"))
                +"\n" +doIndent() +"} else {"
                +p.printStatements((Collection)exp.refGetValue("elseStatements"))
                +"\n" +doIndent() +"}";
  }

  public String printStatements(Collection c) {
    RefObject[] statements = (RefObject[]) c.toArray(new RefObject[0]);
    String r = "";
    for(int i=0; i<statements.length; i++) {
      try {
        r = r + "\n" + doIndent() + print(statements[i]);
      } catch(RuntimeException e) {
        System.err.println("RulePrinter.printStatements: fail at "+ (i+1));
        throw e;
      }
    }
    return r;
  }

  public String ifValueExp(RefObject exp) {
    return "(" +print((RefObject)exp.refGetValue("cond")) +")? "
            +print((RefObject)exp.refGetValue("thenExp")) +" : " +print((RefObject)exp.refGetValue("elseExp"));
  }

  public String modelReferenceExp(RefObject exp) {
    return "allOfType(" + exp.refGetValue("conceptName") +")";
  }


  public String propertyCallExp(RefObject exp) {
    return (String)exp.refGetValue("propName");
  }

  public String compareExp(RefObject exp) {
    return "equals(" + print((RefObject)exp.refGetValue("compareVal")) +")";
  }

  public String isOfTypeExp(RefObject exp) {
    return "isOfType(" +exp.refGetValue("type") +")";
  }

  public String literalExp( RefObject exp) {
	  if( isOfType(exp ,"DoubleLiteral")  ) {
		  return "" + exp.refGetValue("val");

	  } else if( isOfType(exp ,"IntegerLiteral") ) {
		  return "" + exp.refGetValue("val");

	  } else if( isOfType(exp ,"BooleanLiteral") ) {
		  return "" + exp.refGetValue("val");

	  } else if( isOfType(exp ,"StringLiteral") ) {
		  return "" + exp.refGetValue("val");

	  } else if( isOfType(exp ,"EnumLiteral") ) {
                  return "enum("+exp.refGetValue("val") +": "
                  + srcOrTarget((Boolean) exp.refGetValue("isFromMmSrc")) +", "
                  + exp.refGetValue("type") + ")";
          }
	  return null;
  }
  
  String srcOrTarget(Boolean isFromMmSrc) {
    return (isFromMmSrc.equals(Boolean.TRUE))? "src" :"target"; 
  }

  public String getTypeName(RefObject o) {
    return (String) o.refMetaObject().refGetValue("name");
  }
  
  boolean isOfType(RefObject o, String type) {
     RefObject c = o.refOutermostPackage().refClass(type).refMetaObject();
     return o.refIsInstanceOf(c ,true);
  }

  public String doIndent(int indent) {
          String r = "";
          for(int i=0; i<indent; i++) {
            r = r + "  ";
          }
          return r;
  }

  private String doIndent() {
    return doIndent(indent);
  }

}